@@ -233,10 +233,10 @@ calculateGridLayout <-
 
     if (rows.per.page > 1)
         heights.x[pos.heights[["between"]]] <-
-            x$y.between * heights.x[pos.heights[["between"]]]
+            rep(x$y.between, length = rows.per.page - 1) * heights.x[pos.heights[["between"]]]
     if (cols.per.page > 1)
         widths.x[pos.widths[["between"]]] <-
-            x$x.between * widths.x[pos.widths[["between"]]]
+            rep(x$x.between, length = cols.per.page - 1) * widths.x[pos.widths[["between"]]]
 
 
     if (!is.null(panel.height))
@@ -316,7 +316,6 @@ calculateGridLayout <-
             else par.strip.text$cex * par.strip.text$lines * number.of.cond
 
 
-
     ## All the upcoming scary code about labels is only to determine
     ## how much space to leave for them. Much of these calculations
     ## will be repeated later before actually drawing them. 
