@@ -14,14 +14,16 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include <stdlib.h> /* for abs */
+#include <math.h>
 #include <Rmath.h> /* for imax2(.),..*/
 #include <R_ext/Applic.h>
 
@@ -732,6 +734,7 @@ static int maxp;
  *	m_fac	contains the number of factors and 
  *	kt	contains the number of square factors  */
 
+/* non-API, but used by package RandomFields */
 void fft_factor(int n, int *pmaxf, int *pmaxp)
 {
 /* fft_factor - factorization check and determination of memory
