@@ -23,80 +23,71 @@
    See the file COPYLIB.TXT for details.
 */
 
-/* Copyright (C) 2004--2006 	The R Foundation
+/* Copyright (C) 2004--2008	The R Foundation
 
    Additions for R, Chris Jackson
    Find and replace dialog boxes and dialog handlers */
 
+#define _WIN32_WINNT 0x0500
 #include "win-nls.h"
 #include "internal.h"
 #include "ga.h"
 
 #include <shlobj.h>
 
+typedef struct {
+    char default_str[MAX_PATH];
+    char question[40];
+} browserInfo;
+
+#define STATUSTEXT 14146
+
 static int CALLBACK
-InitBrowseCallbackProc( HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData )
+InitBrowseCallbackProc( HWND hwnd, UINT uMsg, LPARAM lp, LPARAM lpData )
 {
-    if (uMsg == BFFM_INITIALIZED)
-	SendMessage(hwnd, BFFM_SETSELECTION, 1, lpData);
+    char szDir[MAX_PATH], status[MAX_PATH + 40];
+
+    if (uMsg == BFFM_INITIALIZED) {
+	SendMessage(hwnd, BFFM_SETSELECTION, 1,
+		    (LPARAM)&((browserInfo*)lpData)->default_str);
+    } else if (uMsg == BFFM_SELCHANGED) {
+	if (SHGetPathFromIDList((LPITEMIDLIST) lp ,szDir)) {
+	    snprintf(status, MAX_PATH+40, "%s\n %s",
+		     ((browserInfo*)lpData)->question, szDir);
+	    SetDlgItemText(hwnd, STATUSTEXT, status);
+	    SendMessage(hwnd, BFFM_ENABLEOK, 0, TRUE);
+	} else
+	    SendMessage(hwnd, BFFM_ENABLEOK, 0, FALSE);
+    }
     return(0);
 }
 
-/* browse for a folder under the Desktop, return the path in the argument */
-
-static void selectfolder(char *folder, const char *title)
-{
-    char buf[MAX_PATH];
-    LPMALLOC g_pMalloc;
-    HWND hwnd=0;
-    BROWSEINFO bi;
-    LPITEMIDLIST pidlBrowse;
-
-    /* Get the shell's allocator. */
-    if (!SUCCEEDED(SHGetMalloc(&g_pMalloc))) return;
-
-    bi.hwndOwner = hwnd;
-    bi.pidlRoot = NULL;
-    bi.pszDisplayName = buf;
-    bi.lpszTitle = title;
-    bi.ulFlags = BIF_RETURNONLYFSDIRS;
-    bi.lpfn = (BFFCALLBACK) InitBrowseCallbackProc;
-    bi.lParam = (LPARAM) folder;
-
-    /* Browse for a folder and return its PIDL. */
-    pidlBrowse = SHBrowseForFolder(&bi);
-    if (pidlBrowse != NULL) {
-	SHGetPathFromIDList(pidlBrowse, folder);
-        g_pMalloc->lpVtbl->Free(g_pMalloc, pidlBrowse);
-    }
-}
-
-
 #define BUFSIZE _MAX_PATH
 static char strbuf[BUFSIZE];
 
 static const char *filter[] = {
-	"All Files (*.*)",	"*.*",
-	"Text Files (*.TXT)",	"*.txt",
-	"HTML Files (*.HTM)",	"*.htm",
-	"PNG Files (*.PNG)",	"*.png",
-	"JPEG Files (*.JPG)",	"*.jpg",
-	"BMP Files (*.BMP)",	"*.bmp",
-	""
+    "All Files (*.*)",	"*.*",
+    "Text Files (*.TXT)",	"*.txt",
+    "HTML Files (*.HTM)",	"*.htm",
+    "PNG Files (*.PNG)",	"*.png",
+    "JPEG Files (*.JPG)",	"*.jpg",
+    "BMP Files (*.BMP)",	"*.bmp",
+    ""
 };
 
 unsigned int TopmostDialogs = 0; /* May be MB_TOPMOST */
 
 static const char *userfilter;
-void setuserfilter(const char *uf) {
-   userfilter=uf;
+void setuserfilter(const char *uf)
+{
+    userfilter=uf;
 }
 
 static HWND hModelessDlg = NULL;
 
 int myMessageBox(HWND h, const char *text, const char *caption, UINT type)
 {
-    if(is_NT && (localeCP != GetACP())) {
+    if(localeCP != GetACP()) {
 	wchar_t wc[1000], wcaption[100];
 	mbstowcs(wcaption, caption, 100);
 	mbstowcs(wc, text, 1000);
@@ -110,71 +101,71 @@ int myMessageBox(HWND h, const char *text, const char *caption, UINT type)
  */
 void apperror(const char *errstr)
 {
-	if (! errstr)
-		errstr = "Unspecified error";
-	myMessageBox(0, errstr, "Graphics Library Error",
-		MB_TASKMODAL | MB_ICONSTOP | MB_OK | TopmostDialogs);
-	exitapp();
+    if (! errstr)
+	errstr = "Unspecified error";
+    myMessageBox(0, errstr, "Graphics Library Error",
+		 MB_TASKMODAL | MB_ICONSTOP | MB_OK | TopmostDialogs);
+    exitapp();
 }
 
 void askok(const char *info)
 {
-	if (! info)
-		info = "";
-	myMessageBox(0, info, "Information",
-		MB_TASKMODAL | MB_ICONINFORMATION | MB_OK | TopmostDialogs);
+    if (! info)
+	info = "";
+    myMessageBox(0, info, "Information",
+		 MB_TASKMODAL | MB_ICONINFORMATION | MB_OK | TopmostDialogs);
 }
 
 int askokcancel(const char *question)
 {
-	int result;
+    int result;
 
-	if (! question)
-		question = "";
-	result = myMessageBox(0, question, G_("Question"),
-		MB_TASKMODAL | MB_ICONQUESTION | MB_OKCANCEL | TopmostDialogs);
+    if (! question)
+	question = "";
+    result = myMessageBox(0, question, G_("Question"),
+			  MB_TASKMODAL | MB_ICONQUESTION | MB_OKCANCEL | TopmostDialogs);
 
-	switch (result) {
-		case IDOK: result = YES; break;
-		case IDCANCEL:
-		default: result = CANCEL; break;
-	}
-	return result;
+    switch (result) {
+    case IDOK: result = YES; break;
+    case IDCANCEL:
+    default: result = CANCEL; break;
+    }
+    return result;
 }
 
 int askyesno(const char *question)
 {
-	int result;
+    int result;
 
-	if (! question)
-		question = "";
-	result = myMessageBox(0, question, G_("Question"),
-		MB_TASKMODAL | MB_ICONQUESTION | MB_YESNO | TopmostDialogs);
+    if (! question)
+	question = "";
+    result = myMessageBox(0, question, G_("Question"),
+			  MB_TASKMODAL | MB_ICONQUESTION | MB_YESNO | TopmostDialogs);
 
-	switch (result) {
-		case IDYES: result = YES; break;
-		case IDNO:  result = NO; break;
-		default: result = CANCEL; break;
-	}
-	return result;
+    switch (result) {
+    case IDYES: result = YES; break;
+    case IDNO:  result = NO; break;
+    default: result = CANCEL; break;
+    }
+    return result;
 }
 
 int askyesnocancel(const char *question)
 {
-	int result;
+    int result;
 
-	if (! question)
-		question = "";
-	result = myMessageBox(0, question, G_("Question"),
-		MB_TASKMODAL | MB_ICONQUESTION | MB_YESNOCANCEL | MB_SETFOREGROUND | TopmostDialogs);
+    if (! question)
+	question = "";
+    result = myMessageBox(0, question, G_("Question"),
+			  MB_TASKMODAL | MB_ICONQUESTION | MB_YESNOCANCEL | MB_SETFOREGROUND | TopmostDialogs);
 
-	switch (result) {
-		case IDYES: result = YES; break;
-		case IDNO:  result = NO; break;
-		case IDCANCEL:
-		default: result = CANCEL; break;
-	}
-	return result;
+    switch (result) {
+    case IDYES: result = YES; break;
+    case IDNO:  result = NO; break;
+    case IDCANCEL:
+    default: result = CANCEL; break;
+    }
+    return result;
 }
 
 static char cod[MAX_PATH]=""; /*current open directory*/
@@ -198,16 +189,16 @@ void askchangedir()
 
 char *askfilename(const char *title, const char *default_name)
 {
-	if (*askfilenames(title, default_name, 0, userfilter?userfilter:filter[0], 0,
-				          strbuf, BUFSIZE, NULL)) return strbuf;
-	else return NULL;
+    if (*askfilenames(title, default_name, 0, userfilter?userfilter:filter[0], 0,
+		      strbuf, BUFSIZE, NULL)) return strbuf;
+    else return NULL;
 }
 
 char *askfilenamewithdir(const char *title, const char *default_name, const char *dir)
 {
-	if (*askfilenames(title, default_name, 0, userfilter?userfilter:filter[0], 0,
-				          strbuf, BUFSIZE, dir)) return strbuf;
-	else return NULL;
+    if (*askfilenames(title, default_name, 0, userfilter?userfilter:filter[0], 0,
+		      strbuf, BUFSIZE, dir)) return strbuf;
+    else return NULL;
 }
 
 char *askfilenames(const char *title, const char *default_name, int multi,
@@ -215,61 +206,61 @@ char *askfilenames(const char *title, const char *default_name, int multi,
 		   char *strbuf, int bufsize,
 		   const char *dir)
 {
-	int i;
-	OPENFILENAME ofn;
-        char cwd[MAX_PATH] = "";
+    int i;
+    OPENFILENAME ofn;
+    char cwd[MAX_PATH] = "";
 
-	if (!default_name) default_name = "";
-	strcpy(strbuf, default_name);
-        GetCurrentDirectory(MAX_PATH, cwd);
-        if (!strcmp(cod, "")) {
-            if (!dir) strcpy(cod, cwd);
-            else strcpy(cod, dir);
-        }
+    if (!default_name) default_name = "";
+    strcpy(strbuf, default_name);
+    GetCurrentDirectory(MAX_PATH, cwd);
+    if (!strcmp(cod, "")) {
+	if (!dir) strcpy(cod, cwd);
+	else strcpy(cod, dir);
+    }
 
-	ofn.lStructSize     = sizeof(OPENFILENAME);
-	ofn.hwndOwner       = current_window ? current_window->handle : 0;
-	ofn.hInstance       = 0;
-	ofn.lpstrFilter     = filters;
-	ofn.lpstrCustomFilter = NULL;
-	ofn.nMaxCustFilter  = 0;
-	ofn.nFilterIndex    = filterindex;
-	ofn.lpstrFile       = strbuf;
-	ofn.nMaxFile        = bufsize;
-	ofn.lpstrFileTitle  = NULL;
-	ofn.nMaxFileTitle   = _MAX_FNAME + _MAX_EXT;
-        ofn.lpstrInitialDir = cod;
-	ofn.lpstrTitle      = title;
-	ofn.Flags           = OFN_CREATEPROMPT | OFN_HIDEREADONLY | OFN_EXPLORER;
-	if (multi) ofn.Flags |= OFN_ALLOWMULTISELECT;
-	ofn.nFileOffset     = 0;
-	ofn.nFileExtension  = 0;
-	ofn.lpstrDefExt     = "*";
-	ofn.lCustData       = 0L;
-	ofn.lpfnHook        = NULL;
-	ofn.lpTemplateName  = NULL;
+    ofn.lStructSize     = sizeof(OPENFILENAME);
+    ofn.hwndOwner       = current_window ? current_window->handle : 0;
+    ofn.hInstance       = 0;
+    ofn.lpstrFilter     = filters;
+    ofn.lpstrCustomFilter = NULL;
+    ofn.nMaxCustFilter  = 0;
+    ofn.nFilterIndex    = filterindex;
+    ofn.lpstrFile       = strbuf;
+    ofn.nMaxFile        = bufsize;
+    ofn.lpstrFileTitle  = NULL;
+    ofn.nMaxFileTitle   = _MAX_FNAME + _MAX_EXT;
+    ofn.lpstrInitialDir = cod;
+    ofn.lpstrTitle      = title;
+    ofn.Flags           = OFN_CREATEPROMPT | OFN_HIDEREADONLY | OFN_EXPLORER;
+    if (multi) ofn.Flags |= OFN_ALLOWMULTISELECT;
+    ofn.nFileOffset     = 0;
+    ofn.nFileExtension  = 0;
+    ofn.lpstrDefExt     = "*";
+    ofn.lCustData       = 0L;
+    ofn.lpfnHook        = NULL;
+    ofn.lpTemplateName  = NULL;
 
-	if (GetOpenFileName(&ofn) == 0) {
-		GetCurrentDirectory(MAX_PATH, cod);
-		SetCurrentDirectory(cwd);
-		strbuf[0] = 0;
-		strbuf[1] = 0;
-		return strbuf;
-	} else {
-		GetCurrentDirectory(MAX_PATH, cod);
-		SetCurrentDirectory(cwd);
-		for (i = 0; i <  10; i++) if (peekevent()) doevent();
-		return strbuf;
-	}
+    if (GetOpenFileName(&ofn) == 0) {
+	GetCurrentDirectory(MAX_PATH, cod);
+	SetCurrentDirectory(cwd);
+	strbuf[0] = 0;
+	strbuf[1] = 0;
+	return strbuf;
+    } else {
+	GetCurrentDirectory(MAX_PATH, cod);
+	SetCurrentDirectory(cwd);
+	for (i = 0; i <  10; i++) if (peekevent()) doevent();
+	return strbuf;
+    }
 }
 
 int countFilenames(const char *list)
 {
-	const char *temp;
-	int count;
-	count = 0;
-	for (temp = list; *temp; temp += strlen(temp)+1) count++;
-	return count;
+    const char *temp;
+    int count;
+    count = 0;
+    for (temp = list; *temp; temp += strlen(temp)+1) count++;
+    return count;
 }
 
 char *askfilesave(const char *title, const char *default_name)
@@ -279,54 +270,54 @@ char *askfilesave(const char *title, const char *default_name)
 
 char *askfilesavewithdir(const char *title, const char *default_name, const char *dir)
 {
-	int i;
-	OPENFILENAME ofn;
-        char *p, cwd[MAX_PATH], *defext = NULL;
+    int i;
+    OPENFILENAME ofn;
+    char *p, cwd[MAX_PATH], *defext = NULL;
 
-	if (!default_name) default_name = "";
-	else if(default_name[0] == '|') {
-	    defext = (char *)default_name + 2;
-	    default_name = "";
-	}
-	strcpy(strbuf, default_name);
+    if (!default_name) default_name = "";
+    else if(default_name[0] == '|') {
+	defext = (char *)default_name + 2;
+	default_name = "";
+    }
+    strcpy(strbuf, default_name);
 
-	ofn.lStructSize     = sizeof(OPENFILENAME);
-	ofn.hwndOwner       = current_window ? current_window->handle : 0;
-	ofn.hInstance       = 0;
-        ofn.lpstrFilter     = userfilter?userfilter:filter[0];
-	ofn.lpstrCustomFilter = NULL;
-	ofn.nMaxCustFilter  = 0;
-	ofn.nFilterIndex    = 0;
-	ofn.lpstrFile       = strbuf;
-	ofn.nMaxFile        = BUFSIZE;
-	ofn.lpstrFileTitle  = NULL;
-	ofn.nMaxFileTitle   = _MAX_FNAME + _MAX_EXT;
-	if(dir && strlen(dir) > 0) {
-	    strcpy(cwd, dir);
-	    for(p = cwd; *p; p++) if(*p == '/') *p = '\\';
+    ofn.lStructSize     = sizeof(OPENFILENAME);
+    ofn.hwndOwner       = current_window ? current_window->handle : 0;
+    ofn.hInstance       = 0;
+    ofn.lpstrFilter     = userfilter?userfilter:filter[0];
+    ofn.lpstrCustomFilter = NULL;
+    ofn.nMaxCustFilter  = 0;
+    ofn.nFilterIndex    = 0;
+    ofn.lpstrFile       = strbuf;
+    ofn.nMaxFile        = BUFSIZE;
+    ofn.lpstrFileTitle  = NULL;
+    ofn.nMaxFileTitle   = _MAX_FNAME + _MAX_EXT;
+    if(dir && strlen(dir) > 0) {
+	strcpy(cwd, dir);
+	for(p = cwd; *p; p++) if(*p == '/') *p = '\\';
+	ofn.lpstrInitialDir = cwd;
+    } else {
+	if (GetCurrentDirectory(MAX_PATH, cwd))
 	    ofn.lpstrInitialDir = cwd;
-	} else {
-	    if (GetCurrentDirectory(MAX_PATH, cwd))
-		ofn.lpstrInitialDir = cwd;
-	    else
-		ofn.lpstrInitialDir = NULL;
-	}
-	ofn.lpstrTitle      = title;
-	ofn.Flags           = OFN_OVERWRITEPROMPT |
-                              OFN_NOCHANGEDIR | OFN_HIDEREADONLY;
-	ofn.nFileOffset     = 0;
-	ofn.nFileExtension  = 0;
-	ofn.lpstrDefExt     = defext;
-	ofn.lCustData       = 0L;
-	ofn.lpfnHook        = NULL;
-	ofn.lpTemplateName  = NULL;
+	else
+	    ofn.lpstrInitialDir = NULL;
+    }
+    ofn.lpstrTitle      = title;
+    ofn.Flags           = OFN_OVERWRITEPROMPT |
+	OFN_NOCHANGEDIR | OFN_HIDEREADONLY;
+    ofn.nFileOffset     = 0;
+    ofn.nFileExtension  = 0;
+    ofn.lpstrDefExt     = defext;
+    ofn.lCustData       = 0L;
+    ofn.lpfnHook        = NULL;
+    ofn.lpTemplateName  = NULL;
 
-	if (GetSaveFileName(&ofn) == 0)
-	    return NULL;
-	else {
-	    for (i = 0; i < 10; i++) if (peekevent()) doevent();
-	    return strbuf;
-	}
+    if (GetSaveFileName(&ofn) == 0)
+	return NULL;
+    else {
+	for (i = 0; i < 10; i++) if (peekevent()) doevent();
+	return strbuf;
+    }
 }
 
 /*
@@ -336,17 +327,17 @@ char *askfilesavewithdir(const char *title, const char *default_name, const char
 /*
  *  Dialog information structure:
  */
-	#define NOT_CHOSEN_YET -2
+#define NOT_CHOSEN_YET -2
 
-	typedef struct dialog_data_class {
-		int 	hit;
-		char *	result;
-		label	question;
-		field	text, pass;
-		button	yes, no, cancel;
-	} dialog_data;
+typedef struct dialog_data_class {
+    int	hit;
+    char *	result;
+    label	question;
+    field	text, pass;
+    button	yes, no, cancel;
+} dialog_data;
 
-	#define data(w) ((dialog_data *) (getdata(w)))
+#define data(w) ((dialog_data *) (getdata(w)))
 
 /*
  *  Some strings to use:
@@ -355,94 +346,83 @@ char *askfilesavewithdir(const char *title, const char *default_name, const char
 	static char * CANCEL_STRING	= "Cancel";
 	static char * BROWSE_STRING	= "Browse"; */
 
-	static const char * QUESTION_TITLE	= "Question";
-	static const char * PASSWORD_TITLE	= "Password Entry";
-	static const char * FINDDIR_TITLE	= "Choose directory";
+static const char * QUESTION_TITLE	= "Question";
+static const char * PASSWORD_TITLE	= "Password Entry";
 
 static void add_data(window w)
 {
-	dialog_data *d;
+    dialog_data *d;
 
-	d = create (dialog_data);
-	if (! d)
-		return;
-	d->hit = NOT_CHOSEN_YET;
+    d = create (dialog_data);
+    if (! d)
+	return;
+    d->hit = NOT_CHOSEN_YET;
 
-	setdata(w, d);
+    setdata(w, d);
 }
 
 static char * get_dialog_string(window w)
 {
-	dialog_data *d = data(w);
+    dialog_data *d = data(w);
 
-	if (d->hit < YES) /* cancelled */
-		return NULL;
-	del_string(d->result);
-	if (d->text)	/* question dialog */
-		d->result = new_string(GA_gettext(d->text));
+    if (d->hit < YES) /* cancelled */
+	return NULL;
+    del_string(d->result);
+    if (d->text)	/* question dialog */
+	d->result = new_string(GA_gettext(d->text));
 
-	return d->result;
+    return d->result;
 }
 
 static void hit_button(control c)
-{
-	window w = parentwindow(c);
-	dialog_data *d = data(w);
-	int value = getvalue(c);
-
-	d->hit = value;
-	hide(w);
-}
-
-static void browse_button(control c)
 {
     window w = parentwindow(c);
     dialog_data *d = data(w);
-    char strbuf[MAX_PATH];
-    strcpy(strbuf, GA_gettext(d->text));
-    selectfolder(strbuf, G_("Choose a folder"));
-    if(strlen(strbuf)) settext(d->text, strbuf);
+    int value = getvalue(c);
+
+    d->hit = value;
+    hide(w);
 }
 
 static void hit_key(window w, int key)
 {
-	button btn;
-	char *name = NULL;
+    button btn;
+    char *name = NULL;
 
-	w = parentwindow(w);
+    w = parentwindow(w);
 
-	if (data(w) == NULL)
-		return;
+    if (data(w) == NULL)
+	return;
 
-	if ((btn = data(w)->yes) != NULL) {
-		name = getname(btn);
-		if ((key == '\n') || (tolower(name[0]) == tolower(key)))
-		{
-			flashcontrol(btn);
-			activatecontrol(btn);
-			return;
-		}
+    if ((btn = data(w)->yes) != NULL) {
+	name = getname(btn);
+	if ((key == '\n') || (tolower(name[0]) == tolower(key)))
+	{
+	    flashcontrol(btn);
+	    activatecontrol(btn);
+	    return;
 	}
+    }
 
-	if ((btn = data(w)->cancel) != NULL) {
-		name = getname(btn);
-		if ((key == ESC) || (tolower(name[0]) == tolower(key)))
-		{
-			flashcontrol(btn);
-			activatecontrol(btn);
-			return;
-		}
+    if ((btn = data(w)->cancel) != NULL) {
+	name = getname(btn);
+	if ((key == ESC) || (tolower(name[0]) == tolower(key)))
+	{
+	    flashcontrol(btn);
+	    activatecontrol(btn);
+	    return;
 	}
+    }
 
-	if ((btn = data(w)->no) != NULL) {
-		name = getname(btn);
-		if ((key == ESC) || (tolower(name[0]) == tolower(key)))
-		{
-			flashcontrol(btn);
-			activatecontrol(btn);
-			return;
-		}
+    if ((btn = data(w)->no) != NULL) {
+	name = getname(btn);
+	if ((key == ESC) || (tolower(name[0]) == tolower(key)))
+	{
+	    flashcontrol(btn);
+	    activatecontrol(btn);
+	    return;
 	}
+    }
 
 }
 
@@ -451,128 +431,148 @@ static void hit_key(window w, int key)
  */
 static int handle_message_dialog(window w)
 {
-	window old;
-	dialog_data *d = data(w);
+    window old;
+    dialog_data *d = data(w);
 
-	old = currentdrawing();
-	d->hit = NOT_CHOSEN_YET;
+    old = currentdrawing();
+    d->hit = NOT_CHOSEN_YET;
 
-	show(w);
-	while (d->hit == NOT_CHOSEN_YET)
-		doevent();
-	hide(w);
+    show(w);
+    while (d->hit == NOT_CHOSEN_YET) {
+	if (!peekevent()) WaitMessage();
+	doevent();
+    }
+    hide(w);
 
-	if (old) drawto(old);
+    if (old) drawto(old);
 
-	return d->hit;
+    return d->hit;
 }
 
 static window init_askstr_dialog(const char *title, const char *question,
 				 const char *default_str)
 {
-	window win;
-	dialog_data *d;
-	int tw, bw, h, middle;
+    window win;
+    dialog_data *d;
+    int tw, bw, h, middle;
 
-	if (! question)
-		question= "";
-	if (! default_str)
-		default_str = "";
+    if (! question)
+	question= "";
+    if (! default_str)
+	default_str = "";
 
-	tw = strwidth(SystemFont, G_("Cancel")) * 8;
-	h = getheight(SystemFont);
+    tw = strwidth(SystemFont, G_("Cancel")) * 8;
+    h = getheight(SystemFont);
 
-	if (tw < 150) tw = 150;
+    if (tw < 150) tw = 150;
 
-	win = newwindow(title, rect(0,0,tw+30,h*9+12),
-			Titlebar | Centered | Modal);
-        setbackground(win, dialog_bg());
-	add_data(win);
-	d = data(win);
-	d->question = newlabel(question, rect(10,h,tw+4,h*2+2),
-			AlignLeft);
-	if (title == FINDDIR_TITLE) {
-	    bw = strwidth(SystemFont, G_("Browse")) * 3/2;
-	    d->text = newfield(default_str, rect(10,h*4,tw+4-bw,h*3/2));
-	    newbutton(G_("Browse"), rect(20+tw-bw, h*4-2, bw, h+10),
-		      browse_button);
-	}
-	else if (title == PASSWORD_TITLE)
-		d->text = newpassword(default_str, rect(10,h*4,tw+4,h*3/2));
-	else
-		d->text = newfield(default_str, rect(10,h*4,tw+4,h*3/2));
+    win = newwindow(title, rect(0,0,tw+30,h*9+12),
+		    Titlebar | Centered | Modal);
+    setbackground(win, dialog_bg());
+    add_data(win);
+    d = data(win);
+    d->question = newlabel(question, rect(10,h,tw+4,h*2+2),
+			   AlignLeft);
+    if (title == PASSWORD_TITLE)
+	d->text = newpassword(default_str, rect(10,h*4,tw+4,h*3/2));
+    else
+	d->text = newfield(default_str, rect(10,h*4,tw+4,h*3/2));
 
-	middle = (tw+30)/2;
-	bw = strwidth(SystemFont, G_("Cancel")) * 3/2;
+    middle = (tw+30)/2;
+    bw = strwidth(SystemFont, G_("Cancel")) * 3/2;
 
-	d->yes = newbutton(G_("OK"),
-			rect(middle-bw-10, h*7, bw, h+10), hit_button);
-	setvalue(d->yes, YES);
+    d->yes = newbutton(G_("OK"),
+		       rect(middle-bw-10, h*7, bw, h+10), hit_button);
+    setvalue(d->yes, YES);
 
-	d->cancel = newbutton(G_("Cancel"),
-			rect(middle+10, h*7, bw, h+10), hit_button);
-	setvalue(d->cancel, CANCEL);
+    d->cancel = newbutton(G_("Cancel"),
+			  rect(middle+10, h*7, bw, h+10), hit_button);
+    setvalue(d->cancel, CANCEL);
 
-	setkeydown(win, hit_key);
+    setkeydown(win, hit_key);
 
-	return win;
+    return win;
 }
 
 char *askstring(const char *question, const char *default_str)
 {
-	static window win = NULL;
-	window prev = current_window;
+    static window win = NULL;
+    window prev = current_window;
 
-	if (! win)
-		win = init_askstr_dialog(QUESTION_TITLE, question, default_str);
-	else {
-		settext(data(win)->question, question);
-		settext(data(win)->text, default_str);
-	}
-	if (TopmostDialogs & MB_TOPMOST)
-	    BringToTop(win, 1);
-	handle_message_dialog(win);
-	current_window = prev;
+    if (! win)
+	win = init_askstr_dialog(QUESTION_TITLE, question, default_str);
+    else {
+	settext(data(win)->question, question);
+	settext(data(win)->text, default_str);
+    }
+    if (TopmostDialogs & MB_TOPMOST)
+	BringToTop(win, 1);
+    handle_message_dialog(win);
+    current_window = prev;
 
-	return get_dialog_string(win);
+    return get_dialog_string(win);
 }
 
 char *askcdstring(const char *question, const char *default_str)
 {
-	static window win = NULL;
-	window prev = current_window;
+    LPMALLOC g_pMalloc;
+    BROWSEINFO bi;
+    LPITEMIDLIST pidlBrowse;
+    browserInfo info;
+    OSVERSIONINFOEX osvi;
 
-	if (! win)
-		win = init_askstr_dialog(FINDDIR_TITLE, question, default_str);
-	else {
-		settext(data(win)->question, question);
-		settext(data(win)->text, default_str);
-	}
-	if (TopmostDialogs & MB_TOPMOST)
-	    BringToTop(win, 1);
-	handle_message_dialog(win);
-	current_window = prev;
+    strncpy(info.question, question, 40);
+    strncpy(info.default_str, default_str, MAX_PATH);
 
-	return get_dialog_string(win);
+    /* Get the shell's allocator. */
+    if (!SUCCEEDED(SHGetMalloc(&g_pMalloc))) return NULL;
+
+    osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
+    GetVersionEx((OSVERSIONINFO *)&osvi);
+
+    ZeroMemory(&bi, sizeof(bi));
+    bi.hwndOwner = 0;
+    if(osvi.dwMajorVersion >= 6) { /* future proof */
+	/* CSIDL_DESKTOP gets mapped to the User's desktop in Vista
+	   (a bug).  SHGetFolderLocation is Win2k or later */
+	if (!SUCCEEDED(SHGetFolderLocation(NULL, CSIDL_DRIVES, NULL, 0,
+					   (LPITEMIDLIST *) &bi.pidlRoot)))
+	    return NULL;
+    }  /* else it is 0, which is CSIDL_DESKTOP */
+    bi.pszDisplayName = strbuf;
+    bi.lpszTitle = question;
+    bi.ulFlags = BIF_RETURNONLYFSDIRS | BIF_USENEWUI;
+    bi.lpfn = (BFFCALLBACK) InitBrowseCallbackProc;
+    bi.lParam = (LPARAM) &info;
+
+    /* Browse for a folder and return its PIDL. */
+    pidlBrowse = SHBrowseForFolder(&bi);
+    if (pidlBrowse != NULL) {
+	SHGetPathFromIDList(pidlBrowse, strbuf);
+	g_pMalloc->lpVtbl->Free(g_pMalloc, pidlBrowse);
+	if (strbuf[0])
+	    return strbuf;
+    }
+    return NULL;
 }
 
 char *askpassword(const char *question, const char *default_str)
 {
-	static window win = NULL;
-	window prev = current_window;
+    static window win = NULL;
+    window prev = current_window;
 
-	if (! win)
-		win = init_askstr_dialog(PASSWORD_TITLE, question, default_str);
-	else {
-		settext(data(win)->question, question);
-		settext(data(win)->text, default_str);
-	}
-	if (TopmostDialogs & MB_TOPMOST)
-	    BringToTop(win, 1);
-	handle_message_dialog(win);
-	current_window = prev;
+    if (! win)
+	win = init_askstr_dialog(PASSWORD_TITLE, question, default_str);
+    else {
+	settext(data(win)->question, question);
+	settext(data(win)->text, default_str);
+    }
+    if (TopmostDialogs & MB_TOPMOST)
+	BringToTop(win, 1);
+    handle_message_dialog(win);
+    current_window = prev;
 
-	return get_dialog_string(win);
+    return get_dialog_string(win);
 }
 
 char *askUserPass(const char *title)
@@ -589,7 +589,7 @@ char *askUserPass(const char *title)
 	if (tw < 150) tw = 150;
 	win = newwindow(title, rect(0, 0, tw+30, h*9+12),
 			Titlebar | Centered | Modal);
-        setbackground(win, dialog_bg());
+	setbackground(win, dialog_bg());
 	add_data(win);
 	d = data(win);
 	d->question = newlabel(G_("User"), rect(10, h, tw+4, h*2+2), AlignLeft);
@@ -645,7 +645,8 @@ HWND get_modeless()
     return hModelessDlg;
 }
 
-void finddialog(textbox t){
+void finddialog(textbox t)
+{
     static FINDREPLACE fr;
     static char szFindWhat[80];
 
@@ -661,7 +662,8 @@ void finddialog(textbox t){
     hModelessDlg = FindText(&fr);
 }
 
-void replacedialog(textbox t){
+void replacedialog(textbox t)
+{
     static FINDREPLACE fr;
     static char szFindWhat[80];
     static char szReplaceWith[80];
@@ -681,6 +683,7 @@ void replacedialog(textbox t){
 }
 
 
+#include <richedit.h>
 /* Find and select a string in a rich edit control */
 
 static int richeditfind(HWND hwnd, char *what, int matchcase,
@@ -777,4 +780,3 @@ void handle_findreplace(HWND hwnd, LPFINDREPLACE pfr)
     else if (pfr->Flags & FR_DIALOGTERM)
 	hModelessDlg = NULL;
 }
-
