@@ -1,3 +1,19 @@
+#  File src/library/base/R/eigen.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 
 isSymmetric <- function(object, ...) UseMethod("isSymmetric")
 
@@ -8,9 +24,9 @@ isSymmetric.matrix <- function(object, tol = 100*.Machine$double.eps, ...) {
     if(d[1] != d[2]) return(FALSE)
     test <-
         if(is.complex(object))
-            all.equal.numeric(object, Conj(t(object)), tol = tol, ...)
+            all.equal.numeric(object, Conj(t(object)), tolerance = tol, ...)
         else # numeric, character, ..
-            all.equal(object, t(object), tol = tol, ...)
+            all.equal(object, t(object), tolerance = tol, ...)
     isTRUE(test)
 }
 
@@ -72,8 +88,8 @@ eigen <- function(x, symmetric, only.values = FALSE, EISPACK = FALSE)
 		stop(gettextf("'ch' returned code %d in 'eigen'", z$ierr),
                      domain = NA)
 	    if(!only.values)
-		z$vectors <- matrix(complex(re=z$vectors,
-					    im=z$ivectors), nc=n)
+		z$vectors <- matrix(complex(real=z$vectors,
+					    imaginary=z$ivectors), ncol=n)
 	}
 	else {
 	    z <- .Fortran("rs",
@@ -115,10 +131,10 @@ eigen <- function(x, symmetric, only.values = FALSE, EISPACK = FALSE)
 	    if (z$ierr)
 		stop(gettextf("'cg' returned code %d in 'eigen'", z$ierr),
                      domain = NA)
-	    z$values <- complex(re=z$values,im=z$ivalues)
+	    z$values <- complex(real=z$values,imaginary=z$ivalues)
 	    if(!only.values)
-		z$vectors <- matrix(complex(re=z$vectors,
-					    im=z$ivectors), nc=n)
+		z$vectors <- matrix(complex(real=z$vectors,
+					    imaginary=z$ivectors), ncol=n)
 	}
 	else {
 	    z <- .Fortran("rg",
@@ -139,10 +155,10 @@ eigen <- function(x, symmetric, only.values = FALSE, EISPACK = FALSE)
 	    ind <- z$ivalues > 0
 	    if(any(ind)) {#- have complex (conjugated) values
 		ind <- seq.int(n)[ind]
-		z$values <- complex(re=z$values,im=z$ivalues)
+		z$values <- complex(real=z$values,imaginary=z$ivalues)
 		if(!only.values) {
-		    z$vectors[, ind] <- complex(re=z$vectors[,ind],
-						im=z$vectors[,ind+1])
+		    z$vectors[, ind] <- complex(real=z$vectors[,ind],
+						imaginary=z$vectors[,ind+1])
 		    z$vectors[, ind+1] <- Conj(z$vectors[,ind])
 		}
 	    }
