@@ -1,3 +1,19 @@
+#  File src/library/base/R/expand.grid.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 expand.grid <- function(..., KEEP.OUT.ATTRS = TRUE)
 {
     ## x should either be a list or a set of vectors or factors
@@ -10,7 +26,7 @@ expand.grid <- function(..., KEEP.OUT.ATTRS = TRUE)
     nmc <- paste("Var", 1:nargs, sep="")
     nm <- names(args)
     if(is.null(nm)) nm <- nmc
-    else if(any(ng0 <- nchar(nm) > 0)) nmc[ng0] <- nm[ng0]
+    else if(any(ng0 <- nzchar(nm))) nmc[ng0] <- nm[ng0]
     names(cargs) <- nmc
     rep.fac <- 1
     d <- sapply(args, length)
