@@ -1,3 +1,19 @@
+#  File src/library/base/R/seq.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 seq <- function(...) UseMethod("seq")
 
 seq.default <-
@@ -61,10 +77,5 @@ seq.default <-
     else stop("too many arguments")
 }
 
-sequence <- function(nvec)
-{
-    s <- integer(0)
-    for(i in nvec)
-	s <- c(s, 1:i)
-    return(s)
-}
+## In reverence to the very first versions of R which already had sequence():
+sequence <- function(nvec) unlist(lapply(nvec, seq_len))
