@@ -153,5 +153,7 @@ as.character.srcref <- function(x, useSource = TRUE, ...)
     lines
 }
 
-print.srcref <- function(x, useSource = TRUE, ...)
+print.srcref <- function(x, useSource = TRUE, ...) {
     cat(as.character(x, useSource = useSource), sep="\n")
+    invisible(x)
+}
