@@ -1,3 +1,24 @@
+#  File src/library/base/R/zzz.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
+## top-level assignments that need to be copied to baseloader.R
+as.numeric <- as.real <- as.double
+is.name <- is.symbol
+
+
 ## extracted from existing NAMESPACE files in Dec 2003
 .knownS3Generics <- local({
 
@@ -28,7 +49,7 @@
 
 .ArgsEnv <- new.env(hash = TRUE, parent = emptyenv())
 
-assign("%*%", function(a, b) NULL, envir = .ArgsEnv)
+assign("%*%", function(x, y) NULL, envir = .ArgsEnv)
 assign(".C", function(name, ..., NAOK = FALSE, DUP = TRUE, PACKAGE,
                       ENCODING) NULL,
        envir = .ArgsEnv)
@@ -48,7 +69,7 @@ assign(".subset", function(x, ...) NULL, envir = .ArgsEnv)
 assign(".subset2", function(x, ...) NULL, envir = .ArgsEnv)
 assign("as.call", function(x) NULL, envir = .ArgsEnv)
 assign("as.environment", function(object) NULL, envir = .ArgsEnv)
-assign("attr", function(x, which) NULL, envir = .ArgsEnv)
+assign("attr", function(x, which, exact = FALSE) NULL, envir = .ArgsEnv)
 assign("attr<-", function(x, which, value) NULL, envir = .ArgsEnv)
 assign("attributes", function(obj) NULL, envir = .ArgsEnv)
 assign("attributes<-", function(obj, value) NULL, envir = .ArgsEnv)
@@ -72,9 +93,7 @@ assign("is.complex", function(x) NULL, envir = .ArgsEnv)
 assign("is.double", function(x) NULL, envir = .ArgsEnv)
 assign("is.environment", function(x) NULL, envir = .ArgsEnv)
 assign("is.expression", function(x) NULL, envir = .ArgsEnv)
-assign("is.finite", function(x) NULL, envir = .ArgsEnv)
 assign("is.function", function(x) NULL, envir = .ArgsEnv)
-assign("is.infinite", function(x) NULL, envir = .ArgsEnv)
 assign("is.integer", function(x) NULL, envir = .ArgsEnv)
 assign("is.language", function(x) NULL, envir = .ArgsEnv)
 assign("is.list", function(x) NULL, envir = .ArgsEnv)
@@ -89,8 +108,11 @@ assign("is.recursive", function(x) NULL, envir = .ArgsEnv)
 assign("is.single", function(x) NULL, envir = .ArgsEnv)
 assign("is.symbol", function(x) NULL, envir = .ArgsEnv)
 assign("list", function(...) NULL, envir = .ArgsEnv)
+assign("lazyLoadDBfetch", function(key, file, compressed, hook) NULL,
+       envir = .ArgsEnv)
 assign("missing", function(x) NULL, envir = .ArgsEnv)
 assign("nargs", function() NULL, envir = .ArgsEnv)
+assign("nzchar", function(x) NULL, envir = .ArgsEnv)
 assign("oldClass", function(x) NULL, envir = .ArgsEnv)
 assign("oldClass<-", function(x, value) NULL, envir = .ArgsEnv)
 assign("on.exit", function(expr, add = FALSE) NULL, envir = .ArgsEnv)
@@ -101,6 +123,7 @@ assign("retracemem", function(x, previous = NULL) NULL, envir = .ArgsEnv)
 assign("seq_along", function(along.with) NULL, envir = .ArgsEnv)
 assign("seq_len", function(length.out) NULL, envir = .ArgsEnv)
 assign("standardGeneric", function(f) NULL, envir = .ArgsEnv)
+assign("storage.mode<-", function(x, value) NULL, envir = .ArgsEnv)
 assign("substitute", function(expr, env) NULL, envir = .ArgsEnv)
 assign("tracemem", function(x) NULL, envir = .ArgsEnv)
 assign("unclass", function(x) NULL, envir = .ArgsEnv)
@@ -109,9 +132,12 @@ assign("untracemem", function(x) NULL, envir = .ArgsEnv)
 assign("UseMethod", function(generic, object) NULL, envir = .ArgsEnv)
 
 
-.S3PrimitiveGenerics <- c("as.character", "c", "dim", "dim<-",
-    "dimnames", "dimnames<-", "is.array", "is.matrix", "is.na",
-    "is.nan", "is.numeric", "length", "length<-", "levels<-",
+.S3PrimitiveGenerics <-
+    c("as.character", "as.complex", "as.double", "as.integer",
+    "as.logical", "as.numeric", "as.raw", "as.real", "c", "dim",
+    "dim<-", "dimnames", "dimnames<-", "is.array", "is.finite",
+    "is.infinite", "is.matrix",
+    "is.na", "is.nan", "is.numeric", "length", "length<-", "levels<-",
     "names", "names<-", "rep", "seq.int")
 
 .GenericArgsEnv <- local({
@@ -123,11 +149,13 @@ assign("UseMethod", function(generic, object) NULL, envir = .ArgsEnv)
         assign(f, fx, envir = env)
     }
     ## now add the group generics
-    ## log, round, signif and the gamma fns are not primitive
+    ## round, signif, log, trunc are handled below
     fx <- function(x) {}
-    for(f in c('abs', 'sign', 'sqrt', 'floor', 'ceiling', 'trunc', 'exp',
+    for(f in c('abs', 'sign', 'sqrt', 'floor', 'ceiling',
+               'exp', 'expm1', 'log1p', 'log10', 'log2',
                'cos', 'sin', 'tan', 'acos', 'asin', 'atan', 'cosh', 'sinh',
                'tanh', 'acosh', 'asinh', 'atanh',
+               'gamma', 'lgamma', 'digamma', 'trigamma',
                'cumsum', 'cumprod', 'cummax', 'cummin')) {
         body(fx) <- substitute(UseMethod(ff), list(ff=f))
         environment(fx) <- .BaseNamespaceEnv
@@ -143,7 +171,13 @@ assign("UseMethod", function(generic, object) NULL, envir = .ArgsEnv)
         assign(f, fx, envir = env)
     }
 
-    ## none of Summary is primitive
+    for(f in c("all", "any", "sum", "prod", "max", "min", "range")) {
+        fx <- function(..., na.rm = FALSE) {}
+        body(fx) <- substitute(UseMethod(ff), list(ff=f))
+        environment(fx) <- .BaseNamespaceEnv
+        assign(f, fx, envir = env)
+    }
+
     for(f in c("Arg", "Conj", "Im", "Mod", "Re")) {
         fx <- function(z) {}
         body(fx) <- substitute(UseMethod(ff), list(ff=f))
@@ -156,6 +190,15 @@ assign("UseMethod", function(generic, object) NULL, envir = .ArgsEnv)
 assign("!", function(x) UseMethod("!"), envir = .GenericArgsEnv)
 assign("as.character", function(x, ...) UseMethod("as.character"),
        envir = .GenericArgsEnv)
+assign("as.complex", function(x, ...) UseMethod("as.complex"),
+       envir = .GenericArgsEnv)
+assign("as.double", function(x, ...) UseMethod("as.double"),
+       envir = .GenericArgsEnv)
+assign("as.integer", function(x, ...) UseMethod("as.integer"),
+       envir = .GenericArgsEnv)
+assign("as.logical", function(x, ...) UseMethod("as.logical"),
+       envir = .GenericArgsEnv)
+assign("as.raw", function(x) UseMethod("as.raw"), envir = .GenericArgsEnv)
 assign("c", function(..., recursive = FALSE) UseMethod("c"),
        envir = .GenericArgsEnv)
 assign("dimnames", function(x) UseMethod("dimnames"), envir = .GenericArgsEnv)
@@ -166,8 +209,21 @@ assign("length<-", function(x, value) UseMethod("length<-"),
        envir = .GenericArgsEnv)
 assign("levels<-", function(x, value) UseMethod("levels<-"),
        envir = .GenericArgsEnv)
+assign("log", function(x, base=exp(1)) UseMethod("log"),
+       envir = .GenericArgsEnv)
 assign("names<-", function(x, value) UseMethod("names<-"),
        envir = .GenericArgsEnv)
 assign("rep", function(x, ...) UseMethod("rep"), envir = .GenericArgsEnv)
+assign("round", function(x, digits=0) UseMethod("round"),
+       envir = .GenericArgsEnv)
 assign("seq.int", function(from, to, by, length.out, along.with, ...)
        UseMethod("seq.int"), envir = .GenericArgsEnv)
+assign("signif", function(x, digits=6) UseMethod("signif"),
+       envir = .GenericArgsEnv)
+assign("trunc", function(x, ...) UseMethod("trunc"), envir = .GenericArgsEnv)
+
+## make these the same object as as.double
+assign("as.numeric", get("as.double", envir = .GenericArgsEnv),
+       envir = .GenericArgsEnv)
+assign("as.real", get("as.double", envir = .GenericArgsEnv),
+       envir = .GenericArgsEnv)
