@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2004-5   The R Development Core Team.
+ *  Copyright (C) 2004-8   The R Development Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,18 +19,24 @@
 
 #include <R.h>
 #include <Rinternals.h>
-#include <Graphics.h>
-#include <Rdevices.h>
+#include <R_ext/GraphicsEngine.h>
 
 #include "grDevices.h"
 #include <R_ext/Rdynload.h>
 
 static R_NativePrimitiveArgType R_chull_t[] = {INTSXP, REALSXP, INTSXP, INTSXP, INTSXP, INTSXP, INTSXP, INTSXP, INTSXP};
 
+#ifndef WIN32
+void *getQuartzAPI();
+#endif
+
 #define CDEF(name)  {#name, (DL_FUNC) &name, sizeof(name ## _t)/sizeof(name ## _t[0]), name ##_t}
 
 static R_CMethodDef CEntries [] = {
     CDEF(R_chull),
+#ifndef WIN32
+    {"getQuartzAPI", (DL_FUNC) getQuartzAPI, 0},
+#endif
     {NULL, NULL, 0}
 };
 
@@ -40,6 +46,9 @@ static const R_CallMethodDef CallEntries[] = {
     CALLDEF(Type1FontInUse, 2),
     CALLDEF(CIDFontInUse, 2),
     {"R_GD_nullDevice", (DL_FUNC) &R_GD_nullDevice, 0},
+#ifndef WIN32
+    CALLDEF(makeQuartzDefault, 0),
+#endif
     {NULL, NULL, 0}
 };
 
@@ -51,10 +60,10 @@ static const R_ExternalMethodDef ExtEntries[] = {
     EXTDEF(XFig, 11),
     EXTDEF(PDF, 13),
 #ifdef WIN32
-    EXTDEF(devga, 15),
+    EXTDEF(devga, 16),
     EXTDEF(savePlot, 3),
 #else
-    EXTDEF(Quartz, -1),
+    EXTDEF(Quartz, 12),
 #endif
     {NULL, NULL, 0}
 };
