@@ -1,6 +1,22 @@
+#  File src/library/graphics/R/filled.contour.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 filled.contour <-
-function (x = seq(0, 1, len = nrow(z)),
-          y = seq(0, 1, len = ncol(z)),
+function (x = seq(0, 1, length.out = nrow(z)),
+          y = seq(0, 1, length.out = ncol(z)),
           z,
           xlim = range(x, finite=TRUE),
           ylim = range(y, finite=TRUE),
@@ -21,7 +37,7 @@ function (x = seq(0, 1, len = nrow(z)),
             }
             else {
                 z <- x
-                x <- seq(0, 1, len = nrow(z))
+                x <- seq.int(0, 1, length.out = nrow(z))
             }
         }
         else stop("no 'z' matrix specified")
@@ -37,7 +53,7 @@ function (x = seq(0, 1, len = nrow(z)),
     on.exit(par(par.orig))
 
     w <- (3 + mar.orig[2]) * par('csi') * 2.54
-    layout(matrix(c(2, 1), nc=2), widths=c(1, lcm(w)))
+    layout(matrix(c(2, 1), ncol=2), widths=c(1, lcm(w)))
     par(las = las)
 
     ## Plot the 'plot key' (scale):
