@@ -1,3 +1,19 @@
+#  File src/library/graphics/R/grid.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 grid <- function (nx = NULL, ny = nx, col="lightgray", lty="dotted",
                   lwd = par("lwd"), equilogs = TRUE)
 {
@@ -9,7 +25,7 @@ grid <- function (nx = NULL, ny = nx, col="lightgray", lty="dotted",
             at <- axTicks(1, axp = ax, log=log)
         } else { # equidistant, also from box borders
             U <- par("usr")
-            at <- seq.int(U[1],U[2], len = nx+1)
+            at <- seq.int(U[1],U[2], length.out = nx+1)
             at <- (if(log) 10^at else at)[-c(1,nx+1)]
         }
         abline(v = at, col = col, lty = lty, lwd = lwd)
@@ -22,7 +38,7 @@ grid <- function (nx = NULL, ny = nx, col="lightgray", lty="dotted",
             at <- axTicks(2, axp = ax, log=log)
         } else { # equidistant, also from box borders
             U <- par("usr")
-            at <- seq.int(U[3],U[4], len = ny+1)
+            at <- seq.int(U[3],U[4], length.out = ny+1)
             at <- (if(log) 10^at else at)[-c(1,ny+1)]
         }
 	abline(h = at, col = col, lty = lty, lwd = lwd)
