@@ -1,3 +1,19 @@
+#  File src/library/graphics/R/layout.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 lcm <- function(x) paste(x, "cm")#-> 3 characters (used in layout!)
 
 layout <-
@@ -32,6 +48,7 @@ layout <-
 	if (is.character(v)) {
 	    wcm <- v[cm.v]
 	    v[cm.v] <- substring(wcm, 1, nchar(wcm, type="c") - 3)
+            v <- chartr(getOption("OutDec"), ".", v)
 	}
 	as.numeric(v)
     }
