@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 2001-3 Paul Murrell
- *                2003-7 The R Development Core Team
+ *                2003-8 The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include "grid.h"
 #include <math.h>
 #include <float.h>
+#include <string.h>
 
 /* NOTE:
  * The extensive use of L or L_ prefixes dates back to when this 
@@ -31,7 +32,7 @@
 
 extern int gridRegisterIndex;
 
-void getDeviceSize(GEDevDesc *dd, double *devWidthCM, double *devHeightCM) 
+void getDeviceSize(pGEDevDesc dd, double *devWidthCM, double *devHeightCM) 
 {
     double left, right, bottom, top;
     dd->dev->size(&left, &right, &bottom, &top, dd->dev);
@@ -77,7 +78,7 @@ SEXP L_killGrid()
 
 /* Get the current device (the graphics engine creates one if nec.)
  */
-GEDevDesc* getDevice() 
+pGEDevDesc getDevice() 
 {
     return GEcurrentDevice();
 }
@@ -85,7 +86,7 @@ GEDevDesc* getDevice()
 /* If this is the first time that a grid operation has occurred for 
  * this device, do some initialisation.
  */
-void dirtyGridDevice(GEDevDesc *dd) {
+void dirtyGridDevice(pGEDevDesc dd) {
     if (!LOGICAL(gridStateElement(dd, GSS_GRIDDEVICE))[0]) {
 	SEXP gsd, griddev;
 	/* Record the fact that this device has now received grid output
@@ -119,7 +120,7 @@ SEXP L_gridDirty()
 {
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     dirtyGridDevice(dd);
     return R_NilValue;
 }
@@ -133,7 +134,7 @@ SEXP L_currentViewport()
 {
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     return gridStateElement(dd, GSS_VP);
 }
 
@@ -147,7 +148,7 @@ SEXP doSetViewport(SEXP vp,
 		    * (or just revisiting an already-pushed viewport?)
 		    */
 		   Rboolean pushing,
-		   GEDevDesc *dd)
+		   pGEDevDesc dd)
 {
     int i, j;
     double devWidthCM, devHeightCM;
@@ -206,7 +207,10 @@ SEXP doSetViewport(SEXP vp,
      */
     else if (viewportClip(vp)) {
 	double rotationAngle = REAL(viewportRotation(vp))[0];
-	if (rotationAngle != 0)
+	if (rotationAngle != 0 &&
+            rotationAngle != 90 &&
+            rotationAngle != 270 &&
+            rotationAngle != 360)
 	    warning(_("Cannot clip to rotated viewport"));
 	else {
 	    /* Calculate a clipping region and set it
@@ -300,7 +304,7 @@ SEXP L_setviewport(SEXP invp, SEXP hasParent)
     SEXP vp;
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     /*
      * Duplicate the viewport passed in because we are going
      * to modify it to hell and gone.
@@ -461,7 +465,7 @@ SEXP L_downviewport(SEXP name, SEXP strict)
 {
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     /* Get the value of the current viewport for the current device
      * Need to do this in here so that redrawing via R BASE display
      * list works 
@@ -595,7 +599,7 @@ SEXP L_downvppath(SEXP path, SEXP name, SEXP strict)
 {
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     /* Get the value of the current viewport for the current device
      * Need to do this in here so that redrawing via R BASE display
      * list works 
@@ -630,7 +634,7 @@ SEXP L_unsetviewport(SEXP n)
     SEXP parentClip;
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     /* Get the value of the current viewport for the current device
      * Need to do this in here so that redrawing via R BASE display
      * list works 
@@ -732,7 +736,7 @@ SEXP L_upviewport(SEXP n)
     SEXP parentClip;
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     /* Get the value of the current viewport for the current device
      * Need to do this in here so that redrawing via R BASE display
      * list works 
@@ -784,7 +788,7 @@ SEXP L_getDisplayList()
 {
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     return gridStateElement(dd, GSS_DL);
 }
 
@@ -792,7 +796,7 @@ SEXP L_setDisplayList(SEXP dl)
 {
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     setGridStateElement(dd, GSS_DL, dl);
     return R_NilValue;
 }
@@ -804,7 +808,7 @@ SEXP L_getDLelt(SEXP index)
 {
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     SEXP dl, result;
     PROTECT(dl = gridStateElement(dd, GSS_DL));
     result = VECTOR_ELT(dl, INTEGER(index)[0]);
@@ -819,7 +823,7 @@ SEXP L_setDLelt(SEXP value)
 {
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     SEXP dl;
     PROTECT(dl = gridStateElement(dd, GSS_DL));
     SET_VECTOR_ELT(dl, INTEGER(gridStateElement(dd, GSS_DLINDEX))[0], value);
@@ -831,7 +835,7 @@ SEXP L_getDLindex()
 {
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     return gridStateElement(dd, GSS_DLINDEX);
 }
 
@@ -839,7 +843,7 @@ SEXP L_setDLindex(SEXP index)
 {
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     setGridStateElement(dd, GSS_DLINDEX, index);
     return R_NilValue;
 }
@@ -848,7 +852,7 @@ SEXP L_getDLon()
 {
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     return gridStateElement(dd, GSS_DLON);
 }
 
@@ -856,7 +860,7 @@ SEXP L_setDLon(SEXP value)
 {
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     SEXP prev;
     prev = gridStateElement(dd, GSS_DLON);
     setGridStateElement(dd, GSS_DLON, value);
@@ -867,7 +871,7 @@ SEXP L_getEngineDLon()
 {
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     return gridStateElement(dd, GSS_ENGINEDLON);
 }
 
@@ -875,63 +879,50 @@ SEXP L_setEngineDLon(SEXP value)
 {
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     setGridStateElement(dd, GSS_ENGINEDLON, value);
     return R_NilValue;
 }
 
 SEXP L_getCurrentGrob() 
 {
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     return gridStateElement(dd, GSS_CURRGROB);
 }
 
 SEXP L_setCurrentGrob(SEXP value)
 {
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     setGridStateElement(dd, GSS_CURRGROB, value);
     return R_NilValue;
 }
 
 SEXP L_getEngineRecording() 
 {
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     return gridStateElement(dd, GSS_ENGINERECORDING);
 }
 
 SEXP L_setEngineRecording(SEXP value)
 {
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     setGridStateElement(dd, GSS_ENGINERECORDING, value);
     return R_NilValue;
 }
 
-SEXP L_getAsk() 
-{
-    GEDevDesc *dd = getDevice();
-    return gridStateElement(dd, GSS_ASK);
-}
-
-SEXP L_setAsk(SEXP value)
-{
-    GEDevDesc *dd = getDevice();
-    setGridStateElement(dd, GSS_ASK, value);
-    return R_NilValue;
-}
-
 SEXP L_currentGPar()
 {
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     return gridStateElement(dd, GSS_GPAR);
 }
 
 SEXP L_newpagerecording()
 {
-    GEDevDesc *dd = getDevice();
-    if (LOGICAL(gridStateElement(dd, GSS_ASK))[0]) {
-	NewFrameConfirm();
+    pGEDevDesc dd = getDevice();
+    if (dd->ask) {
+	NewFrameConfirm(dd->dev);
 	/*
 	 * User may have killed device during pause for prompt
 	 */
@@ -949,7 +940,7 @@ SEXP L_newpagerecording()
 
 SEXP L_newpage()
 {
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     R_GE_gcontext gc;
     /* 
      * Has the device been drawn on yet?
@@ -981,27 +972,27 @@ SEXP L_newpage()
 
 SEXP L_initGPar()
 {
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     initGPar(dd);
     return R_NilValue;
 }
 
 SEXP L_initViewportStack()
 {
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     initVP(dd);
     return R_NilValue;
 }
 
 SEXP L_initDisplayList()
 {
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     initDL(dd);
     return R_NilValue;
 }
 
 void getViewportTransform(SEXP currentvp, 
-			  GEDevDesc *dd, 
+			  pGEDevDesc dd, 
 			  double *vpWidthCM, double *vpHeightCM,
 			  LTransform transform, double *rotationAngle) 
 {
@@ -1066,7 +1057,7 @@ SEXP L_convert(SEXP x, SEXP whatfrom,
     /* 
      * Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     currentvp = gridStateElement(dd, GSS_VP);
     currentgp = gridStateElement(dd, GSS_GPAR);
     /* 
@@ -1201,7 +1192,7 @@ SEXP L_layoutRegion(SEXP layoutPosRow, SEXP layoutPosCol) {
     /* 
      * Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     currentvp = gridStateElement(dd, GSS_VP);
     currentgp = gridStateElement(dd, GSS_GPAR);
     /* 
@@ -1503,7 +1494,7 @@ static void hullEdge(double *x, double *y, int n,
  * Assume vertices are in DEVICE coordinates.
  */
 static void drawArrow(double *x, double *y, SEXP type, int i,
-		      R_GE_gcontext *gc, GEDevDesc *dd) 
+		      const pGEcontext gc, pGEDevDesc dd) 
 {
     int nt = LENGTH(type);
     switch (INTEGER(type)[i % nt]) {
@@ -1527,7 +1518,7 @@ static void calcArrow(double x1, double y1,
 		      LViewportContext vpc,
 		      double vpWidthCM, double vpHeightCM,
 		      double *vertx, double *verty,
-		      R_GE_gcontext *gc, GEDevDesc *dd) 
+		      const pGEcontext gc, pGEDevDesc dd) 
 {
     int na = LENGTH(angle);
     int nl = LENGTH(length);
@@ -1572,7 +1563,7 @@ static void arrows(double *x, double *y, int n,
 		   Rboolean start, Rboolean end,
 		   LViewportContext vpc,
 		   double vpWidthCM, double vpHeightCM,
-		   R_GE_gcontext *gc, GEDevDesc *dd) 
+		   const pGEcontext gc, pGEDevDesc dd) 
 {
     /*
      * Write a checkArrow() function to make
@@ -1634,7 +1625,7 @@ SEXP L_moveTo(SEXP x, SEXP y)
     SEXP currentvp, currentgp;
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     currentvp = gridStateElement(dd, GSS_VP);
     currentgp = gridStateElement(dd, GSS_GPAR);
     PROTECT(prevloc = gridStateElement(dd, GSS_PREVLOC));
@@ -1676,7 +1667,7 @@ SEXP L_lineTo(SEXP x, SEXP y, SEXP arrow)
     SEXP currentvp, currentgp;
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     currentvp = gridStateElement(dd, GSS_VP);
     currentgp = gridStateElement(dd, GSS_GPAR);
     PROTECT(prevloc = gridStateElement(dd, GSS_PREVLOC));
@@ -1739,7 +1730,7 @@ SEXP L_lines(SEXP x, SEXP y, SEXP index, SEXP arrow)
     SEXP currentvp, currentgp;
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     currentvp = gridStateElement(dd, GSS_VP);
     currentgp = gridStateElement(dd, GSS_GPAR);
     getViewportTransform(currentvp, dd, 
@@ -1841,7 +1832,7 @@ SEXP gridXspline(SEXP x, SEXP y, SEXP s, SEXP o, SEXP a, SEXP rep, SEXP index,
     double ymax = -DOUBLE_XMAX;
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     currentvp = gridStateElement(dd, GSS_VP);
     currentgp = gridStateElement(dd, GSS_GPAR);
     getViewportTransform(currentvp, dd, 
@@ -2028,7 +2019,7 @@ SEXP L_segments(SEXP x0, SEXP y0, SEXP x1, SEXP y1, SEXP arrow)
     SEXP currentvp, currentgp;
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     currentvp = gridStateElement(dd, GSS_VP);
     currentgp = gridStateElement(dd, GSS_GPAR);
     getViewportTransform(currentvp, dd, 
@@ -2145,7 +2136,7 @@ SEXP L_arrows(SEXP x1, SEXP x2, SEXP xnm1, SEXP xn,
     SEXP devloc = R_NilValue; /* -Wall */
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     currentvp = gridStateElement(dd, GSS_VP);
     currentgp = gridStateElement(dd, GSS_GPAR);
     getViewportTransform(currentvp, dd, 
@@ -2246,7 +2237,7 @@ SEXP L_polygon(SEXP x, SEXP y, SEXP index)
     SEXP currentvp, currentgp;
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     currentvp = gridStateElement(dd, GSS_VP);
     currentgp = gridStateElement(dd, GSS_GPAR);
     getViewportTransform(currentvp, dd, 
@@ -2325,7 +2316,7 @@ static SEXP gridCircle(SEXP x, SEXP y, SEXP r,
     double edgex, edgey;
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     currentvp = gridStateElement(dd, GSS_VP);
     currentgp = gridStateElement(dd, GSS_GPAR);
     getViewportTransform(currentvp, dd, 
@@ -2471,7 +2462,7 @@ static SEXP gridRect(SEXP x, SEXP y, SEXP w, SEXP h,
     double ymax = -DOUBLE_XMAX;
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     currentvp = gridStateElement(dd, GSS_VP);
     currentgp = gridStateElement(dd, GSS_GPAR);
     getViewportTransform(currentvp, dd, 
@@ -2716,7 +2707,7 @@ static SEXP gridText(SEXP label, SEXP x, SEXP y, SEXP hjust, SEXP vjust,
     SEXP currentvp, currentgp;
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     currentvp = gridStateElement(dd, GSS_VP);
     currentgp = gridStateElement(dd, GSS_GPAR);
     getViewportTransform(currentvp, dd, 
@@ -2817,7 +2808,9 @@ static SEXP gridText(SEXP label, SEXP x, SEXP y, SEXP hjust, SEXP vjust,
 				   &gc, dd);
 		    else
 			GEText(xx[i], yy[i], 
-			       translateChar(STRING_ELT(txt, i % LENGTH(txt))), 
+			       CHAR(STRING_ELT(txt, i % LENGTH(txt))),
+			       (gc.fontface == 5) ? CE_SYMBOL :
+			       getCharCE(STRING_ELT(txt, i % LENGTH(txt))),
 			       REAL(hjust)[i % LENGTH(hjust)], 
 			       REAL(vjust)[i % LENGTH(vjust)], 
 			       numeric(rot, i % LENGTH(rot)) + 
@@ -2946,7 +2939,7 @@ SEXP L_points(SEXP x, SEXP y, SEXP pch, SEXP size)
     SEXP currentvp, currentgp;
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     currentvp = gridStateElement(dd, GSS_VP);
     currentgp = gridStateElement(dd, GSS_GPAR);
     getViewportTransform(currentvp, dd, 
@@ -2977,7 +2970,7 @@ SEXP L_points(SEXP x, SEXP y, SEXP pch, SEXP size)
 	    /* FIXME:  The symbols will not respond to viewport
 	     * rotations !!!
 	     */
-	    int ipch;
+	    int ipch = NA_INTEGER /* -Wall */;
 	    gcontextFromgpar(currentgp, i, &gc, dd);
 	    symbolSize = transformWidthtoINCHES(size, i, vpc, &gc,
 						vpWidthCM, vpHeightCM, dd);
@@ -2986,15 +2979,17 @@ SEXP L_points(SEXP x, SEXP y, SEXP pch, SEXP size)
 	    symbolSize = toDeviceWidth(symbolSize, GE_INCHES, dd);
 	    if (R_FINITE(symbolSize)) {
 	        if (isString(pch)) {
-		    ipch = CHAR(STRING_ELT(pch, i % npch))[0];
-		    /*
-		     * special case for pch = "."
-		     */
-		    if (ipch == 46) 
-		      symbolSize = gpCex(currentgp, i);
-		} else {
+		    ipch = GEstring_to_pch(STRING_ELT(pch, i % npch));
+		} else if (isInteger(pch)) {
 		    ipch = INTEGER(pch)[i % npch];
-		}
+		} else if (isReal(pch)) {
+		    ipch = R_FINITE(REAL(pch)[i % npch]) ? 
+			REAL(pch)[i % npch] : NA_INTEGER;
+		} else error(_("invalid plotting symbol"));
+		/*
+		 * special case for pch = "."
+		 */
+		if (ipch == 46) symbolSize = gpCex(currentgp, i);
 	        GESymbol(xx[i], yy[i], ipch, symbolSize, &gc, dd);
 	    }
 	}
@@ -3014,7 +3009,7 @@ SEXP L_clip(SEXP x, SEXP y, SEXP w, SEXP h, SEXP hjust, SEXP vjust)
     SEXP currentvp, currentgp, currentClip;
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     currentvp = gridStateElement(dd, GSS_VP);
     currentgp = gridStateElement(dd, GSS_GPAR);
     getViewportTransform(currentvp, dd, 
@@ -3123,13 +3118,15 @@ SEXP L_pretty(SEXP scale) {
  * 
  * The answer is in INCHES
  */
+
 SEXP L_locator() {
     double x = 0;
     double y = 0;
     SEXP answer;
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
+    GEMode(2, dd);
     PROTECT(answer = allocVector(REALSXP, 2));
     /*
      * Get a mouse click
@@ -3143,6 +3140,7 @@ SEXP L_locator() {
 	REAL(answer)[1] = NA_REAL;	
     }
     UNPROTECT(1);
+    GEMode(0, dd);
     return answer;
 }
 
@@ -3180,7 +3178,7 @@ SEXP L_locnBounds(SEXP x, SEXP y, SEXP theta)
     double edgex, edgey;
     /* Get the current device 
      */
-    GEDevDesc *dd = getDevice();
+    pGEDevDesc dd = getDevice();
     currentvp = gridStateElement(dd, GSS_VP);
     currentgp = gridStateElement(dd, GSS_GPAR);
     getViewportTransform(currentvp, dd, 
