@@ -41,7 +41,8 @@ int gridRegisterIndex;
  * GSS_ENGINERECORDING 13 = are we already inside a .Call.graphics call?
  *   Used by grid.Call.graphics to avoid unnecessary recording on 
  *   engine display list
- * GSS_ASK 14 = should we prompt the user before starting a new page?
+ * [GSS_ASK 14 = should we prompt the user before starting a new page?
+ *  Replaced by per-device setting as from R 2.7.0.]
  * GSS_SCALE 15 = a scale or "zoom" factor for all output 
  *   (to support "fit to window" resizing on windows device)
  * 
@@ -54,7 +55,7 @@ SEXP createGridSystemState()
     return allocVector(VECSXP, 16);
 }
 
-void initDL(GEDevDesc *dd)
+void initDL(pGEDevDesc dd)
 {
     SEXP dl, dlindex;
     SEXP vp = gridStateElement(dd, GSS_VP);
@@ -78,7 +79,7 @@ void initDL(GEDevDesc *dd)
  * nor is the ROOT viewport (see initVP),
  * nor is the current gpar (see initGP)
  */
-void initOtherState(GEDevDesc* dd)
+void initOtherState(pGEDevDesc dd)
 {
     SEXP currloc, prevloc, recording;
     SEXP state = (SEXP) dd->gesd[gridRegisterIndex]->systemSpecific;
@@ -94,11 +95,12 @@ void initOtherState(GEDevDesc* dd)
     SET_VECTOR_ELT(state, GSS_ENGINERECORDING, recording);
 }
 
-void fillGridSystemState(SEXP state, GEDevDesc* dd) 
+void fillGridSystemState(SEXP state, pGEDevDesc dd) 
 {
-    SEXP devsize, currloc, prevloc, dlon, enginedlon, recording;
-    SEXP griddev, gridask, gridscale;
-    PROTECT(devsize = allocVector(REALSXP, 2));
+    SEXP devsize, currloc, prevloc;
+
+    PROTECT(state);
+    devsize = allocVector(REALSXP, 2);
     REAL(devsize)[0] = 0;
     REAL(devsize)[1] = 0;
     SET_VECTOR_ELT(state, GSS_DEVSIZE, devsize);
@@ -106,24 +108,18 @@ void fillGridSystemState(SEXP state, GEDevDesc* dd)
      * Initial setting relies on the fact that all values sent to devices
      * are in INCHES;  so (0, 0) is the bottom-left corner of the device.
      */
-    PROTECT(currloc = allocVector(REALSXP, 2));
+    currloc = allocVector(REALSXP, 2);
     REAL(currloc)[0] = NA_REAL;
     REAL(currloc)[1] = NA_REAL;    
     SET_VECTOR_ELT(state, GSS_CURRLOC, currloc);
-    PROTECT(prevloc = allocVector(REALSXP, 2));
+    prevloc = allocVector(REALSXP, 2);
     REAL(prevloc)[0] = NA_REAL;
     REAL(prevloc)[1] = NA_REAL;    
     SET_VECTOR_ELT(state, GSS_PREVLOC, prevloc);
-    PROTECT(dlon = allocVector(LGLSXP, 1));
-    LOGICAL(dlon)[0] = TRUE;
-    SET_VECTOR_ELT(state, GSS_DLON, dlon);
-    PROTECT(enginedlon = allocVector(LGLSXP, 1));
-    LOGICAL(enginedlon)[0] = TRUE;
-    SET_VECTOR_ELT(state, GSS_ENGINEDLON, enginedlon);
+    SET_VECTOR_ELT(state, GSS_DLON, ScalarLogical(TRUE));
+    SET_VECTOR_ELT(state, GSS_ENGINEDLON, ScalarLogical(TRUE));
     SET_VECTOR_ELT(state, GSS_CURRGROB, R_NilValue);
-    PROTECT(recording = allocVector(LGLSXP, 1));    
-    LOGICAL(recording)[0] = FALSE;
-    SET_VECTOR_ELT(state, GSS_ENGINERECORDING, recording);
+    SET_VECTOR_ELT(state, GSS_ENGINERECORDING, ScalarLogical(FALSE));
     initGPar(dd);
     SET_VECTOR_ELT(state, GSS_GPSAVED, R_NilValue);
     /* Do NOT initialise top-level viewport or grid display list for
@@ -132,25 +128,21 @@ void fillGridSystemState(SEXP state, GEDevDesc* dd)
     SET_VECTOR_ELT(state, GSS_GLOBALINDEX, R_NilValue);
     /* Note that no grid output has occurred on the device yet.
      */
-    PROTECT(griddev = allocVector(LGLSXP, 1));
-    LOGICAL(griddev)[0] = FALSE;
-    SET_VECTOR_ELT(state, GSS_GRIDDEVICE, griddev);
-    PROTECT(gridask = allocVector(LGLSXP, 1));
-    LOGICAL(gridask)[0] = FALSE;
-    SET_VECTOR_ELT(state, GSS_ASK, gridask);
-    PROTECT(gridscale = allocVector(REALSXP, 1));
-    REAL(gridscale)[0] = 1.0;
-    SET_VECTOR_ELT(state, GSS_SCALE, gridscale);
-    UNPROTECT(9);
+    SET_VECTOR_ELT(state, GSS_GRIDDEVICE, ScalarLogical(FALSE));
+#if 0
+    SET_VECTOR_ELT(state, GSS_ASK, ScalarLogical(dd->ask));
+#endif
+    SET_VECTOR_ELT(state, GSS_SCALE, ScalarReal(1.0));
+    UNPROTECT(1);
 }
 
-SEXP gridStateElement(GEDevDesc *dd, int elementIndex)
+SEXP gridStateElement(pGEDevDesc dd, int elementIndex)
 {
     return VECTOR_ELT((SEXP) dd->gesd[gridRegisterIndex]->systemSpecific, 
 		      elementIndex);
 }
 
-void setGridStateElement(GEDevDesc *dd, int elementIndex, SEXP value)
+void setGridStateElement(pGEDevDesc dd, int elementIndex, SEXP value)
 {
     SET_VECTOR_ELT((SEXP) dd->gesd[gridRegisterIndex]->systemSpecific, 
 		   elementIndex, value);
@@ -192,7 +184,7 @@ static void globaliseState(SEXP state)
     UNPROTECT(2);
 }
 
-SEXP gridCallback(GEevent task, GEDevDesc *dd, SEXP data) {
+SEXP gridCallback(GEevent task, pGEDevDesc dd, SEXP data) {
     SEXP result = R_NilValue;
     SEXP valid, scale;
     SEXP gridState;
