@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 2001-3 Paul Murrell
- *                2003-5 The R Development Core Team
+ *                2003-8 The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,13 +13,15 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  A copy of the GNU General Public License is available via WWW at
- *  http://www.gnu.org/copyleft/gpl.html.  You can also obtain it by
- *  writing to the Free Software Foundation, Inc., 51 Franklin Street
- *  Fifth Floor, Boston, MA 02110-1301  USA.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 #include "grid.h"
+#include <math.h>
+#include <float.h>
+#include <string.h>
 
 int isUnitArithmetic(SEXP ua) {
     return inherits(ua, "unit.arithmetic");
@@ -35,15 +37,12 @@ int isUnitList(SEXP ul) {
 SEXP unit(double value, int unit) 
 {
     SEXP u, units, classname;
-    PROTECT(u = allocVector(REALSXP, 1));
-    REAL(u)[0] = value;
-    PROTECT(units = allocVector(INTSXP, 1));
-    INTEGER(units)[0] = unit;
+    PROTECT(u = ScalarReal(value));
+    PROTECT(units = ScalarInteger(unit));
     /* NOTE that we do not set the "unit" attribute */
     setAttrib(u, install("valid.unit"), units);
     setAttrib(u, install("data"), R_NilValue);
-    PROTECT(classname = allocVector(STRSXP, 1));
-    SET_STRING_ELT(classname, 0, mkChar("unit"));
+    PROTECT(classname = mkString("unit"));
     classgets(u, classname);
     UNPROTECT(3);
     return u;
@@ -91,7 +90,7 @@ SEXP unitData(SEXP unit, int index) {
 
 /* Accessor functions for unit arithmetic object
  */
-char* fName(SEXP ua) {
+const char* fName(SEXP ua) {
     return CHAR(STRING_ELT(getListElement(ua, "fname"), 0));
 }
 
@@ -271,9 +270,9 @@ double pureNullUnitValue(SEXP unit, int index)
     return result;
 }
 
-int pureNullUnitArithmetic(SEXP unit, int index, GEDevDesc *dd);
+int pureNullUnitArithmetic(SEXP unit, int index, pGEDevDesc dd);
 
-int pureNullUnit(SEXP unit, int index, GEDevDesc *dd) {
+int pureNullUnit(SEXP unit, int index, pGEDevDesc dd) {
     int result;
     if (isUnitArithmetic(unit)) 
 	result = pureNullUnitArithmetic(unit, index, dd);
@@ -386,7 +385,7 @@ int pureNullUnit(SEXP unit, int index, GEDevDesc *dd) {
     return result;    
 }
 
-int pureNullUnitArithmetic(SEXP unit, int index, GEDevDesc *dd) {
+int pureNullUnitArithmetic(SEXP unit, int index, pGEDevDesc dd) {
     /* 
      * Initialised to shut up compiler
      */
@@ -435,7 +434,7 @@ double evaluateGrobUnit(double value, SEXP grob,
 			 * 0 = x, 1 = y, 2 = width, 3 = height
 			 */
 			int evalType,
-			GEDevDesc *dd) 
+			pGEDevDesc dd) 
 {
     double vpWidthCM, vpHeightCM;
     double rotationAngle;
@@ -560,8 +559,7 @@ double evaluateGrobUnit(double value, SEXP grob,
 	 */
 	{
 	    SEXP val;
-	    PROTECT(val = allocVector(REALSXP, 1));
-	    REAL(val)[0] = value;
+	    PROTECT(val = ScalarReal(value));
 	    PROTECT(R_fcall2x = lang3(evalFnx, grob, val));
 	    PROTECT(unitx = eval(R_fcall2x, R_gridEvalEnv));
 	    PROTECT(R_fcall2y = lang3(evalFny, grob, val));
@@ -676,7 +674,7 @@ double evaluateGrobUnit(double value, SEXP grob,
 double evaluateGrobXUnit(double value, SEXP grob, 
 			 double vpheightCM, double vpwidthCM,
 			 int nullLMode, int nullAMode,
-			 GEDevDesc *dd) 
+			 pGEDevDesc dd) 
 {
     return evaluateGrobUnit(value, grob, vpheightCM, vpwidthCM, 
 			    nullLMode, nullAMode, 0, dd);
@@ -685,7 +683,7 @@ double evaluateGrobXUnit(double value, SEXP grob,
 double evaluateGrobYUnit(double value, SEXP grob, 
 			 double vpheightCM, double vpwidthCM,
 			 int nullLMode, int nullAMode,
-			 GEDevDesc *dd) 
+			 pGEDevDesc dd) 
 {
     return evaluateGrobUnit(value, grob, vpheightCM, vpwidthCM, 
 			    nullLMode, nullAMode, 1, dd);
@@ -694,7 +692,7 @@ double evaluateGrobYUnit(double value, SEXP grob,
 double evaluateGrobWidthUnit(SEXP grob, 
 			     double vpheightCM, double vpwidthCM,
 			     int nullLMode, int nullAMode,
-			     GEDevDesc *dd) 
+			     pGEDevDesc dd) 
 {
     return evaluateGrobUnit(1, grob, vpheightCM, vpwidthCM, 
 			    nullLMode, nullAMode, 2, dd);
@@ -703,7 +701,7 @@ double evaluateGrobWidthUnit(SEXP grob,
 double evaluateGrobHeightUnit(SEXP grob, 
 			     double vpheightCM, double vpwidthCM,
 			     int nullLMode, int nullAMode,
-			     GEDevDesc *dd) 
+			     pGEDevDesc dd) 
 {
     return evaluateGrobUnit(1, grob, vpheightCM, vpwidthCM, 
 			    nullLMode, nullAMode, 3, dd);
@@ -723,9 +721,9 @@ double evaluateGrobHeightUnit(SEXP grob,
  */
 double transform(double value, int unit, SEXP data,
 		 double scalemin, double scalemax,
-		 R_GE_gcontext *gc,
+		 const pGEcontext gc,
 		 double thisCM, double otherCM,
-		 int nullLMode, int nullAMode, GEDevDesc *dd)
+		 int nullLMode, int nullAMode, pGEDevDesc dd)
 {
     double result = value;
     switch (unit) {
@@ -790,8 +788,10 @@ double transform(double value, int unit, SEXP data,
 		fromDeviceWidth(GEExpressionWidth(VECTOR_ELT(data, 0), gc, dd),
 				GE_INCHES, dd);
 	else
+	    /* FIXME: what encoding is this? */
 	    result = result*
-		fromDeviceWidth(GEStrWidth(CHAR(STRING_ELT(data, 0)), gc, dd),
+		fromDeviceWidth(GEStrWidth(CHAR(STRING_ELT(data, 0)), 
+					   CE_ANY, gc, dd),
 				GE_INCHES, dd);
 	break;
     case L_STRINGHEIGHT:
@@ -802,8 +802,9 @@ double transform(double value, int unit, SEXP data,
 						    gc, dd),
 				 GE_INCHES, dd);
 	else
+	    /* FIXME: what encoding is this? */
 	    result = result*
-		fromDeviceHeight(GEStrHeight(CHAR(STRING_ELT(data, 0)),
+		fromDeviceHeight(GEStrHeight(CHAR(STRING_ELT(data, 0)), -1,
 					     gc, dd),
 				 GE_INCHES, dd);
 	break;
@@ -859,9 +860,9 @@ double transform(double value, int unit, SEXP data,
 /* FIXME:  scales are only linear at the moment */
 double transformLocation(double location, int unit, SEXP data,
 			 double scalemin, double scalemax,
-			 R_GE_gcontext *gc,
+			 const pGEcontext gc,
 			 double thisCM, double otherCM,
-			 int nullLMode, int nullAMode, GEDevDesc *dd)
+			 int nullLMode, int nullAMode, pGEDevDesc dd)
 {
     double result = location;
     switch (unit) {
@@ -877,15 +878,15 @@ double transformLocation(double location, int unit, SEXP data,
 
 double transformXArithmetic(SEXP x, int index,
 			    LViewportContext vpc,
-			    R_GE_gcontext *gc,
+			    const pGEcontext gc,
 			    double widthCM, double heightCM,
-			    int nullLMode, GEDevDesc *dd);
+			    int nullLMode, pGEDevDesc dd);
 
 double transformX(SEXP x, int index,
 		  LViewportContext vpc,
-		  R_GE_gcontext *gc,
+		  const pGEcontext gc,
 		  double widthCM, double heightCM,
-		  int nullLMode, int nullAMode, GEDevDesc *dd)
+		  int nullLMode, int nullAMode, pGEDevDesc dd)
 {
     double result;
     int unit;
@@ -919,15 +920,15 @@ double transformX(SEXP x, int index,
 
 double transformYArithmetic(SEXP y, int index,
 			    LViewportContext vpc,
-			    R_GE_gcontext *gc,
+			    const pGEcontext gc,
 			    double widthCM, double heightCM,
-			    int nullLMode, GEDevDesc *dd);
+			    int nullLMode, pGEDevDesc dd);
 
 double transformY(SEXP y, int index, 
 		  LViewportContext vpc,
-		  R_GE_gcontext *gc,
+		  const pGEcontext gc,
 		  double widthCM, double heightCM,
-		  int nullLMode, int nullAMode, GEDevDesc *dd)
+		  int nullLMode, int nullAMode, pGEDevDesc dd)
 {
     double result;
     int unit;
@@ -961,10 +962,10 @@ double transformY(SEXP y, int index,
 
 double transformDimension(double dim, int unit, SEXP data,
 			  double scalemin, double scalemax,
-			  R_GE_gcontext *gc,
+			  const pGEcontext gc,
 			  double thisCM, double otherCM,
 			  int nullLMode, int nullAMode,
-			  GEDevDesc *dd)
+			  pGEDevDesc dd)
 {
     double result = dim;
     switch (unit) {
@@ -980,15 +981,15 @@ double transformDimension(double dim, int unit, SEXP data,
 
 double transformWidthArithmetic(SEXP width, int index,
 				LViewportContext vpc,
-				R_GE_gcontext *gc,
+				const pGEcontext gc,
 				double widthCM, double heightCM,
-				int nullLMode, GEDevDesc *dd);
+				int nullLMode, pGEDevDesc dd);
 
 double transformWidth(SEXP width, int index, 
 		      LViewportContext vpc,
-		      R_GE_gcontext *gc,
+		      const pGEcontext gc,
 		      double widthCM, double heightCM,
-		      int nullLMode, int nullAMode, GEDevDesc *dd)
+		      int nullLMode, int nullAMode, pGEDevDesc dd)
 {
     double result;
     int unit;
@@ -1022,15 +1023,15 @@ double transformWidth(SEXP width, int index,
 
 double transformHeightArithmetic(SEXP height, int index,
 				 LViewportContext vpc,
-				 R_GE_gcontext *gc,
+				 const pGEcontext gc,
 				 double widthCM, double heightCM,
-				 int nullLMode, GEDevDesc *dd);
+				 int nullLMode, pGEDevDesc dd);
 
 double transformHeight(SEXP height, int index,
 		       LViewportContext vpc,
-		       R_GE_gcontext *gc,
+		       const pGEcontext gc,
 		       double widthCM, double heightCM,
-		       int nullLMode, int nullAMode, GEDevDesc *dd)
+		       int nullLMode, int nullAMode, pGEDevDesc dd)
 {
     double result;
     int unit;
@@ -1064,9 +1065,9 @@ double transformHeight(SEXP height, int index,
 
 double transformXArithmetic(SEXP x, int index,
 			    LViewportContext vpc,
-			    R_GE_gcontext *gc,
+			    const pGEcontext gc,
 			    double widthCM, double heightCM,
-			    int nullLMode, GEDevDesc *dd)
+			    int nullLMode, pGEDevDesc dd)
 {
     int i;
     double result = 0;
@@ -1144,9 +1145,9 @@ double transformXArithmetic(SEXP x, int index,
 
 double transformYArithmetic(SEXP y, int index,
 			    LViewportContext vpc,
-			    R_GE_gcontext *gc,
+			    const pGEcontext gc,
 			    double widthCM, double heightCM,
-			    int nullLMode, GEDevDesc *dd)
+			    int nullLMode, pGEDevDesc dd)
 {
     int i;
     double result = 0;
@@ -1224,9 +1225,9 @@ double transformYArithmetic(SEXP y, int index,
 
 double transformWidthArithmetic(SEXP width, int index,
 				LViewportContext vpc,
-				R_GE_gcontext *gc,
+				const pGEcontext gc,
 				double widthCM, double heightCM,
-				int nullLMode, GEDevDesc *dd)
+				int nullLMode, pGEDevDesc dd)
 {
     int i;
     double result = 0;
@@ -1304,9 +1305,9 @@ double transformWidthArithmetic(SEXP width, int index,
 
 double transformHeightArithmetic(SEXP height, int index,
 				 LViewportContext vpc,
-				 R_GE_gcontext *gc,
+				 const pGEcontext gc,
 				 double widthCM, double heightCM,
-				 int nullLMode, GEDevDesc *dd)
+				 int nullLMode, pGEDevDesc dd)
 {
     int i;
     double result = 0;
@@ -1409,9 +1410,9 @@ double transformHeightArithmetic(SEXP height, int index,
  */
 double transformXtoINCHES(SEXP x, int index, 
 			  LViewportContext vpc, 
-			  R_GE_gcontext *gc,
+			  const pGEcontext gc,
 			  double widthCM, double heightCM,
-			  GEDevDesc *dd)
+			  pGEDevDesc dd)
 {
     return transformX(x, index, vpc, gc,
 		      widthCM, heightCM, 0, 0, dd);
@@ -1419,9 +1420,9 @@ double transformXtoINCHES(SEXP x, int index,
 
 double transformYtoINCHES(SEXP y, int index, 
 			  LViewportContext vpc,
-			  R_GE_gcontext *gc,
+			  const pGEcontext gc,
 			  double widthCM, double heightCM,
-			  GEDevDesc *dd)
+			  pGEDevDesc dd)
 {
     return transformY(y, index, vpc, gc,
 		      widthCM, heightCM, 0, 0, dd);
@@ -1429,9 +1430,9 @@ double transformYtoINCHES(SEXP y, int index,
 
 void transformLocn(SEXP x, SEXP y, int index, 
 		   LViewportContext vpc,
-		   R_GE_gcontext *gc,
+		   const pGEcontext gc,
 		   double widthCM, double heightCM,
-		   GEDevDesc *dd,
+		   pGEDevDesc dd,
 		   LTransform t,
 		   double *xx, double *yy)
 {
@@ -1451,9 +1452,9 @@ void transformLocn(SEXP x, SEXP y, int index,
 
 double transformWidthtoINCHES(SEXP w, int index,
 			      LViewportContext vpc,
-			      R_GE_gcontext *gc,
+			      const pGEcontext gc,
 			      double widthCM, double heightCM,
-			      GEDevDesc *dd)
+			      pGEDevDesc dd)
 {
     return transformWidth(w, index, vpc, gc,
 			  widthCM, heightCM, 0, 0, dd);
@@ -1461,9 +1462,9 @@ double transformWidthtoINCHES(SEXP w, int index,
 
 double transformHeighttoINCHES(SEXP h, int index,
 			       LViewportContext vpc,
-			       R_GE_gcontext *gc,
+			       const pGEcontext gc,
 			       double widthCM, double heightCM,
-			       GEDevDesc *dd)
+			       pGEDevDesc dd)
 {
     return transformHeight(h, index, vpc, gc,
 			   widthCM, heightCM, 0, 0, dd);
@@ -1471,9 +1472,9 @@ double transformHeighttoINCHES(SEXP h, int index,
 
 void transformDimn(SEXP w, SEXP h, int index, 
 		   LViewportContext vpc,
-		   R_GE_gcontext *gc,
+		   const pGEcontext gc,
 		   double widthCM, double heightCM,
-		   GEDevDesc *dd,
+		   pGEDevDesc dd,
 		   double rotationAngle,
 		   double *ww, double *hh)
 {
@@ -1502,9 +1503,9 @@ void transformDimn(SEXP w, SEXP h, int index,
  */
 
 double transformFromINCHES(double value, int unit, 
-			   R_GE_gcontext *gc,
+			   const pGEcontext gc,
 			   double thisCM, double otherCM,
-			   GEDevDesc *dd)
+			   pGEDevDesc dd)
 {
     /*      
      * Convert to NPC
@@ -1615,9 +1616,9 @@ double transformFromINCHES(double value, int unit,
  */
 double transformXYFromINCHES(double location, int unit, 
 			     double scalemin, double scalemax,
-			     R_GE_gcontext *gc,
+			     const pGEcontext gc,
 			     double thisCM, double otherCM,
-			     GEDevDesc *dd)
+			     pGEDevDesc dd)
 {
     double result = location;
     switch (unit) {
@@ -1633,9 +1634,9 @@ double transformXYFromINCHES(double location, int unit,
 
 double transformWidthHeightFromINCHES(double dimension, int unit, 
 				      double scalemin, double scalemax,
-				      R_GE_gcontext *gc,
+				      const pGEcontext gc,
 				      double thisCM, double otherCM,
-				      GEDevDesc *dd)
+				      pGEDevDesc dd)
 {
     double result = dimension;
     switch (unit) {
