@@ -145,12 +145,9 @@ setGeneric <-
     }
     assign(name, fdef, where)
     .cacheGeneric(name, fdef)
-    if(!.UsingMethodsTables() &&
-       length(fdef@group)> 0 && !is.null(getGeneric(fdef@group[[1]], where = where)))
-        methods <- getAllMethods(name, fdef, where)
-    else
-        methods <- fdef@default # empty or containing the default
-    assignMethodsMetaData(name, methods, fdef, where)
+    methods <- fdef@default # empty or containing the default
+    assignMethodsMetaData(name, methods, fdef, where) # MethodsList, eventually will go away
+    .assignMethodsTableMetaData(name, fdef, where)
     name
 }
 
@@ -167,27 +164,32 @@ isGeneric <-
   function(f, where = topenv(parent.frame()), fdef = NULL, getName = FALSE)
 {
     if(is.character(f) && f %in% c("as.double", "as.real")) f <- "as.numeric"
-    if(is.null(fdef))
+    if(is.null(fdef) && missing(where))
         fdef <- .getGenericFromCache(f, where)
     if(is.null(fdef))
         fdef <- getFunction(f, where=where, mustFind = FALSE)
     if(is.null(fdef))
       return(FALSE)
-    ## check primitives. These are never stored as explicit generic functions.
-    ## The definition of isGeneric for them is that methods metadata exists,
-    ## either on this database or anywhere (where == -1)
-    if(!identical(typeof(fdef), "closure"))
-      return(exists(mlistMetaName(f, "base"))) # all primitives are on package base
+    ## check primitives. These are never found as explicit generic functions.
+    if(is.primitive(fdef)) {
+        ## the definition of isGeneric() for a primitive is that methods are defined
+        ## (other than the default primitive)
+        gen <- genericForPrimitive(f)
+        return(is.function(gen) && length(objects(.getMethodsTable(gen), all=TRUE)) > 1)
+    }
     if(!is(fdef, "genericFunction"))
         return(FALSE)
-    gen <- fdef@generic
-    if(getName)
-        return(gen)
-    else if(missing(f) || .identC(gen, f))
-        return(TRUE)
+    gen <- fdef@generic # the name with package attribute
+    if(missing(f) || .identC(gen, f)) {
+	if(getName)
+	    gen
+	else
+	    TRUE
+    }
     else {
-        warning(gettextf("function \"%s\" appears to be a generic function, but with generic name \"%s\"", f, gen), domain = NA)
-        return(FALSE)
+        warning(gettextf("function \"%s\" appears to be a generic function, but with generic name \"%s\"",
+                         f, gen), domain = NA)
+        FALSE
     }
 }
 
@@ -208,8 +210,10 @@ removeGeneric <-
     }
     found <- is(fdef, "genericFunction")
     if(found) {
-        removeMethodsObject(f, where)
-        .removeMethodsMetaTable(fdef, where)
+         .removeMethodsMetaTable(fdef, where)
+         oldMetaName <- methodsPackageMetaName("M",fdef@generic, fdef@package)
+         if(exists(oldMetaName, where, inherits = FALSE))
+           rm(list = oldMetaName, pos = where)
         .uncacheGeneric(f, fdef)
         rm(list = fdef@generic, pos = where)
     }
@@ -231,60 +235,48 @@ getMethods <-
     ##
     ## The `where' argument optionally says where to look for the function, if
     ## `f' is given as the name.
-    ## Methods objects are kept during the session in a special environment.
-    ## Inside this environment, individual methods are added and updated as
-    ## they are found (for example, a method that is inherited is stored,
-    ## again, under the actual signature).  These updates speed up the
-    ## method search; however, they are not stored with the original generic
-    ## function, since they might change in future sessions.
+    ## This function returns a MethodsList object, no longer used for method dispatch
+    ## A better structure for most purposes is the linear methods list returned by findMethods()
+    ## There are no plans currently to make getMethods defunct, but it will be less
+    ## efficient than findMethods()  both for creating the object and using it.
 
-  ## Note for methods tables.  The function getMethods() continues to
-  ## return a methods list object, even when methods tables are being
-  ## used for dispatch, but now this is the metadata from where, and
-  ## _not_ a fully merged list, since avoiding this computation ws a
-  ## major motivation for the tables.
+  ##  The function getMethods continues to
+  ## return a methods list object, but now this is the metadata from where,
+  ## or is converted from the internal table if where is missing.
 
     function(f, where = topenv(parent.frame()))
 {
-    nowhere <- missing(where) # remember it now, R changes the truth later!
+    nowhere <- missing(where)
     if(is.character(f))
         fdef <- getGeneric(f, where = where)
     else if(is(f, "genericFunction")) {
-        if(nowhere)
-          where <- .genEnv(f)
         fdef <- f
         f <- fdef@generic
     }
     else
-        stop(gettextf("invalid argument \"f\", expected a function or its name, got an object of class \"%s\"", class(f)), domain = NA)
-    if(!is.null(fdef)) { # else NULL
-        ## getMethods() always returns a methods  list.  When using
-        ## methods tables, this only makes sense as the metadata from
-        ## where (by default, the location of the generic)
-        if(.UsingMethodsTables()) {
-            gwhere <-  ( if(nowhere)
-                        findFunction(f, TRUE, where)[[1]] # can't be empty since fdef is nonnull
-                        else where)
-            value <- getMethodsMetaData(f, where = gwhere)
-            if(is.null(value)) # return empty methods list
-              new("MethodsList", argument = fdef@default@argument)
+        stop(gettextf("invalid argument \"f\", expected a function or its name, got an object of class \"%s\"",
+                      class(f)), domain = NA)
+    if(!is.null(fdef)) {
+        if(nowhere) {
+            if(is(fdef, "genericFunction"))
+              value <- .makeMlistFromTable(fdef)
             else
-              value
+              value <- NULL
         }
         else
-          getMethodsForDispatch(f, fdef)
+          value <- getMethodsMetaData(f, where = where)
+        if(is.null(value))              # return empty methods list
+            new("MethodsList", argument = fdef@default@argument)
+        else
+            value
     }
+    else
+      NULL
 }
 
-getMethodsForDispatch <- function(f, fdef)
+getMethodsForDispatch <- function(fdef)
 {
-  ev <- environment(fdef)
-    if(.UsingMethodsTables())
-        .getMethodsTable(fdef, ev)
-    else {
-        if(exists(".Methods", envir = ev, inherits = FALSE))
-            get(".Methods", envir = ev) ## else NULL
-    }
+    .getMethodsTable(fdef, environment(fdef))
 }
 
 ## some functions used in MethodsListSelect, that must be safe against recursive
@@ -309,7 +301,7 @@ getMethodsForDispatch <- function(f, fdef)
 
 ##NB used internally in MethodsListSelect.  Must NOT use the standard version
 ## to prevent recursion
-.getMethodsForDispatch <- function(f, fdef) {
+.getMethodsForDispatch <- function(fdef) {
     ev <- .evBasic(fdef)
     if(.existsBasic(".Methods", envir = ev)) {
         .getBasic(".Methods", envir = ev)
@@ -330,26 +322,16 @@ cacheMethod <-
   ## cache the given definition in the method metadata for f
   ## Support function:  DON'T USE DIRECTLY (does no checking)
   function(f, sig, def, args = names(sig), fdef, inherited = FALSE) {
-    ev <- environment(fdef)
-    if(.UsingMethodsTables()) {
-        .cacheMethodInTable(fdef, sig, def,
-                            .getMethodsTable(fdef, ev, inherited = inherited))
-        ## if this is not an inherited method, update the inherited table as well
-        ## TODO:  in this case, should uncache inherited methods, though the callin
-        ##  function will normally have done this.
-        if(!inherited)
-          .cacheMethodInTable(fdef, sig, def,
-                              .getMethodsTable(fdef, ev, inherited = TRUE))
-    }
-    else {
-      methods <- get(".Methods", envir = ev)
-      methods <- insertMethod(methods, sig, args, def, TRUE)
-      assign(".Methods", methods, envir = ev)
-      deflt <- finalDefaultMethod(methods)
-      if(is.primitive(deflt))
-        setPrimitiveMethods(f, deflt, "set", fdef, methods)
-      methods
-    }
+      ev <- environment(fdef)
+
+      .cacheMethodInTable(fdef, sig, def,
+			  .getMethodsTable(fdef, ev, inherited = inherited))
+      ## if this is not an inherited method, update the inherited table as well
+      ## TODO:	in this case, should uncache inherited methods, though the callin
+      ##  function will normally have done this.
+      if(!inherited)
+	  .cacheMethodInTable(fdef, sig, def,
+			      .getMethodsTable(fdef, ev, inherited = TRUE))
   }
 
 .removeCachedMethod <- function(f, sig, fdef = getGeneric(f))
@@ -459,7 +441,8 @@ setMethod <-
           setGeneric(f) #  turn on this generic and cache it.
     }
     if(isSealedMethod(f, signature, fdef, where=where))
-        stop(gettextf("the method for function \"%s\" and signature %s is sealed and cannot be re-defined", f, .signatureString(fdef, signature)), domain = NA)
+        stop(gettextf("the method for function \"%s\" and signature %s is sealed and cannot be re-defined",
+                      f, .signatureString(fdef, signature)), domain = NA)
     signature <- matchSignature(signature, fdef, where)
     switch(typeof(definition),
            closure = {
@@ -478,7 +461,7 @@ setMethod <-
                }
                if(!identical(mnames, fnames)) {
                    ## omitted classes in method => "missing"
-                   fullSig <- conformMethod(signature, mnames, fnames, f, fdef)
+                   fullSig <- conformMethod(signature, mnames, fnames, f, fdef, definition)
                    if(!identical(fullSig, signature)) {
                        formals(definition, envir = environment(definition)) <- formals(fdef)
                        signature <- fullSig
@@ -498,7 +481,8 @@ setMethod <-
            "NULL" = {
 
            },
-           stop(gettextf("invalid method definition: expected a function, got an object of class \"%s\"", class(definition)), domain = NA)
+           stop(gettextf("invalid method definition: expected a function, got an object of class \"%s\"",
+                         class(definition)), domain = NA)
            )
     fenv <- environment(fdef)
     ## check length against active sig. length, reset if necessary in .addToMetaTable
@@ -510,16 +494,12 @@ setMethod <-
       definition@generic <- fdef@generic
     whereMethods <- .getOrMakeMethodsList(f, where, fdef)
     whereMethods <- insertMethod(whereMethods, signature, margs, definition)
-    allMethods <- getMethodsForDispatch(f, fdef)
-    if(is.environment(allMethods)) {
+    allMethods <- getMethodsForDispatch(fdef)
         ## cache in both direct and inherited tables
       .cacheMethodInTable(fdef, signature, definition, allMethods) #direct
       .cacheMethodInTable(fdef, signature, definition) # inherited, by default
       if(!identical(where, baseenv()))
          .addToMetaTable(fdef, signature, definition,where, nSig)
-    }
-    else # but currently unused (9/06)-- should be deleted after testing
-      allMethods <- insertMethod(allMethods, signature, margs, definition)
     resetGeneric(f, fdef, allMethods, gwhere, deflt) # Note: gwhere not used by resetGeneric
     ## assigns the methodslist object
     ## and deals with flags for primitives & for updating group members
@@ -566,7 +546,12 @@ removeMethod <- function(f, signature = character(), where = topenv(parent.frame
   }
 
 findMethod <- function(f, signature, where = topenv(parent.frame())) {
-    fM <- mlistMetaName(f)
+    fdef <- getGeneric(f, where = where)
+    if(is.null(fdef)) {
+        warning(gettextf("no generic function \"%s\" found", f), domain = NA)
+        return(character())
+    }
+    fM <- .TableMetaName(fdef@generic, fdef@package)
     where <- .findAll(fM, where)
     found <- logical(length(where))
     for(i in seq_along(where)) {
@@ -595,12 +580,8 @@ getMethod <-
 {
     if(!is(fdef, "genericFunction")) # must be the optional case, else an error in getGeneric
         return(NULL)
-    if(missing(mlist)) {
-        if(missing(where))
-            mlist <- getMethodsForDispatch(f, fdef)
-        else
-            mlist <- getMethods(f, where)
-    }
+    if(missing(mlist))
+            mlist <- getMethodsForDispatch(fdef)
     if(is.environment(mlist)) {
         value <- .findMethodInTable(signature, mlist, fdef)
         if(is.null(value) && !optional)
@@ -608,6 +589,8 @@ getMethod <-
                         f, paste(signature, collapse = ", ")))
         return(value)
     }
+    ## the rest of the code will be executed only if a methods list object is supplied
+    ## as an argument.  Should be deprecated from 2.8.0
     i <- 1
     argNames <- fdef@signature
     signature <- matchSignature(signature, fdef)
@@ -652,19 +635,58 @@ dumpMethod <-
   ## Dump the method for this generic function and signature.
   ## The resulting source file will recreate the method.
   function(f, signature=character(), file = defaultDumpName(f, signature),
-           where = -1,
+           where = topenv(parent.frame()),
            def = getMethod(f, signature, where=where, optional = TRUE))
 {
     if(!is.function(def))
         def <- getMethod(f, character(), where=where, optional = TRUE)
-    if(file != "")
-        sink(file)
+
+    ## sink() handling as general as possible -- unbelievably unpretty coding:
+    closeit <- TRUE ; isSTDOUT <- FALSE
+    if (is.character(file)) {
+        if(!(isSTDOUT <- file == "")) ## stdout() -- no sink() needed
+            file <- file(file, "w")
+    }
+    else if (inherits(file, "connection")) {
+	if (!isOpen(file)) open(file, "w") else closeit <- FALSE
+    } else stop("'file' must be a character string or a connection")
+    if(!isSTDOUT){ sink(file); on.exit({sink(); if(closeit) close(file)}) }
+
     cat("setMethod(\"", f, "\", ", deparse(signature), ",\n", sep="")
-    dput(def)
+    dput(def@.Data)
     cat(")\n", sep="")
-    if(file != "")
-        sink()
-    file
+    if(!isSTDOUT) { on.exit(); sink(); if(closeit) close(file) }
+    invisible(file)
+}
+
+dumpMethods <- function(f, file = "", signature = character(), methods,
+                        where = topenv(parent.frame()) )
+{
+    ## Dump all the methods for this generic.
+    ##
+    ## If `signature' is supplied only the methods matching this initial signature
+    ## are dumped.
+    if(missing(methods))
+        methods <- getMethods(f, where = where)@methods
+
+    ## sink() handling as general as possible -- unbelievably unpretty coding:
+    closeit <- TRUE ; isSTDOUT <- FALSE
+    if (is.character(file)) {
+        if(!(isSTDOUT <- file == "")) ## stdout() -- no sink() needed
+            file <- file(file, "w")
+    }
+    else if (inherits(file, "connection")) {
+	if (!isOpen(file)) open(file, "w") else closeit <- FALSE
+    } else stop("'file' must be a character string or a connection")
+    if(!isSTDOUT){ sink(file); on.exit({sink(); if(closeit) close(file)}) }
+
+    for(what in names(methods)) {
+        el <- methods[[what]]
+        if(is.function(el))
+            dumpMethod(f, c(signature, what), file = "", def = el)
+        else
+            dumpMethods(f, "", c(signature, what), el, where)
+    }
 }
 
 
@@ -679,7 +701,7 @@ selectMethod <-
   ## mlist = Optional MethodsList object to use in the search.
     function(f, signature, optional = FALSE,
              useInherited = TRUE,
-             mlist = (if(is.null(fdef)) NULL else getMethodsForDispatch(f, fdef)),
+	     mlist = if(!is.null(fdef)) getMethodsForDispatch(fdef),
              fdef = getGeneric(f, !optional),
              verbose = FALSE)
 {
@@ -696,7 +718,7 @@ selectMethod <-
 	if(is.null(method)) {
 	    methods <-
 		if(any(useInherited)) {
-		    allmethods <- .getMethodsTable(fdef, fenv, FALSE, TRUE)
+		    allmethods <- .getMethodsTable(fdef, fenv, check=FALSE, inherited=TRUE)
 		    ## look in the supplied (usually standard) table, cache w. inherited
 		    .findInheritedMethods(signature, fdef,
 					  mtable = allmethods, table = mlist,
@@ -704,7 +726,7 @@ selectMethod <-
 					  verbose = verbose)
 		    ##MM: TODO? allow 'excluded' to be passed
 		}
-		else list() # just look in the direct table
+		## else list() : just look in the direct table
 
 	    if(length(methods) > 0)
 		return(methods[[1]])
@@ -811,29 +833,6 @@ existsMethod <-
     }
 }
 
-dumpMethods <-
-  ## Dump all the methods for this generic.
-  ##
-  ## If `signature' is supplied only the methods matching this initial signature
-  ## are dumped.  (This feature is not found in S-Plus:  don't use it if you want
-  ## compatibility.)
-  function(f, file = "", signature = character(), methods, where = topenv(parent.frame()) )
-{
-    if(missing(methods))
-        methods <-  getMethods(f, where = where)
-    if(file != "")
-        sink(file)
-    on.exit(if(file!="")
-            sink())
-    for(what in names(methods)) {
-        el <- methods[[what]]
-        if(is.function(el))
-            dumpMethod(f, c(signature, what), file = "", def = el)
-        else
-            dumpMethods(f, "", c(signature, what), el, where)
-    }
-}
-
 signature <-
   ## A named list of classes to be matched to arguments of a generic function.
   ## It is recommended to supply signatures to `setMethod' via a call to `signature',
@@ -871,7 +870,7 @@ showMethods <-
     ##
     function(f = character(), where = topenv(parent.frame()), classes = NULL,
              includeDefs = FALSE, inherited = !includeDefs,
-             showEmpty, printTo = stdout())
+             showEmpty, printTo = stdout(), fdef = getGeneric(f, where = where))
 {
     if(missing(showEmpty))
 	showEmpty <- !missing(f)
@@ -881,7 +880,8 @@ showMethods <-
     if(is(f, "function"))
         f <- as.character(substitute(f))
     if(!is(f, "character"))
-        stop(gettextf("first argument should be the name(s) of generic functions (got object of class \"%s\")", class(f)), domain = NA)
+        stop(gettextf("first argument should be the name(s) of generic functions (got object of class \"%s\")",
+                      class(f)), domain = NA)
     if(length(f)==0) {
         f <- if(missing(where)) getGenerics() else getGenerics(where)
     }
@@ -889,39 +889,50 @@ showMethods <-
 	cat(file = con, "No applicable functions\n")
     else if(length(f) > 1) {
 	for(ff in f) { ## recall for each
-	    fdef <- getGeneric(ff, where = where)
-	    if(is.null(fdef))
-		next
-	    Recall(ff, where=where, classes=classes,
+            ffdef <- getGeneric(ff, where = where)
+            if(missing(where)) {
+                if(isGeneric(ff))
+                  Recall(ff, classes=classes,
 		   includeDefs=includeDefs, inherited=inherited,
-		   showEmpty=showEmpty, printTo=printTo)
+		   showEmpty=showEmpty, printTo=printTo, fdef = ffdef)
+            }
+            else if(isGeneric(ff, where)) {
+                Recall(ff, where=where, classes=classes,
+		   includeDefs=includeDefs, inherited=inherited,
+		   showEmpty=showEmpty, printTo=printTo, fdef = ffdef)
+            }
 	}
     }
     else { ## f of length 1 --- the "workhorse" :
         out <- paste("\nFunction \"", f, "\":\n", sep="")
-        if(!isGeneric(f, where))
+        isGen  <- if(missing(fdef)) {
+            (if(missing(where)) isGeneric(f) else isGeneric(f, where)) }
+                     else is(fdef, "genericFunction")
+        if(!isGen)
             cat(file = con, out, "<not a generic function>\n")
         else
             ## maybe no output for showEmpty=FALSE
-            .showMethodsTable(getGeneric(f, where = where), includeDefs, inherited,
+            .showMethodsTable(fdef, includeDefs, inherited,
 				  classes = classes, showEmpty = showEmpty,
 				  printTo = con)
     }
     invisible(printTo)
 }
 
-## this should be made obsolete
+## this should be made obsolete:  asserted that it is no longer called in this package
 removeMethodsObject <-
     function(f, where = topenv(parent.frame()))
 {
+    .Deprecated()# (for 2.7.0) -- defunct for 2.8.0
+
     fdef <- getGeneric(f, where=where)
     if(!is(fdef, "genericFunction")) {
       warning(gettextf(
     "No generic function found for \"%s\"; no action taken in removeMethodsObject", f))
       return(FALSE)
   }
-  what <- mlistMetaName(f, fdef@package)
-  if(!exists(what, where))
+  what <- methodsPackageMetaName("M", f, fdef@package)
+  if(!exists(what, where, inherits = FALSE))
       return(FALSE)
   where <- as.environment(where)
   if(environmentIsLocked(where)) {
@@ -942,7 +953,7 @@ removeMethods <-
   ## a simple function with this definition; otherwise, it will be removed.  The
   ## assignment or removal can be controlled by optional argument `where', which
   ## defaults to the first element of the search list having a function called `f'.
-  function(f, where = topenv(parent.frame()), all = TRUE)
+  function(f, where = topenv(parent.frame()), all = missing(where))
 {
     ## NOTE:  The following is more delicate than one would like, all because of
     ## methods for primitive functions.  For those, no actual generic function exists,
@@ -959,41 +970,43 @@ removeMethods <-
         return(FALSE)
     }
 
-    methods <- getMethodsForDispatch(f, fdef) # list or table
-    if(is.environment(methods)) {
-##      remove(list=objects(methods, all.names=TRUE), envir = methods)
-      mlist <- getMethods(fdef) # always a methods list
-    }
-    else
-      mlist <- methods
+    methods <- getMethodsForDispatch(fdef)
     default <- getMethod(fdef, "ANY", optional = TRUE)
-    fMetaName <- mlistMetaName(fdef)
+    fMetaName <- .TableMetaName(fdef@generic, fdef@package)
+    oldMetaName <- methodsPackageMetaName("M",fdef@generic, fdef@package)
     allWhere <- .findAll(fMetaName, where)
     if(!all)
         allWhere <- allWhere[1]
     value <- rep(TRUE, length(allWhere))
-    for(i in seq_along(allWhere)) {
-        db <- allWhere[[i]]
-        obj <- get(fMetaName, db)
-        ## remove non-empty methods list objects
-        if(is(obj, "MethodsList") && length(obj@methods)>0)
-            value[[i]] <- removeMethodsObject(f, where = db)
-    }
     ## cacheGenericsMetaData is called to clear primitive methods if there
     ## are none for this generic on other databases.
     cacheGenericsMetaData(f, fdef, FALSE, where)
     .uncacheGeneric(f, fdef) # in case it gets removed or re-assigned
-   allWhere <- allWhere[value] # process functions only where methods successfully removed
+    doGeneric <- TRUE # modify the function
     for(i in seq_along(allWhere)) {
         db <- as.environment(allWhere[[i]])
-        if(isGeneric(f, db)) { # note use of isGeneric to work for primitives
-            if(environmentIsLocked(db)) {
-                warning(gettextf("cannot restore previous version of \"%s\" in locked environment/package \"%s\"",
+        if(environmentIsLocked(db)) {
+                warning(gettextf("cannot remove methods for  \"%s\" in locked environment/package \"%s\"",
                         f, getPackageName(db)), domain = NA)
                 value[[i]] <- FALSE
+                next
             }
+            if(exists(fMetaName, db, inherits = FALSE)) {
+                ## delete these methods from the generic
+                theseMethods <- get(fMetaName, db)
+                .mergeMethodsTable(fdef, methods, theseMethods, FALSE)
+                rm(list = fMetaName, pos = db)
+                if(exists(oldMetaName, db, inherits = FALSE))
+                  rm(list = oldMetaName, pos = db)
+            }
+    }
+    all <- all && base::all(value) # leave methods on any locked packages
+    # now find and reset the generic function
+    for(i in seq_along(allWhere)) {
+        db <- as.environment(allWhere[[i]])
+        if(doGeneric && isGeneric(f, db)) {
             ## restore the original function if one was used as default
-            if(is(default, "derivedDefaultMethod")) {
+            if(all && is(default, "derivedDefaultMethod")) {
                 default <- as(default, "function") # strict, removes slots
                 rm(list=f, pos = db)
                 if(!existsFunction(f, FALSE, db)) {
@@ -1003,36 +1016,29 @@ removeMethods <-
                 }
                 ## else the generic is removed, nongeneric will be found elsewhere
             }
-            ## else, leave the generic in place
+            ## else, leave the generic in place, with methods removed
+            ## and inherited methods reset
             else {
-                mlist@methods <- mlist@allMethods <- list()
-                resetGeneric(f, fdef, methods, db, default)
-                assignMethodsMetaData(f, mlist, fdef, db, default)
+                resetGeneric(f, fdef, where = db, deflt = default)
             }
-            break
+            doGeneric <- FALSE
         }
     }
     any(value)
 }
 
 
-resetGeneric <- function(f, fdef = getGeneric(f, where = where), mlist = getMethodsForDispatch(f, fdef), where = topenv(parent.frame()), deflt = finalDefaultMethod(mlist)) {
+resetGeneric <- function(f, fdef = getGeneric(f, where = where),
+			 mlist = getMethodsForDispatch(fdef),
+			 where = topenv(parent.frame()),
+			 deflt = finalDefaultMethod(mlist))
+{
     if(!is(fdef, "genericFunction")) {
-        if(missing(mlist)) {
-            warning(gettextf("cannot reset \"%s\", the definition is not a generic function object", f), domain = NA)
-            return(NULL)
-        }
-        else
             stop(gettextf("error in updating generic function \"%s\"; the function definition is not a generic function (class \"%s\")", f, class(fdef)),
                  domain = NA)
-    }
-    ## uncache
-    if(is.environment(mlist))
-        .updateMethodsInTable(fdef, where, "reset")
-    else {
-        mlist@allMethods <- mlist@methods
-        .genericAssign(f, fdef, mlist, where, deflt)
-    }
+        }
+    ## reset inherited methods
+    .updateMethodsInTable(fdef, attach = "reset")
     f
 }
 
@@ -1114,7 +1120,7 @@ callGeneric <- function(...)
         fname <- as.name(f)
         if(nargs() == 0) {
             call[[1]] <- as.name(fname) # in case called from .local
-        ## exapnd the ... if this is  a locally modified argument list.
+        ## expand the ... if this is  a locally modified argument list.
         ## This is a somewhat ambiguous case and may not do what the
         ## user expects.  Not clear there is a single solution.  Should we warn?
             call <- match.call(fdef, call, expand.dots = localArgs)
@@ -1135,11 +1141,13 @@ initMethodDispatch <- function(where = topenv(parent.frame()))
     .Call("R_initMethodDispatch", as.environment(where),
           PACKAGE = "methods")# C-level initialization
 
-isSealedMethod <- function(f, signature, fdef = getGeneric(f, FALSE, where = where), where = topenv(parent.frame())) {
+isSealedMethod <- function(f, signature, fdef = getGeneric(f, FALSE, where = where),
+			   where = topenv(parent.frame()))
+{
     ## look for the generic to see if it is a primitive
     fGen <- getFunction(f, TRUE, FALSE, where = where)
     if(!is.primitive(fGen)) {
-        mdef <- getMethod(f, signature, optional = TRUE, where = where)
+        mdef <- getMethod(f, signature, optional = TRUE, where = where, fdef = fGen)
         return(is(mdef, "SealedMethodDefinition"))
     }
     ## else, a primitive
@@ -1168,7 +1176,7 @@ isSealedMethod <- function(f, signature, fdef = getGeneric(f, FALSE, where = whe
       return(TRUE)
     name <- fdef@generic
     package <- fdef@package
-    objs <- c(name, mlistMetaName(name, package))
+    objs <- c(name, .TableMetaName(name, package))
     for(obj in objs) {
         hasIt <- exists(obj, env, inherits = FALSE)
         ## the method object may be bound, or a new one may be needed
@@ -1191,24 +1199,26 @@ implicitGeneric <- function(...) NULL
 ### table.  This is the reccomended approach and is required if you
 ### want the saved generic to include any non-default methods.
 ###
-### Otherwise, a generic is temporarily created by a call to
-### setGeneric(), and then saved.
-###
-### Unless restore is FALSE,  the generic will be removed by a call
-### to removeMethods(), which will leave the nongeneric (== the
-### default method) as the function.  It's currently an error if there
-### is no such default, because the nongeneric was supposed to be in
-### this same package/environment.  The case keepGeneric=TRUE is used
-### by the methods package to establish implicit generics for
-### functions in the base package, without owning a version of these.
   {
+      if(!nzchar(name))
+        stop(gettextf('expected a non-empty character string for argument name'), domain = NA)
+      if(!missing(generic) && is(generic, "genericFunction") && !.identC(name, generic@generic))
+        stop(gettextf('generic function supplied was not created for \"%s\"', name), domain = NA)
       createGeneric <- (missing(generic) || !is(generic, "genericFunction")) && !isGeneric(name, where)
       if(createGeneric) {
           fdefault <- getFunction(name, where = where, mustFind = FALSE)
           if(is.null(fdefault))
             return(NULL)  # no implicit generic
           fdefault <- .derivedDefaultMethod(fdefault)
-          package <- getPackageName(environment(fdefault))
+          if(is.primitive(fdefault)) {
+              value <- genericForPrimitive(name)
+              if(!missing(generic) && !identical(value, generic))
+                stop(gettextf('"%s" is a primitive function; its generic form cannot be redefined',name), domain = NA)
+              generic <- value
+              package <- "base"
+               }
+          else
+            package <- getPackageName(environment(fdefault))
           ## look for a group
           if(identical(package,"base"))
             group <- .getImplicitGroup(name, .methodsNamespace)
@@ -1238,16 +1248,11 @@ setGenericImplicit <- function(name, where = topenv(parent.frame()), restore = T
                          name), domain = NA)
         return(FALSE)
     }
-    generic <- getGeneric(name, where)
-    if(restore) {
-        fdefault <- getMethod(name, fdef = generic, optional = TRUE)
-        ## must either restore a nongeneric or keep the generic
-        if(is.null(fdefault))
-          stop(gettextf("No nongeneric version of function \"%s\" exists to be restored",
-                        name), domain = NA)
+    generic <- getGeneric(name, where = where)
+    if(restore)
+        removeMethods(name, where)
+    else
         removeGeneric(name, where)
-        assign(name, as(fdefault, "function"), where)
-    }
     .saveToImplicitGenerics(name, generic, where)
 }
 
@@ -1317,9 +1322,10 @@ registerImplicitGenerics <- function(what = .ImplicitGenericsTable(where), where
         warning(gettextf("Implicit generic exists for \"%s\", but will be overriden by explicit call to setGeneric()", name), domain = NA)
       else {
           ## compare this to the end of setGeneric().  It does not check group membership
-          ## or assign a methods table object.  Keep an eye on the differences.
+          ## or assign a methods list object.  Keep an eye on the differences.
           assign(name, genericFunction, where)
           .cacheGeneric(name, genericFunction)
+          .assignMethodsTableMetaData(name, genericFunction, where)
       }
       OK
   }
@@ -1356,3 +1362,120 @@ registerImplicitGenerics <- function(what = .ImplicitGenericsTable(where), where
     }
     list()
 }
+
+findMethods <- function(f, where, classes = character(), inherited = FALSE) {
+    if(is(f, "genericFunction")) {
+        fdef <- f
+        f <- fdef@generic
+    }
+    else if(is.character(f) && length(f) == 1) {
+        fdef <- if(missing(where))
+            getGeneric(f)
+        else
+            getGeneric(f, where = where)
+    }
+    else if(!is(f, "function"))
+        stop(gettextf("argument \"f\" must be a generic function or a single character string; got an object of class \"%s\"", class(f)), domain = NA)
+    else {
+        fdef <- f
+        f <- deparse(substitute(f))
+    }
+    if(!is(fdef, "genericFunction")) {
+        warning("non-generic function given to findMethods()")
+        return(list())
+    }
+    if(missing(where))
+      table <- get(if(inherited) ".AllMTable" else ".MTable", envir = environment(fdef))
+    else {
+        if(!identical(inherited, FALSE))
+          stop("Only FALSE meaningful for inherited, when where is supplied (got ", inherited, "\"")
+        where <- as.environment(where)
+        what <- .TableMetaName(f, fdef@package)
+        if(exists(what, envir = where, inherits = FALSE))
+          table <- get(what, envir = where)
+        else
+          return(list())
+    }
+    objNames <- objects(table, all.names = TRUE)
+    if(length(classes)>0) {
+        classesPattern <- paste("#", classes, "#", sep="", collapse = "|")
+        which <- grep(classesPattern, paste("#",objNames,"#", sep=""))
+        objNames <- objNames[which]
+    }
+    if(length(objNames) > 0) {
+        value <- lapply(objNames, function(x)get(x, envir = table))
+        names(value) <- objNames
+        value
+    }
+    else
+      list()
+}
+
+findMethodSignatures <- function(..., target = TRUE, methods = findMethods(...)) {
+    ## unless target is FALSE, the signatures are just the names from the table
+    ## unscrambled.
+    if(length(methods) < 1)
+      return(methods)
+    ## find the argument names
+    prims <- sapply(methods, is.primitive)
+    if(!all(prims)) {
+        what <- lapply(methods[!prims], function(x)  names(x@target))
+        lens <- sapply(what, length)
+        if(length(unique(lens)) == 1) # asserted to be true for legit. method tables
+          what <- what[[1]]
+        else
+          what <- what[lens == max(lens)][[1]]
+    }
+    else
+      what <- names(formals(args(methods[[1]]))) # uses the hack that args() returns a function if its argument is a primitve!
+    if(target)
+      sigs <- strsplit(names(methods), "#", fixed = TRUE)
+    else {
+        anySig <- rep("ANY", length(what))
+        sigs <- lapply(methods, function(x) if(is.primitive(x)) anySig else as.character(x@defined))
+    }
+    lens <- unique(sapply(sigs, length))
+    if(length(lens) > 1)
+      sigs
+    else
+      t(matrix(unlist(sigs), nrow = lens, dimnames = list(what, NULL)))
+}
+
+hasMethods <- function(f, where,package) {
+    fdef <- NULL
+    nowhere <- missing(where) # because R resets this if where is assigned
+    if(is(f, "genericFunction")) {
+        fdef <- f
+        f <- fdef@generic
+    }
+    else if(!(is.character(f) && length(f) == 1))
+        stop(gettextf("argument \"f\" must be a generic function or a single character string; got an object of class \"%s\"", class(f)), domain = NA)
+    if(missing(package)) {
+        package <- packageSlot(f)
+        if(is.null(package)) {
+            if(missing(where))
+              where <- .GlobalEnv
+            fdef <- getFunction(f, where = where)
+            if(is(fdef, "genericFunction"))
+              package <- fdef@package
+            else if(is.primitive(fdef))
+              package <- "base"
+            else if( length(ff <- findFunction(f, where = where)) == 1)
+                package <- getPackageName(ff[[1]])
+            else
+              stop(gettextf("f does not correspond to a generic function and package not specified"), domain = NA)
+        }
+    }
+    what <- .TableMetaName(f, package)
+    testEv <- function(ev)
+      exists(what, envir = ev, inherits = FALSE) && (length(objects(get(what, envir = ev), all.names = TRUE)) > 0)
+    if(nowhere) {
+        for(i in seq(along = search())) {
+            if(testEv(as.environment(i)))
+              return(TRUE)
+        }
+        return(FALSE)
+    }
+    else
+      testEv(as.environment(where))
+}
