@@ -509,7 +509,6 @@ newBasic <-
                "character" =,
                "complex" =,
                "integer" =,
-               "double" =,
                "raw" =,
                "list" =  as.vector(c(...), Class),
                "expression" = eval(substitute(expression(...))),
@@ -1028,15 +1027,18 @@ classMetaName <-
   function(name)
   methodsPackageMetaName("C", name)
 
-##FIXME:  C code should take multiple strings in name so the paste() call in
-## mlistMetaName, etc. could be avoided.
+##FIXME:  C code should take multiple strings in name so paste() calls could  be avoided.
 methodsPackageMetaName <-
   ## a name mangling device to simulate the meta-data in S4
-  function(prefix, name)
+  function(prefix, name, package = "")
   ## paste(".", prefix, name, sep="__") # too slow
-    .Call("R_methodsPackageMetaName", prefix, name, PACKAGE = "methods")
-
+    .Call("R_methodsPackageMetaName", prefix, name, package, PACKAGE = "methods")
 
+## a  non-exported regexp that matches  methods metanames
+## This is quite general and matches all patterns that could be generated
+## by calling methodsPackageMetaName() with a sequence of capital Latin letters
+## Used by package.skeleton in utils
+.methodsPackageMetaNamePattern <- "^[.]__[A-Z]+__"
 
 requireMethods <-
   ## Require a subclass to implement methods for the generic functions, for this signature.
@@ -1311,43 +1313,39 @@ setDataPart <- function(object, value) {
 .simpleCoerceExpr <- function(fromClass, toClass, fromSlots, toDef) {
     toSlots <- names(toDef@slots)
     sameSlots <- (length(fromSlots) == length(toSlots) &&
-                      !any(is.na(match(fromSlots, toSlots))))
+		  !any(is.na(match(fromSlots, toSlots))))
     if(sameSlots)
-        expr <- substitute({class(from)[[1]] <- CLASS; from},
-                   list(CLASS = toClass))
+	substitute({class(from)[[1]] <- CLASS; from}, list(CLASS = toClass))
+    else if(length(toSlots)==0) {
+	## either a basic class or something with the same representation
+	if(is.na(match(toClass, .BasicClasses)))
+	    substitute({ attributes(from) <- NULL; class(from)[[1]] <- CLASS; from},
+		       list(CLASS = toClass))
+	else if(isVirtualClass(toDef))
+	    quote(from)
+	else {
+	    ## a basic class; a vector type, matrix, array, or ts
+	    switch(toClass,
+		   matrix = , array = {
+		       quote({.dm <- dim(from); .dn <- dimnames(from)
+			      attributes(from) <- NULL; dim(from) <- .dm
+			      dimnames(from) <- .dn; from})
+		   },
+		   ts = {
+		       quote({.tsp <- tsp(from); attributes(from) <- NULL
+			      tsp(from) <- .tsp; class(from) <- "ts"; from})
+		   },
+		   quote({attributes(from) <- NULL; from})
+		   )
+	}
+    }
     else {
-        if(length(toSlots)==0) {
-            ## either a basic class or something with the same representation
-            if(is.na(match(toClass, .BasicClasses)))
-                expr <- substitute({ attributes(from) <- NULL; class(from)[[1]] <- CLASS; from},
-                                   list(CLASS=toClass))
-            else if(isVirtualClass(toDef))
-                expr <- quote(from)
-            else {
-                ## a basic class; a vector type, matrix, array, or ts
-                switch(toClass,
-                       matrix = , array = {
-                           expr <- quote({.dm <- dim(from); .dn <- dimnames(from)
-                                    attributes(from) <- NULL; dim(from) <- .dm
-                                    dimnames(from) <- .dn; from})
-                       },
-                       ts = {
-                           expr <- quote({.tsp <- tsp(from); attributes(from) <- NULL
-                                          tsp(from) <- .tsp; class(from) <- "ts"; from})
-                       },
-                       expr <- quote({attributes(from) <- NULL; from})
-                       )
-            }
-        }
-        else {
-            expr <- substitute({
-            value <- new(CLASS)
-            for(what in TOSLOTS)
-                slot(value, what) <- slot(from, what)
-            value }, list(CLASS=toClass, TOSLOTS = toSlots))
-        }
+	substitute({ value <- new(CLASS)
+		     for(what in TOSLOTS)
+			 slot(value, what) <- slot(from, what)
+		     value },
+		   list(CLASS = toClass, TOSLOTS = toSlots))
     }
-    expr
 }
 
 .simpleReplaceExpr <- function(toDef) {
@@ -1405,25 +1403,28 @@ newClassRepresentation <- function(...) {
         cl
 }
 
-substituteFunctionArgs <- function(def, newArgs, args = formalArgs(def), silent = FALSE, functionName = "a function") {
+substituteFunctionArgs <-
+    function(def, newArgs, args = formalArgs(def), silent = FALSE,
+             functionName = "a function")
+{
     if(!identical(args, newArgs)) {
-        if( !missing(functionName) )
-            functionName = paste("for", functionName)
-        
+        if( !missing(functionName) ) # this style does not allow translation
+            functionName <- paste("for", functionName)
+
         n <- length(args)
         if(n != length(newArgs))
-            stop(gettextf("trying to change the argument list of %s with %d arguments to have arguments (%s)",
-                          functionName, n, paste(newArgs, collapse = ", ")),
+            stop(sprintf("trying to change the argument list of %s with %d arguments to have arguments (%s)",
+                         functionName, n, paste(newArgs, collapse = ", ")),
                  domain = NA)
         bdy <- body(def)
         ## check for other uses of newArgs
         checkFor <- newArgs[is.na(match(newArgs, args))]
         locals <- all.vars(bdy)
         if(length(checkFor) > 0 && any(!is.na(match(checkFor, locals))))
-            stop(gettextf("get rid of variables in definition %s (%s); they conflict with the needed change to argument names (%s)",
-                          functionName,
-                          paste(checkFor[!is.na(match(checkFor, locals))], collapse = ", "),
-                          paste(newArgs, collapse = ", ")), domain = NA)
+            stop(sprintf("get rid of variables in definition %s (%s); they conflict with the needed change to argument names (%s)",
+                         functionName,
+                         paste(checkFor[!is.na(match(checkFor, locals))], collapse = ", "),
+                         paste(newArgs, collapse = ", ")), domain = NA)
         ll <- vector("list", 2*n)
         for(i in seq_len(n)) {
             ll[[i]] <- as.name(args[[i]])
@@ -1433,11 +1434,11 @@ substituteFunctionArgs <- function(def, newArgs, args = formalArgs(def), silent
         body(def, envir = environment(def)) <- substituteDirect(bdy, ll)
         if(!silent) {
             msg <-
-                gettextf("arguments in definition %s changed from (%s) to (%s)",
-                         functionName,
-                         paste(args, collapse = ", "),
-                         paste(newArgs, collapse = ", "))
-            message(strwrap(msg), domain = NA)
+                sprintf("NOTE: arguments in definition %s changed from (%s) to (%s)",
+                        functionName,
+                        paste(args, collapse = ", "),
+                        paste(newArgs, collapse = ", "))
+            message(msg, domain = NA)
         }
     }
     def
@@ -1447,7 +1448,7 @@ substituteFunctionArgs <- function(def, newArgs, args = formalArgs(def), silent
     if(!is.null(validity)) {
         if(!is(validity, "function"))
             stop(gettextf("a validity method must be a function of one argument, got an object of class \"%s\"", class(validity)), domain = NA)
-        validity <- substituteFunctionArgs(validity, "object", functionName = paste("validity method for class", Class))
+        validity <- substituteFunctionArgs(validity, "object", functionName = sprintf("validity method for class '%s'", Class))
     }
     validity
 }
@@ -1491,7 +1492,7 @@ substituteFunctionArgs <- function(def, newArgs, args = formalArgs(def), silent
 ## real version of .requirePackage
 ..requirePackage <- function(package, mustFind = TRUE) {
     value <- package
-    if(is.character(package)) {
+    if(nzchar(package)) {
         if(package %in% loadedNamespaces())
             value <- getNamespace(package)
         else {
@@ -1511,7 +1512,8 @@ substituteFunctionArgs <- function(def, newArgs, args = formalArgs(def), silent
     if(exists(".packageName", topEnv, inherits=TRUE) &&
        .identC(package, get(".packageName", topEnv)))
         return(topEnv) # kludge for source'ing package code
-    if(!require(package, character.only = TRUE)) {
+    if(nzchar(package) && require(package, character.only = TRUE)) {}
+    else {
         if(mustFind)
           stop(gettextf("unable to find required package \"%s\"", package),
                domain = NA)
