@@ -1,6 +1,21 @@
+#  File src/library/stats/R/cor.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 #### cor() , cov() and var() : Based on the same C code
 
-## cor() and cov() only differ by one single letter :
 cor <-
 function(x, y=NULL, use="all.obs", method = c("pearson", "kendall", "spearman"))
 {
@@ -17,17 +32,14 @@ function(x, y=NULL, use="all.obs", method = c("pearson", "kendall", "spearman"))
 	}
     }
     if(method == "pearson")
-        .Internal(cor(x, y, na.method, method == "kendall"))
-    else if (na.method != 3) {
+        .Internal(cor(x, y, na.method, FALSE))
+    else if (na.method != 3L) {
 	## Rank transform
-	Rank <- function(u)
-	    if(is.matrix(u)) apply(u, 2, rank, na.last="keep")
-	    else rank(u, na.last="keep")
-
-        if (na.method == 2){ # complete.obs
-            ok <- complete.cases(x,y)
-            x <- if (is.matrix(x)) x[ok,] else x[ok]
-            if(!is.null(y)) y <- if(is.matrix(y)) y[ok,] else y[ok]
+	Rank <- function(u) {
+            ## take care not to drop dims on a 0-row matrix
+            if(length(u) == 0L) u else
+            if(is.matrix(u)) apply(u, 2L, rank, na.last="keep")
+            else rank(u, na.last="keep")
         }
 
 	x <- Rank(x)
@@ -39,43 +51,49 @@ function(x, y=NULL, use="all.obs", method = c("pearson", "kendall", "spearman"))
          ## matrix
          if (is.null(y)) {
              ncy <- ncx <- ncol(x)
-             r <- matrix(0, nrow=ncx, ncol=ncy)
-             for (i in seq(2, length = ncx - 1)) {
-                 for (j in seq_len(i - 1)) {
+             if(ncx == 0) stop("'x' is empty")
+             r <- matrix(0, nrow = ncx, ncol = ncy)
+             ## 2.6.0 assumed the diagonal was 1, but not so for all NAs,
+             ## nor single non-NA pairs.
+             for (i in seq_len(ncx)) {
+                 for (j in seq_len(i)) {
                      x2 <- x[,i]
                      y2 <- x[,j]
                      ok <- complete.cases(x2, y2)
                      x2 <- rank(x2[ok])
                      y2 <- rank(y2[ok])
-                     r[i, j] <- .Internal(cor(x2, y2, na.method, method == "kendall"))
+                     ## we've removed all NAs
+                     r[i, j] <- if(any(ok)) .Internal(cor(x2, y2, 1L, method == "kendall")) else NA
                  }
              }
-             r <- r+t(r)
-             diag(r) <- 1
+             r <- r + t(r) - diag(diag(r))
 	     rownames(r) <- colnames(x)
 	     colnames(r) <- colnames(x)
              r
          }
-         ## matrix x matrix
+         ## vector/matrix x vector/matrix
          else {
-	     if (!is.matrix(x)) x <- matrix(x, ncol=1)
-	     if (!is.matrix(y)) y <- matrix(y, ncol=1)
+             if(length(x) == 0 || length(y) == 0)
+                 stop("both 'x' and 'y' must be non-empty")
+             matrix_result <- is.matrix(x) || is.matrix(y)
+	     if (!is.matrix(x)) x <- matrix(x, ncol=1L)
+	     if (!is.matrix(y)) y <- matrix(y, ncol=1L)
              ncx <- ncol(x)
              ncy <- ncol(y)
-             r <- matrix(0, nrow=ncx, ncol=ncy)
-             for (i in 1:ncx) {
-                 for (j in 1:ncy) {
+             r <- matrix(0, nrow = ncx, ncol = ncy)
+             for (i in seq_len(ncx)) {
+                 for (j in seq_len(ncy)) {
                      x2 <- x[,i]
                      y2 <- y[,j]
                      ok <- complete.cases(x2, y2)
                      x2 <- rank(x2[ok])
                      y2 <- rank(y2[ok])
-                     r[i, j] <- .Internal(cor(x2, y2, na.method, method == "kendall"))
+                     r[i, j] <- if(any(ok)) .Internal(cor(x2, y2, 1L, method == "kendall")) else NA
                  }
              }
 	     rownames(r) <- colnames(x)
 	     colnames(r) <- colnames(y)
-             r
+             if(matrix_result) r else drop(r)
          }
      }
 }
@@ -97,16 +115,12 @@ function(x, y=NULL, use="all.obs", method = c("pearson", "kendall", "spearman"))
     }
     if(method == "pearson")
         .Internal(cov(x, y, na.method, method == "kendall"))
-    else if (na.method != 3) {
+    else if (na.method != 3L) {
 	## Rank transform
-	Rank <- function(u)
-	    if(is.matrix(u)) apply(u, 2, rank, na.last="keep")
-	    else rank(u, na.last="keep")
-
-        if (na.method == 2){ # complete.obs
-            ok <- complete.cases(x,y)
-            x <- if (is.matrix(x)) x[ok,] else x[ok]
-            if(!is.null(y)) y <- if(is.matrix(y)) y[ok,] else y[ok]
+	Rank <- function(u) {
+            if(length(u) == 0) u else
+            if(is.matrix(u)) apply(u, 2, rank, na.last="keep")
+            else rank(u, na.last="keep")
         }
 
 	x <- Rank(x)
@@ -135,14 +149,14 @@ cov2cor <- function(V)
     ## ----------------------------------------------------------------------
     ## Author: Martin Maechler, Date: 12 Jun 2003, 11:50
     p <- (d <- dim(V))[1]
-    if(!is.numeric(V) || length(d) != 2 || p != d[2])
+    if(!is.numeric(V) || length(d) != 2L || p != d[2L])
 	stop("'V' is not a square numeric matrix")
     Is <- sqrt(1/diag(V)) # diag( 1/sigma_i )
     if(any(!is.finite(Is)))
-	warning("diagonal has non-finite entries")
+	warning("diag(.) had 0 or NA entries; non-finite result is doubtful")
     r <- V # keep dimnames
     r[] <- Is * V * rep(Is, each = p)
     ##	== D %*% V %*% D  where D = diag(Is)
-    r[cbind(1:p,1:p)] <- 1 # exact in diagonal
+    r[cbind(1L:p,1L:p)] <- 1 # exact in diagonal
     r
 }
