@@ -1,3 +1,19 @@
+#  File src/library/stats/R/models.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 formula <- function(x, ...) UseMethod("formula")
 formula.default <- function (x, env = parent.frame(), ...)
 {
@@ -13,7 +29,7 @@ formula.default <- function (x, env = parent.frame(), ...)
                        NULL = structure(NULL, class = "formula"),
                        character = formula(eval(parse(text = x)[[1]])),
                        call = eval(x), stop("invalid formula"))
-        environment(form)<-env
+        environment(form) <- env
         form
     }
 }
@@ -31,17 +47,16 @@ formula.terms <- function(x, ...) {
 formula.data.frame <- function (x, ...)
 {
     nm <- sapply(names(x), as.name)
-    lhs <- nm[1]
     if (length(nm) > 1) {
         rhs <- nm[-1]
-    }
-    else {
+        lhs <- nm[1]
+    } else if (length(nm) == 1) {
         rhs <- nm[1]
         lhs <- NULL
-    }
+    } else stop("cannot create a formula from a zero-column data frame")
     ff <- parse(text = paste(lhs, paste(rhs, collapse = "+"), sep = "~"))
-    ff<-eval(ff)
-    environment(ff)<-parent.frame()
+    ff <- eval(ff)
+    environment(ff) <- parent.frame()
     ff
 }
 
@@ -55,6 +70,7 @@ formula.character <- function(x, env = parent.frame(), ...)
 print.formula <- function(x, ...) {
     attr(x, ".Environment") <- NULL
     print.default(unclass(x), ...)
+    invisible(x)
 }
 
 "[.formula" <- function(x,i) {
@@ -84,11 +100,14 @@ terms.default <- function(x, ...) {
     v <- x$terms
     if(is.null(v))
         stop("no terms component")
-    return(v)
+    v
 }
 
 terms.terms <- function(x, ...) x
-print.terms <- function(x, ...) print.default(unclass(x))
+print.terms <- function(x, ...) { 
+    print.default(unclass(x))
+    invisible(x)
+}
 
 ## moved from base/R/labels.R
 labels.terms <- function(object, ...) attr(object, "term.labels")
@@ -212,8 +231,15 @@ deviance <- function(object, ...) UseMethod("deviance")
 deviance.default <- function(object, ...) object$deviance
 
 fitted <- function(object, ...) UseMethod("fitted")
+## we really do need partial matching here
 fitted.default <- function(object, ...)
     napredict(object$na.action, object$fitted)
+fitted.default <- function(object, ...)
+{
+    xx <- if("fitted.values" %in% names(object))
+        object$fitted.values else object$fitted
+    napredict(object$na.action, xx)
+}
 fitted.values <- fitted
 
 anova <- function(object, ...)UseMethod("anova")
@@ -253,12 +279,12 @@ offset <- function(object) object
         wrong <- old != new
         if(sum(wrong) == 1)
             stop(gettextf(
-    "variable '%s' was fitted with class \"%s\" but class \"%s\" was supplied",
+    "variable '%s' was fitted with type \"%s\" but type \"%s\" was supplied",
                           names(old)[wrong], old[wrong], new[wrong]),
                  call. = FALSE, domain = NA)
         else
             stop(gettextf(
-    "variables %s were specified with different classes from the fit",
+    "variables %s were specified with different types from the fit",
                  paste(sQuote(names(old)[wrong]), collapse=", ")),
                  call. = FALSE, domain = NA)
     }
@@ -274,7 +300,11 @@ offset <- function(object) object
     if(is.factor(x))  return("factor")
     if(is.matrix(x) && is.numeric(x))
         return(paste("nmatrix", ncol(x), sep="."))
-    if(is.vector(x) && is.numeric(x)) return("numeric")
+    ## this is unclear.  Prior to 2.6.0 we assumed numeric with attributes
+    ## meant something, but at least for now model.matrix does not
+    ## treat it differently.
+##    if(is.vector(x) && is.numeric(x)) return("numeric")
+    if(is.numeric(x)) return("numeric")
     return("other")
 }
 
@@ -370,7 +400,7 @@ model.frame.default <-
 	for(nm in names(xlev))
 	    if(!is.null(xl <- xlev[[nm]])) {
 		xi <- data[[nm]]
-		if(is.null(nxl <- levels(xi)))
+		if(!is.factor(xi) || is.null(nxl <- levels(xi)))
 		    warning(gettextf("variable '%s' is not a factor", nm),
                             domain = NA)
 		else {
@@ -539,10 +569,10 @@ makepredictcall.default  <- function(var, call)
 
 .getXlevels <- function(Terms, m)
 {
-    xvars <- sapply(attr(Terms, "variables"),deparse,width.cutoff=500)[-1]
+    xvars <- sapply(attr(Terms, "variables"), deparse, width.cutoff=500)[-1]
     if((yvar <- attr(Terms, "response")) > 0) xvars <- xvars[-yvar]
     if(length(xvars) > 0) {
-        xlev <- lapply(m[xvars], levels)
+        xlev <- lapply(m[xvars], function(x) if(is.factor(x)) levels(x) else NULL)
         xlev[!sapply(xlev, is.null)]
     } else NULL
 }
