@@ -18,6 +18,7 @@ tkStartGUI <- function() {
     ## Philippe Grosjean: this is added for more explicit error message under Windows
     if (.Platform$OS.type == "windows")
     	stop("The tkGUI is not available under Windows")
+    tclServiceMode(FALSE)
     tcl("source", file.path(.Library, "tcltk", "exec", "console.tcl"))
     .C("RTcl_ActivateConsole", PACKAGE = "tcltk")
     Menu <- .Tk.newwin(".menu")
@@ -57,7 +58,7 @@ tkStartGUI <- function() {
 
     loadpackageWidget <- function()
     {
-	pkglist <- .packages(all=TRUE)
+	pkglist <- .packages(all.available=TRUE)
         lvar <- tclVar()
 	tclObj(lvar) <- pkglist
 	box <- tklistbox(tt<-tktoplevel(),
@@ -136,4 +137,5 @@ tkStartGUI <- function() {
     })
     #tkadd(helpMenu,"command", label=gettext("Help on topic..."), command=topicHelp)
     assign(".GUIenv", environment(), envir=.GlobalEnv)
+    invisible(tclServiceMode(TRUE))
 }
