@@ -29,7 +29,7 @@ function(dir, outDir)
     ## path to the DESCRIPTION file, and returns an object with check
     ## results and not the package metadata ...
     ok <- .check_package_description(file.path(dir, "DESCRIPTION"))
-    if(any(as.integer(sapply(ok, length))) > 0) {
+    if(any(as.integer(sapply(ok, length)) > 0L)) {
         stop(paste(gettext("Invalid DESCRIPTION file") ,
                    paste(.capture_output_from_print(ok),
                          collapse = "\n"),
@@ -54,7 +54,7 @@ function(dir, outDir)
         "i386-pc-mingw32"
     else
         R.version$platform
-    if (length(grep("-apple-darwin",R.version$platform)) > 0 &&
+    if (length(grep("-apple-darwin",R.version$platform)) > 0L &&
         nzchar(Sys.getenv("R_ARCH")))
         OStype <- sub(".*-apple-darwin", "universal-apple-darwin", OStype)
     Built <-
@@ -103,8 +103,8 @@ function(dir, outDir)
 function(db, verbose = FALSE)
 {
     if(!is.na(Built <- db["Built"])) {
-        Built <- as.list(strsplit(Built, "; ")[[1]])
-        if(length(Built) != 4) {
+        Built <- as.list(strsplit(Built, "; ")[[1L]])
+        if(length(Built) != 4L) {
             warning(gettextf("*** someone has corrupted the Built field in package '%s' ***",
                              db["Package"]),
                     domain = NA,
@@ -119,26 +119,27 @@ function(db, verbose = FALSE)
     ## might perhaps have multiple entries
     Depends <- .split_dependencies(db[names(db) %in% "Depends"])
     if("R" %in% names(Depends)) {
-        if(verbose && sum("R" == names(Depends)) > 1) {
-            entries <- Depends["R" == names(Depends)]
-            entries <- lapply(entries, function(x)
-                paste(lapply(x, as.character), collapse="")
-            )
-            message("WARNING: 'Depends' entry has multiple dependencies: ",
+        Rdeps2 <- Depends["R" == names(Depends)]
+        names(Rdeps2) <- NULL
+        if(verbose && length(Rdeps2)> 1L) {
+            entries <- lapply(Rdeps2, function(x)
+                paste(lapply(x, as.character), collapse=""))
+            message("WARNING: 'Depends' entry has multiple dependencies on R: ",
                     paste(unlist(entries), collapse=', '),
-                    "\n\tonly the first will be used")
+                    "\n\tonly the first will be used in R < 2.7.0")
         }
         Rdeps <- Depends[["R", exact = TRUE]] # the first one
         Depends <- Depends[names(Depends) != "R"]
         ## several packages have 'Depends: R', which is a noop.
-        if(verbose && length(Rdeps) == 1)
+        if(verbose && length(Rdeps) == 1L)
              message("WARNING: omitting pointless dependence on 'R' without a version requirement")
-        if(length(Rdeps) <= 1) Rdeps <- NULL
-    } else Rdeps <- NULL
+        if(length(Rdeps) <= 1L) Rdeps <- NULL
+    } else Rdeps2 <- Rdeps <- NULL
     Rdeps <- as.vector(Rdeps)
     Suggests <- .split_dependencies(db[names(db) %in% "Suggests"])
     Imports <- .split_dependencies(db[names(db) %in% "Imports"])
-    structure(list(DESCRIPTION = db, Built = Built, Rdepends = Rdeps,
+    structure(list(DESCRIPTION = db, Built = Built,
+                   Rdepends = Rdeps, Rdepends2 = Rdeps2,
                    Depends = Depends, Suggests = Suggests,
                    Imports = Imports),
               class = "packageDescription2")
@@ -227,16 +228,8 @@ function(dir, outDir)
     collationField <-
         c(paste("Collate", .OStype(), sep = "."), "Collate")
     if(any(i <- collationField %in% names(db))) {
-        ## We have a Collate specification in the DESCRIPTION file:
-        ## currently, file paths relative to codeDir, separated by
-        ## white space, possibly quoted.  Note that we could have
-        ## newlines in DCF entries but do not allow them in file names,
-        ## hence we gsub() them out.
-        collationField <- collationField[i][1]
-        con <- textConnection(gsub("\n", " ", db[collationField]))
-        on.exit(close(con))
-        codeFilesInCspec <- scan(con, what = character(),
-                                 strip.white = TRUE, quiet = TRUE)
+        collationField <- collationField[i][1L]
+        codeFilesInCspec <- .read_collate_field(db[collationField])
         ## Duplicated entries in the collation spec?
         badFiles <-
             unique(codeFilesInCspec[duplicated(codeFilesInCspec)])
@@ -460,7 +453,7 @@ function(dir, outDir)
     if(!length(list_files_with_type(vignetteDir, "vignette"))) {
         ## we don't want to write an index if the directory is in fact empty
         files <- list.files(vignetteDir, all.files = TRUE) # includes . and ..
-        if((length(files) > 2) && !hasHtmlIndex)
+        if((length(files) > 2L) && !hasHtmlIndex)
             .writeVignetteHtmlIndex(packageName, htmlIndex)
         return(invisible())
     }
@@ -468,13 +461,25 @@ function(dir, outDir)
     vignetteIndex <- .build_vignette_index(vignetteDir)
     ## For base package vignettes there is no PDF in @file{vignetteDir}
     ## but there might/should be one in @file{outVignetteDir}.
-    if(NROW(vignetteIndex) > 0) {
+    if(NROW(vignetteIndex) > 0L) {
         vignettePDFs <-
             sub("$", ".pdf",
                 basename(file_path_sans_ext(vignetteIndex$File)))
         ind <- file_test("-f", file.path(outVignetteDir, vignettePDFs))
         vignetteIndex$PDF[ind] <- vignettePDFs[ind]
+
+        ## install tangled versions of all vignettes
+        cwd <- getwd()
+        setwd(outVignetteDir)
+        for(srcfile in vignetteIndex$File){
+            yy <- try(utils::Stangle(srcfile))
+            if(inherits(yy, "try-error")) stop(yy)
+        }
+        vignetteIndex$R <-
+            sub("$", ".R", basename(file_path_sans_ext(vignetteIndex$File)))
+        setwd(cwd)
     }
+
     if(!hasHtmlIndex)
         .writeVignetteHtmlIndex(packageName, htmlIndex, vignetteIndex)
 
@@ -555,16 +560,18 @@ function(dir, outDir, keep.source = FALSE)
     setwd(buildDir)
 
     ## Argh.  We need to ensure that vignetteDir is in TEXINPUTS and
-    ## BIBINPUTS.  Note that this does not work with MiKTeX.
-    envSep <- if(.Platform$OS.type == "windows") ";" else ":"
+    ## BIBINPUTS.  Note that this does not work with MiKTeX, but the
+    ## use of 'texi2dvi -I' as from R 2.7.0 suffices.
+    envSep <- .Platform$path.sep
     ## (Yes, it would be nice to have envPath() similar to file.path().)
     texinputs <- Sys.getenv("TEXINPUTS")
     bibinputs <- Sys.getenv("BIBINPUTS")
+    Rtexmf <- gsub("\\\\", "/", file.path(R.home(), "share", "texmf"))
     on.exit(Sys.setenv(TEXINPUTS = texinputs, BIBINPUTS = bibinputs),
             add = TRUE)
-    Sys.setenv(TEXINPUTS = paste(vignetteDir, Sys.getenv("TEXINPUTS"),
+    Sys.setenv(TEXINPUTS = paste(vignetteDir, Rtexmf, Sys.getenv("TEXINPUTS"),
                sep = envSep),
-               BIBINPUTS = paste(vignetteDir, Sys.getenv("BIBINPUTS"),
+               BIBINPUTS = paste(vignetteDir, Rtexmf, Sys.getenv("BIBINPUTS"),
                sep = envSep))
 
     for(srcfile in vignetteFiles[!upToDate]) {
@@ -572,7 +579,8 @@ function(dir, outDir, keep.source = FALSE)
         message("processing '", basename(srcfile), "'")
         texfile <- paste(base, ".tex", sep = "")
         yy <- try(utils::Sweave(srcfile, pdf = TRUE, eps = FALSE,
-                                quiet = TRUE, keep.source = keep.source))
+                                quiet = TRUE, keep.source = keep.source,
+                                stylepath = FALSE))
         if(inherits(yy, "try-error")) stop(yy)
         ## In case of an error, do not clean up: should we point to
         ## buildDir for possible inspection of results/problems?
@@ -694,11 +702,11 @@ function(dir, packages)
     ## -I"/path/to/package/include" ...
 
     if(!is.null(file)) {
-        tmp <- read.dcf(file, "LinkingTo")[1,1]
+        tmp <- read.dcf(file, "LinkingTo")[1L, 1L]
         if(is.na(tmp)) return(invisible())
         pkgs <- tmp
     }
-    pkgs <- strsplit(pkgs[1], ",[:blank]*")[[1]]
+    pkgs <- strsplit(pkgs[1L], ",[:blank]*")[[1L]]
     paths <- .find.package(pkgs, lib.loc, quiet=TRUE)
     if(length(paths))
         cat(paste(paste('-I"', paths, '/include"', sep=""), collapse=" "))
@@ -760,34 +768,37 @@ function(dir)
 {
     if(missing(dir)) dir <- "."
     meta <- .read_description(file.path(dir, "DESCRIPTION"))
-    depends <- .split_description(meta, verbose = TRUE)$Rdepends
+    deps <- .split_description(meta, verbose = TRUE)$Rdepends2
     status <- 0
-    ## .split_description will have ensured that this is NULL or
-    ## of length 3.
-    if(length(depends) > 1) {
-        ## .check_package_description will insist on these operators
-        if(!depends$op %in% c("<=", ">="))
-            message("WARNING: malformed 'Depends' field in 'DESCRIPTION'")
-        else
-            status <- !do.call(depends$op,
-                               list(getRversion(), depends$version))
-        if(status != 0) {
-            package <- Sys.getenv("R_PACKAGE_NAME")
-            if(!nzchar(package))
-                package <- meta["Package"]
-            if(nzchar(package))
-                msg <- gettextf("ERROR: this R is version %s, package '%s' requires R %s %s",
-                                getRversion(), package,
-                                depends$op, depends$version)
-            else if (nzchar(bundle <-  meta["Bundle"]) && !is.na(bundle))
-                msg <- gettextf("ERROR: this R is version %s, bundle '%s' requires R %s %s",
-                                getRversion(), bundle,
-                                depends$op, depends$version)
+    current <- getRversion()
+    for(depends in deps) {
+        ## .split_description will have ensured that this is NULL or
+        ## of length 3.
+        if(length(depends) > 1L) {
+            ## .check_package_description will insist on these operators
+            if(!depends$op %in% c("<=", ">=", "<", ">", "==", "!="))
+                message("WARNING: malformed 'Depends' field in 'DESCRIPTION'")
             else
-                msg <- gettextf("ERROR: this R is version %s, required is R %s %s",
-                                getRversion(),
-                                depends$op, depends$version)
-            message(strwrap(msg, exdent = 2))
+                status <- !do.call(depends$op,
+                                   list(current, depends$version))
+            if(status != 0) {
+                package <- Sys.getenv("R_PACKAGE_NAME")
+                if(!nzchar(package))
+                    package <- meta["Package"]
+                if(nzchar(package))
+                    msg <- gettextf("ERROR: this R is version %s, package '%s' requires R %s %s",
+                                    current, package,
+                                    depends$op, depends$version)
+                else if (nzchar(bundle <-  meta["Bundle"]) && !is.na(bundle))
+                    msg <- gettextf("ERROR: this R is version %s, bundle '%s' requires R %s %s",
+                                    current, bundle,
+                                    depends$op, depends$version)
+                else
+                    msg <- gettextf("ERROR: this R is version %s, required is R %s %s",
+                                    current, depends$op, depends$version)
+                message(strwrap(msg, exdent = 2L))
+                break
+            }
         }
     }
     q(status = status)
