@@ -13,8 +13,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 /* <UTF8> OK since this is intended to treat chars as byte streams */
@@ -28,7 +28,8 @@ SEXP Rmd5(SEXP files)
 {
     SEXP ans;
     int i, j, nfiles = length(files), res;
-    char *path, out[33];
+    const char *path;
+    char out[33];
     FILE *fp;
     unsigned char resblock[16];
 
