@@ -95,7 +95,18 @@ localeToCharset <- function(locale = Sys.getlocale("LC_CTYPE"))
                   "georgianps", "koi8r", "koi8u", "tcvn",
                   "big5" , "gb2312", "gb18030", "gbk",
                   "tis-620", "sjis", "eucn", "big5-hkscs")
-            if(enc %in% names(known)) return(as.vector(known[enc]))
+	    if (length(grep("darwin",R.version$os))) {
+	        k <- c(known, "ISO8859-1", "ISO8859-2", "ISO8859-4",
+		  "ISO8859-7", "ISO8859-9", "ISO8859-13", "ISO8859-15",
+		  "KOI8-U", "KOI8-R", "PT154", "ASCII", "ARMSCII-8",
+		  "ISCII-DEV", "BIG5-HKCSC")
+		names(k) <- c(names(known), "iso8859-1", "iso8859-2", "iso8859-4",
+		  "iso8859-7", "iso8859-9", "iso8859-13", "iso8859-15",
+		  "koi8-u", "koi8-r", "pt154", "us-ascii", "armscii-8",
+		  "iscii-dev", "big5hkscs")
+		known <- k
+            }
+	    if(enc %in% names(known)) return(as.vector(known[enc]))
             if(length(grep("^cp-", enc)))  # old Linux
                 return(sub("cp-([0-9]+)", "CP\\1", enc))
             if(enc == "EUC") {
@@ -107,6 +118,11 @@ localeToCharset <- function(locale = Sys.getlocale("LC_CTYPE"))
                 }
             }
         }
+	## on Darwin all real locales w/o encoding are UTF-8
+	## HOWEVER! unlike the C code, we cannot filter out
+	## invalid locales, so wit ill be wrong for non-supported
+	## locales (why is this duplicated in R code anyway?)
+	if (length(grep("darwin",R.version$os))) return("UTF-8")
         ## let's hope it is a ll_* name.
         if(length(grep("^[[:alpha:]]{2}_", x[1]))) {
             ll <- substr(x[1], 1, 2)
