@@ -1,3 +1,19 @@
+#  File src/library/utils/R/summRprof.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 
 
 summaryRprof<-function(filename = "Rprof.out", chunksize=5000,
@@ -21,14 +37,14 @@ summaryRprof<-function(filename = "Rprof.out", chunksize=5000,
         return(Rprof_memory_summary(filename=filename, chunksize=chunksize,
                              aggregate=index,diff=diff,exclude=exclude,
                                     sample.interval=sample.interval))
-        
-    
+
+
     fnames<-NULL
     ucounts<-NULL
     fcounts<-NULL
     memcounts<-NULL
     umem<-NULL
-  
+
     repeat({
 
        chunk<-readLines(filename,n=chunksize)
@@ -42,26 +58,26 @@ summaryRprof<-function(filename = "Rprof.out", chunksize=5000,
                memcounts<-c(0,rowSums(memcounts[,1:3]))
                rm(memstuff)
            }
-           chunk<-substr(chunk,memprefix+1,nchar(chunk))
-           if(any((nc<-nchar(chunk))==0)){
+           chunk<-substr(chunk,memprefix+1,nchar(chunk, "c"))
+           if(any((nc<-nchar(chunk, "c"))==0)){
                 chunk<-chunk[nc>0]
                 memcounts<-memcounts[nc>0]
            }
        }
-       
+
        chunk<-strsplit(chunk," ")
 
        newfirsts<-sapply(chunk, "[[", 1)
        newuniques<-lapply(chunk, unique)
        ulen<-sapply(newuniques,length)
        newuniques<-unlist(newuniques)
-           
+
        new.utable<-table(newuniques)
        new.ftable<-table(factor(newfirsts,levels=names(new.utable)))
        if (memory=="both"){
            new.umem<-rowsum(memcounts[rep.int(1:length(memcounts),ulen)],newuniques)
        }
-       
+
        fcounts<-rowsum( c(as.vector(new.ftable),fcounts),
 			c(names(new.ftable),fnames) )
        ucounts<-rowsum( c(as.vector(new.utable),ucounts),
@@ -90,7 +106,7 @@ summaryRprof<-function(filename = "Rprof.out", chunksize=5000,
     if (memory=="both"){
         memtotal<- round(umem/1048576,1)     ## 0.1MB
     }
-    
+
     index1<-order(-firstnum,-uniquenum)
     index2<-order(-uniquenum,-firstnum)
 
@@ -103,19 +119,19 @@ summaryRprof<-function(filename = "Rprof.out", chunksize=5000,
 
     list(by.self=rval[index1,],
              by.total=rval[index2,c(3,4, if(memory=="both") 5,1,2)],
-             sampling.time=sum(fcounts)*sample.interval)    
+             sampling.time=sum(fcounts)*sample.interval)
 }
 
 Rprof_memory_summary<-function(filename, chunksize=5000,
                    label=c(1,-1), aggregate=0, diff=FALSE,
                                exclude=NULL,sample.interval){
-    
+
     fnames<-NULL
     memcounts<-NULL
     firsts<-NULL
     labels<-vector("list",length(label))
     index<-NULL
-    
+
     repeat({
 
        chunk<-readLines(filename,n=chunksize)
@@ -125,9 +141,9 @@ Rprof_memory_summary<-function(filename, chunksize=5000,
                        "match.length")
        memstuff<-substr(chunk,2,memprefix-1)
        memcounts<-rbind(t(sapply(strsplit(memstuff,":"),as.numeric)))
-       
-       chunk<-substr(chunk,memprefix+1,nchar(chunk))
-       if(any((nc<-nchar(chunk))==0)){
+
+       chunk<-substr(chunk,memprefix+1,nchar(chunk, "c"))
+       if(any((nc<-nchar(chunk, "c"))==0)){
            memcounts<-memcounts[nc>0,]
            chunk<-chunk[nc>0]
        }
@@ -136,10 +152,10 @@ Rprof_memory_summary<-function(filename, chunksize=5000,
 
        if (length(exclude))
            chunk<-lapply(chunk, function(l) l[!(l %in% exclude)])
-       
+
        newfirsts<-sapply(chunk, "[[", 1)
        firsts<-c(firsts,newfirsts)
- 
+
        if (!aggregate && length(label)){
            for(i in 1:length(label)){
 
@@ -163,7 +179,7 @@ Rprof_memory_summary<-function(filename, chunksize=5000,
                                       function(line)
                                       paste(rev(line)[1:min(aggregate,length(line))],
                                             collapse=":")))
-               
+
            } else {
                index<-c(index, sapply(chunk,
                                       function(line)
@@ -171,7 +187,7 @@ Rprof_memory_summary<-function(filename, chunksize=5000,
                                             collapse=":")))
            }
        }
-       
+
 
        if (length(chunk)<chunksize)
            break
@@ -180,8 +196,8 @@ Rprof_memory_summary<-function(filename, chunksize=5000,
     if (length(memcounts)==0)
         stop("no events were recorded")
 
-    
-    
+
+
     memcounts<-as.data.frame(memcounts)
     names(memcounts)<-c("vsize.small","vsize.large","nodes","duplications")
     if (!aggregate){
@@ -189,7 +205,7 @@ Rprof_memory_summary<-function(filename, chunksize=5000,
         names(labels)<-paste("stack",label,sep=":")
         memcounts<-cbind(memcounts,labels)
     }
-    
+
     if (diff)
         memcounts[-1,1:3]<-pmax(0,apply(memcounts[,1:3],2,diff))
 
@@ -208,10 +224,10 @@ Rprof_memory_summary<-function(filename, chunksize=5000,
                                                    ))
                                            )
                       )
-    
+
 
     return(memcounts)
-    
-    
-    
+
+
+
 }
