@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995-1996 Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997-2006 The R Development Core Team
+ *  Copyright (C) 1997-2008 The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,11 +14,13 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
-/* <UTF8> char here is handled as a whole string */
+/* <UTF8> char here is handled as a whole string, but
+   use of translateChar assumes that strings are native.
+ */
 
 /*
   This is an effort to merge the 3 different dynload.c files in the
@@ -164,11 +166,25 @@ DL_FUNC R_dlsym(DllInfo *dll, char const *name,
 void attribute_hidden
 InitDynload()
 {
-   DllInfo *dll;
-   int which = addDLL(strdup("base"), "base", NULL);
-   dll = &LoadedDLL[which];
-   R_init_base(dll);
-   InitFunctionHashing();
+    DllInfo *dll;
+    int which = addDLL(strdup("base"), "base", NULL);
+    dll = &LoadedDLL[which];
+    R_init_base(dll);
+    InitFunctionHashing();
+}
+
+/* returns DllInfo used by the embedding application.
+   the underlying "(embedding)" entry is created if not present */
+DllInfo *R_getEmbeddingDllInfo()
+{
+    DllInfo *dll = R_getDllInfo("(embedding)");
+    if (dll == NULL) {
+	int which = addDLL(strdup("(embedding)"), "(embedding)", NULL);
+	dll = &LoadedDLL[which];
+	/* make sure we don't attempt dynamic lookup */
+	R_useDynamicSymbols(dll, FALSE);
+    }
+    return dll;
 }
 
 #ifdef UNUSED
@@ -181,11 +197,11 @@ getBaseDllInfo()
 
 Rboolean R_useDynamicSymbols(DllInfo *info, Rboolean value)
 {
-  Rboolean old;
-  old = info->useDynamicLookup;
-  info->useDynamicLookup = value;
+    Rboolean old;
+    old = info->useDynamicLookup;
+    info->useDynamicLookup = value;
 
-  return(old);
+    return(old);
 }
 
 static void 
@@ -243,11 +259,12 @@ R_registerRoutines(DllInfo *info, const R_CMethodDef * const croutines,
     if(info == NULL)
 	error(_("R_RegisterRoutines called with invalid DllInfo object."));
 
-
-    info->useDynamicLookup = TRUE; /* Default is to look in registered and then dynamic.
-                                      Potentially change in the future to be only registered
-                                      if there are any registered values.
-                                    */
+    /* Default is to look in registered and then dynamic (unless
+       the is no handle such as in "base" or "embedded")
+       Potentially change in the future to be only registered
+       if there are any registered values.
+    */
+    info->useDynamicLookup = (info->handle)?TRUE:FALSE;
 
     if(croutines) {
 	for(num=0; croutines[num].name != NULL; num++) {;}
@@ -327,9 +344,9 @@ R_addFortranRoutine(DllInfo *info,
     sym->fun = croutine->fun;
     sym->numArgs = croutine->numArgs > -1 ? croutine->numArgs : -1;
     if(croutine->types)
-       R_setPrimitiveArgTypes(croutine, sym);
+	R_setPrimitiveArgTypes(croutine, sym);
     if(croutine->styles)
-       R_setArgStyles(croutine, sym);
+	R_setArgStyles(croutine, sym);
 }
 
 static void
@@ -352,9 +369,9 @@ R_addCRoutine(DllInfo *info, const R_CMethodDef * const croutine,
     sym->fun = croutine->fun;
     sym->numArgs = croutine->numArgs > -1 ? croutine->numArgs : -1;
     if(croutine->types)
-       R_setPrimitiveArgTypes(croutine, sym);
+	R_setPrimitiveArgTypes(croutine, sym);
     if(croutine->styles)
-       R_setArgStyles(croutine, sym);
+	R_setArgStyles(croutine, sym);
 
 }
 
@@ -433,8 +450,7 @@ R_callDLLUnload(DllInfo *dllInfo)
 
     snprintf(buf, 1024, "R_unload_%s", dllInfo->name);
     f = (DllInfoUnloadCall) R_dlsym(dllInfo, buf, &symbol);
-    if(f)
-       f(dllInfo);
+    if(f) f(dllInfo);
 
     return(TRUE);
 }
@@ -443,7 +459,7 @@ R_callDLLUnload(DllInfo *dllInfo)
 	/* Returns 1 if the DLL was found and removed from */
 	/* the list and returns 0 otherwise. */
 
-static int DeleteDLL(char *path)
+static int DeleteDLL(const char *path)
 {
     int   i, loc;
 
@@ -454,7 +470,7 @@ static int DeleteDLL(char *path)
 	}
     }
     return 0;
- found:
+found:
 #ifdef CACHE_DLL_SYM
     if(R_osDynSymbol->deleteCachedSymbols)
         R_osDynSymbol->deleteCachedSymbols(&LoadedDLL[loc]);
@@ -512,7 +528,8 @@ static char DLLerror[DLLerrBUFSIZE] = "";
 	/* or if dlopen fails for some reason. */
 
 
-static DllInfo* AddDLL(char *path, int asLocal, int now)
+static DllInfo* AddDLL(const char *path, int asLocal, int now,
+		       const char *DLLsearchpath)
 {
     HINSTANCE handle;
     DllInfo *info = NULL;
@@ -523,7 +540,7 @@ static DllInfo* AddDLL(char *path, int asLocal, int now)
 	return NULL;
     }
 
-    handle = R_osDynSymbol->loadLibrary(path, asLocal, now);
+    handle = R_osDynSymbol->loadLibrary(path, asLocal, now, DLLsearchpath);
 
     if(handle == NULL) {
         R_osDynSymbol->getError(DLLerror, DLLerrBUFSIZE);
@@ -608,7 +625,7 @@ addDLL(char *dpath, char *DLLname, HINSTANCE handle)
     if(name == NULL) {
 	strcpy(DLLerror, _("could not allocate space for 'name'"));
 	if(handle)
-	   R_osDynSymbol->closeLibrary(handle);
+	    R_osDynSymbol->closeLibrary(handle);
 	free(dpath);
 	return 0;
     }
@@ -758,11 +775,10 @@ R_dlsym(DllInfo *info, char const *name,
     DL_FUNC f;
 
     f = R_getDLLRegisteredSymbol(info, name, symbol);
-    if(f)
-	return(f);
+    if(f) return(f);
 
-    if(info->useDynamicLookup == FALSE)
-	return(NULL);
+
+    if(info->useDynamicLookup == FALSE) return(NULL);
 
 #ifdef HAVE_NO_SYMBOL_UNDERSCORE
     snprintf(buf, MAXIDSIZE+1, "%s", name);
@@ -772,18 +788,25 @@ R_dlsym(DllInfo *info, char const *name,
 
 #ifdef HAVE_F77_UNDERSCORE
     if(symbol && symbol->type == R_FORTRAN_SYM) {
-	buf[strlen(buf)+1] = '\0';
-	buf[strlen(buf)] = '_';
-    }
-#endif
-#ifdef HAVE_F77_EXTRA_UNDERSCORE
-    if(strchr(name, '_') && symbol && symbol->type == R_FORTRAN_SYM) {
-	buf[strlen(buf)+1] = '\0';
-	buf[strlen(buf)] = '_';
+	strcat(buf, "_");
+# ifdef HAVE_F77_EXTRA_UNDERSCORE
+	if(strchr(name, '_')) strcat(buf, "_");
+# endif
     }
 #endif
 
-    return (DL_FUNC) R_osDynSymbol->dlsym(info, buf);
+    f = (DL_FUNC) R_osDynSymbol->dlsym(info, buf);
+#ifdef HAVE_F77_UNDERSCORE
+    if (!f && symbol && symbol->type == R_ANY_SYM) {
+	strcat(buf, "_");
+# ifdef HAVE_F77_EXTRA_UNDERSCORE
+	if(strchr(name, '_')) strcat(buf, "_");
+# endif
+	f = (DL_FUNC) R_osDynSymbol->dlsym(info, buf);
+    }
+#endif
+    
+    return f;
 }
 
 	/* R_FindSymbol checks whether one of the libraries */
@@ -809,9 +832,7 @@ DL_FUNC R_FindSymbol(char const *name, char const *pkg,
     if(R_osDynSymbol->lookupCachedSymbol)
 	fcnptr = R_osDynSymbol->lookupCachedSymbol(name, pkg, all);
 
-    if(fcnptr)
-	return(fcnptr);
-
+    if(fcnptr) return(fcnptr);
 
     /* The following is not legal ANSI C. */
     /* It is only meant to be used in systems supporting */
@@ -845,7 +866,7 @@ DL_FUNC R_FindSymbol(char const *name, char const *pkg,
 }
 
 
-static void GetFullDLLPath(SEXP call, char *buf, char *path)
+static void GetFullDLLPath(SEXP call, char *buf, const char *const path)
 {
     R_osDynSymbol->getFullDLLPath(call, buf, path);
 }
@@ -875,13 +896,13 @@ SEXP attribute_hidden do_dynload(SEXP call, SEXP op, SEXP args, SEXP env)
 
     checkArity(op,args);
     if (!isString(CAR(args)) || length(CAR(args)) < 1)
-	errorcall(call, _("character argument expected"));
+	error(_("character argument expected"));
     GetFullDLLPath(call, buf, translateChar(STRING_ELT(CAR(args), 0)));
     /* AddDLL does this DeleteDLL(buf); */
-    info = AddDLL(buf, LOGICAL(CADR(args))[0], LOGICAL(CADDR(args))[0]);
+    info = AddDLL(buf, LOGICAL(CADR(args))[0], LOGICAL(CADDR(args))[0],
+		  translateChar(STRING_ELT(CADDDR(args), 0)));
     if(!info)
-	errorcall(call, _("unable to load shared library '%s':\n  %s"),
-		  buf, DLLerror);
+	error(_("unable to load shared library '%s':\n  %s"), buf, DLLerror);
     return(Rf_MakeDLLInfo(info));
 }
 
@@ -891,15 +912,15 @@ SEXP attribute_hidden do_dynunload(SEXP call, SEXP op, SEXP args, SEXP env)
 
     checkArity(op,args);
     if (!isString(CAR(args)) || length(CAR(args)) < 1)
-	errorcall(call, _("character argument expected"));
+	error(_("character argument expected"));
     GetFullDLLPath(call, buf, translateChar(STRING_ELT(CAR(args), 0)));
     if(!DeleteDLL(buf))
-	errorcall(call, _("dynamic/shared library '%s\' was not loaded"),
-		  buf);
+	error(_("dynamic/shared library '%s\' was not loaded"), buf);
     return R_NilValue;
 }
 
-int R_moduleCdynload(char *module, int local, int now)
+attribute_hidden
+int R_moduleCdynload(const char *module, int local, int now)
 {
     char dllpath[PATH_MAX], *p = getenv("R_HOME");
     DllInfo *res;
@@ -907,12 +928,12 @@ int R_moduleCdynload(char *module, int local, int now)
     if(!p) return 0;
 #ifdef R_ARCH
     snprintf(dllpath, PATH_MAX, "%s%smodules%s%s%s%s%s", p, FILESEP, FILESEP,
-	    R_ARCH, FILESEP, module, SHLIB_EXT);
+	     R_ARCH, FILESEP, module, SHLIB_EXT);
 #else
     snprintf(dllpath, PATH_MAX, "%s%smodules%s%s%s", p, FILESEP, FILESEP,
-            module, SHLIB_EXT);
+	     module, SHLIB_EXT);
 #endif
-    res = AddDLL(dllpath, local, now);
+    res = AddDLL(dllpath, local, now, "");
     if(!res)
 	warning(_("unable to load shared library '%s':\n  %s"),
 		dllpath, DLLerror);
@@ -928,18 +949,14 @@ int R_moduleCdynload(char *module, int local, int now)
 static SEXP
 Rf_MakeNativeSymbolRef(DL_FUNC f)
 {                     
-  SEXP ref, klass;
+    SEXP ref, klass;
 
-  /* The (void *) here is illegal C */
-  PROTECT(ref = R_MakeExternalPtr((void *) f, Rf_install("native symbol"),
-				  R_NilValue));
-
-  PROTECT(klass = allocVector(STRSXP, 1));
-  SET_STRING_ELT(klass, 0, mkChar("NativeSymbol"));
-  setAttrib(ref, R_ClassSymbol, klass);
-
-  UNPROTECT(2);
-  return(ref);
+    PROTECT(ref = R_MakeExternalPtrFn(f, Rf_install("native symbol"),
+				      R_NilValue));
+    PROTECT(klass = mkString("NativeSymbol"));
+    setAttrib(ref, R_ClassSymbol, klass);
+    UNPROTECT(2);
+    return(ref);
 }
 
 static void 
@@ -954,26 +971,25 @@ freeRegisteredNativeSymbolCopy(SEXP ref)
 static SEXP
 Rf_MakeRegisteredNativeSymbol(R_RegisteredNativeSymbol *symbol)
 {                     
-  SEXP ref, klass;
-  R_RegisteredNativeSymbol *copy;
-  copy = (R_RegisteredNativeSymbol *) malloc(1 * sizeof(R_RegisteredNativeSymbol));
-  if(!copy) {
-     error(_("cannot allocate memory for registered native symbol (%d bytes)"),
-	     (int) sizeof(R_RegisteredNativeSymbol));
-  }
-  *copy = *symbol;
+    SEXP ref, klass;
+    R_RegisteredNativeSymbol *copy;
+    copy = (R_RegisteredNativeSymbol *) malloc(1 * sizeof(R_RegisteredNativeSymbol));
+    if(!copy) {
+	error(_("cannot allocate memory for registered native symbol (%d bytes)"),
+	      (int) sizeof(R_RegisteredNativeSymbol));
+    }
+    *copy = *symbol;
 
-  PROTECT(ref = R_MakeExternalPtr((void *) copy, 
-                                  Rf_install("registered native symbol"),
-                                  R_NilValue));
-  R_RegisterCFinalizer(ref, freeRegisteredNativeSymbolCopy);
+    PROTECT(ref = R_MakeExternalPtr(copy, 
+				    Rf_install("registered native symbol"),
+				    R_NilValue));
+    R_RegisterCFinalizer(ref, freeRegisteredNativeSymbolCopy);
 
-  PROTECT(klass = allocVector(STRSXP, 1));
-  SET_STRING_ELT(klass, 0, mkChar("RegisteredNativeSymbol"));
-  setAttrib(ref, R_ClassSymbol, klass);
+    PROTECT(klass = mkString("RegisteredNativeSymbol"));
+    setAttrib(ref, R_ClassSymbol, klass);
 
-  UNPROTECT(2);
-  return(ref);
+    UNPROTECT(2);
+    return(ref);
 }
 
 
@@ -1062,7 +1078,7 @@ Rf_MakeDLLInfo(DllInfo *info)
 SEXP attribute_hidden
 R_getSymbolInfo(SEXP sname, SEXP spackage, SEXP withRegistrationInfo)
 {
-    char *package, *name;
+    const char *package, *name;
     R_RegisteredNativeSymbol symbol = {R_ANY_SYM, {NULL}, NULL};
     SEXP sym = R_NilValue;
     DL_FUNC f = NULL;
@@ -1072,23 +1088,22 @@ R_getSymbolInfo(SEXP sname, SEXP spackage, SEXP withRegistrationInfo)
     name = translateChar(STRING_ELT(sname, 0));
 
     if(length(spackage)) {
-	if(TYPEOF(spackage) == STRSXP) {
+	if(TYPEOF(spackage) == STRSXP)
 	    package = translateChar(STRING_ELT(spackage, 0));
-	}
 	else if(TYPEOF(spackage) == EXTPTRSXP &&
 		R_ExternalPtrTag(spackage) == Rf_install("DLLInfo")) {
-	    f = R_dlsym((DllInfo *)R_ExternalPtrAddr(spackage), name, &symbol);
+	    f = R_dlsym((DllInfo *) R_ExternalPtrAddr(spackage), name, &symbol);
 	    package = NULL;
-	} else {
+	} else
 	    error(_("must pass package name or DllInfo reference"));
-	}
     }
 
     if(package)
-      f = R_FindSymbol(name, package, &symbol);
+	f = R_FindSymbol(name, package, &symbol);
 
     if(f)
-      sym = createRSymbolObject(sname, f, &symbol, LOGICAL(withRegistrationInfo)[0]);
+	sym = createRSymbolObject(sname, f, &symbol, 
+				  LOGICAL(withRegistrationInfo)[0]);
 
     return(sym);
 }
@@ -1096,16 +1111,16 @@ R_getSymbolInfo(SEXP sname, SEXP spackage, SEXP withRegistrationInfo)
 SEXP attribute_hidden
 R_getDllTable()
 {
-	int i;
-	SEXP ans;
+    int i;
+    SEXP ans;
 
-	PROTECT(ans = allocVector(VECSXP, CountDLL));
-	for(i = 0; i < CountDLL; i++) {
-		SET_VECTOR_ELT(ans, i, Rf_MakeDLLInfo(&(LoadedDLL[i])));
-	}
-	setAttrib(ans, R_ClassSymbol, mkString("DLLInfoList"));
-	UNPROTECT(1);
-	return(ans);
+    PROTECT(ans = allocVector(VECSXP, CountDLL));
+    for(i = 0; i < CountDLL; i++) {
+	SET_VECTOR_ELT(ans, i, Rf_MakeDLLInfo(&(LoadedDLL[i])));
+    }
+    setAttrib(ans, R_ClassSymbol, mkString("DLLInfoList"));
+    UNPROTECT(1);
+    return(ans);
 }
 
 static SEXP
@@ -1129,8 +1144,8 @@ createRSymbolObject(SEXP sname, DL_FUNC f, R_RegisteredNativeSymbol *symbol,
 
     SET_VECTOR_ELT(sym, 1, 
                    withRegistrationInfo && symbol && symbol->symbol.c && symbol->dll 
-                     ? Rf_MakeRegisteredNativeSymbol(symbol)  
-                     : Rf_MakeNativeSymbolRef(f));
+		   ? Rf_MakeRegisteredNativeSymbol(symbol)  
+		   : Rf_MakeNativeSymbolRef(f));
     SET_STRING_ELT(names, 1, mkChar("address"));
     if(symbol->dll)
 	SET_VECTOR_ELT(sym, 2, Rf_MakeDLLInfo(symbol->dll));
@@ -1241,21 +1256,18 @@ R_getRoutineSymbols(NativeSymbolType type, DllInfo *info)
 SEXP attribute_hidden
 R_getRegisteredRoutines(SEXP dll)
 {
-
     DllInfo *info;
     SEXP ans, snames;
     int i;
     const char * const names[] = {".C", ".Call", ".Fortran", ".External"};
 
     if(TYPEOF(dll) != EXTPTRSXP &&
-       R_ExternalPtrTag(dll) != Rf_install("DLLInfo")) {
+       R_ExternalPtrTag(dll) != Rf_install("DLLInfo"))
 	error(_("R_getRegisteredRoutines() expects a DllInfo reference"));
-    }
 
     info = (DllInfo *) R_ExternalPtrAddr(dll);
-    if(!info) {
-	error(_("NULL value passed for DllInfo"));
-    }
+    if(!info) error(_("NULL value passed for DllInfo"));
+
 
     PROTECT(ans = allocVector(VECSXP, 4));
 
@@ -1265,9 +1277,8 @@ R_getRegisteredRoutines(SEXP dll)
     SET_VECTOR_ELT(ans, 3, R_getRoutineSymbols(R_EXTERNAL_SYM, info));
 
     PROTECT(snames = allocVector(STRSXP, 4));
-    for(i = 0; i < 4; i++) {
+    for(i = 0; i < 4; i++)
 	SET_STRING_ELT(snames, i, mkChar(names[i]));
-    }
     setAttrib(ans, R_NamesSymbol, snames);
     UNPROTECT(2);
     return(ans);
@@ -1281,13 +1292,12 @@ void InitFunctionHashing()
 
 attribute_hidden
 DL_FUNC R_FindSymbol(char const *name, char const *pkg,
-                       R_RegisteredNativeSymbol *symbol)
+		     R_RegisteredNativeSymbol *symbol)
 {
     int i;
-    for(i=0 ; CFunTab[i].name ; i++)
-	if(!strcmp(name, CFunTab[i].name))
-	    return CFunTab[i].func;
-    return (DL_FUNC)0;
+    for(i = 0; CFunTab[i].name; i++)
+	if(!strcmp(name, CFunTab[i].name)) return CFunTab[i].func;
+    return (DL_FUNC) 0;
 }
 
 SEXP attribute_hidden do_dynload(SEXP call, SEXP op, SEXP args, SEXP env)
@@ -1330,7 +1340,7 @@ R_getRegisteredRoutines(SEXP dll)
 
 static SEXP CEntryTable = NULL;
 
-static SEXP get_package_CEntry_table(char *package)
+static SEXP get_package_CEntry_table(const char *package)
 {
     SEXP penv, pname;
 
@@ -1348,16 +1358,16 @@ static SEXP get_package_CEntry_table(char *package)
 }
 
 
-void R_RegisterCCallable(char *package, char *name, DL_FUNC fptr)
+void R_RegisterCCallable(const char *package, const char *name, DL_FUNC fptr)
 {
     SEXP penv = get_package_CEntry_table(package);
-    SEXP eptr = R_MakeExternalPtr((void *) fptr, R_NilValue, R_NilValue);
+    SEXP eptr = R_MakeExternalPtrFn(fptr, R_NilValue, R_NilValue);
     PROTECT(eptr);
     defineVar(install(name), eptr, penv);
     UNPROTECT(1);
 }
 
-DL_FUNC R_GetCCallable(char *package, char *name)
+DL_FUNC R_GetCCallable(const char *package, const char *name)
 {
     SEXP penv = get_package_CEntry_table(package);
     SEXP eptr = findVarInFrame(penv, install(name));
@@ -1365,5 +1375,5 @@ DL_FUNC R_GetCCallable(char *package, char *name)
 	error(_("function '%s' not provided by package '%s'"), name, package);
     else if (TYPEOF(eptr) != EXTPTRSXP) 
 	error(_("table entry must be an external pointer"));
-    return (DL_FUNC) R_ExternalPtrAddr(eptr);
+    return R_ExternalPtrAddrFn(eptr);
 }
