@@ -202,8 +202,13 @@ double R_pow(double x, double y) /* = x ^ y */
 	/* y < 0 */ return(R_PosInf);
     }
     if (R_FINITE(x) && R_FINITE(y))
-/* work around a bug in May 2007 snapshots of gcc pre-4.3.0 */
-#if __GNUC__ == 4 && __GNUC_MINOR__ == 3
+/* work around a bug in May 2007 snapshots of gcc pre-4.3.0, also
+   present in the release version.  If compiled with, say, -g -O3
+   on x86_64 Linux this compiles to a call to sqrtsd and gives
+   100^0.5 as 3.162278.  -g is needed, as well as -O2 or higher.
+   example(pbirthday) will fail.
+ */
+#if __GNUC__ == 4 && __GNUC_MINOR__ >= 3
 	return (y == 2.0) ? x*x : pow(x, y);
 #else
 	return (y == 2.0) ? x*x : ((y == 0.5) ? sqrt(x) : pow(x, y));
@@ -1316,14 +1321,14 @@ SEXP attribute_hidden do_atan(SEXP call, SEXP op, SEXP args, SEXP env)
 /* The S4 Math2 group, round and signif */
 SEXP attribute_hidden do_Math2(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    SEXP res;
-    int n;
+    SEXP res, ap;
+    int n, nprotect = 1;
 
     if (length(args) >= 2 && 
         isSymbol(CADR(args)) && R_isMissing(CADR(args), env)) {
         double digits = 0;
 	if(PRIMVAL(op) == 10004) digits = 6.0;
-	SETCAR(CDR(args), ScalarReal(digits));
+	PROTECT(args = list2(CAR(args), ScalarReal(digits))); nprotect++;
     }
 
     PROTECT(args = evalListKeepMissing(args, env));
@@ -1343,11 +1348,21 @@ SEXP attribute_hidden do_Math2(SEXP call, SEXP op, SEXP args, SEXP env)
 	    double digits = 0.0;
 	    if(PRIMVAL(op) == 10004) digits = 6.0;
 	    SETCDR(args, CONS(ScalarReal(digits), R_NilValue));
-	} else if (length(CADR(args)) == 0)
-            errorcall(call, _("invalid second argument of length 0"));
+	} else {
+	    /* If named, do argument matching by name */
+	    if (TAG(args) != R_NilValue || TAG(CDR(args)) != R_NilValue) {
+		PROTECT(ap = CONS(R_NilValue, list1(R_NilValue)));
+		SET_TAG(ap,  install("x"));
+		SET_TAG(CDR(ap), install("digits"));
+		PROTECT(args = matchArgs(ap, args, call));
+		nprotect +=2;
+	    }
+	    if (length(CADR(args)) == 0)
+		errorcall(call, _("invalid second argument of length 0"));
+	}
 	res = do_math2(call, op, args, env);
     }
-    UNPROTECT(1);
+    UNPROTECT(nprotect);
     return res;
 }
 
@@ -1380,7 +1395,7 @@ SEXP attribute_hidden do_log(SEXP call, SEXP op, SEXP args, SEXP env)
 #else
         double e = exp(1.);
 #endif
-	SETCAR(CDR(args), ScalarReal(e));
+	PROTECT(args = list2(CAR(args), ScalarReal(e))); nprotect++;
     }
     PROTECT(args = evalListKeepMissing(args, env));
     n = length(args);
