@@ -2,7 +2,7 @@
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
  *  Copyright (C) 1998-2001   The R Development Core Team
- *  Copyright (C) 2002--2006  The R Foundation
+ *  Copyright (C) 2002--2008  The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -58,19 +58,23 @@ SEXP GetColNames(SEXP dimnames)
 
 SEXP attribute_hidden do_matrix(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    SEXP vals, snr, snc;
-    int nr, nc, byrow, lendat;
+    SEXP vals, ans, snr, snc, dimnames = R_NilValue;
+    int nr, nc, byrow, lendat,nargs;
 
     checkArity(op, args);
+    nargs = length(args);
+    if(nargs < 4 || nargs > 5)
+	error("incorrect number of arguments to 'matrix'");
     vals = CAR(args);
     snr = CADR(args);
     snc = CADDR(args);
-    byrow = asLogical(CADR(CDDR(args)));
-    if (byrow == NA_INTEGER)
-	error(_("invalid 'byrow' value"));
-
     if (!isNumeric(snr) || !isNumeric(snc))
 	error(_("non-numeric matrix extent"));
+    byrow = asLogical(CADDDR(args));
+    if (byrow == NA_INTEGER)
+	error(_("invalid '%s' argument"), "byrow");
+    if(nargs == 5) dimnames = CAD4R(args);
+
 
     lendat = length(vals);
     nr = asInteger(snr);
@@ -101,34 +105,34 @@ SEXP attribute_hidden do_matrix(SEXP call, SEXP op, SEXP args, SEXP rho)
     if ((double)nr * (double)nc > INT_MAX)
 	error(_("too many elements specified"));
 
-    PROTECT(snr = allocMatrix(TYPEOF(vals), nr, nc));
+    PROTECT(ans = allocMatrix(TYPEOF(vals), nr, nc));
     if(lendat) {
 	if (isVector(vals))
-	    copyMatrix(snr, vals, byrow);
+	    copyMatrix(ans, vals, byrow);
 	else
-	    copyListMatrix(snr, vals, byrow);
+	    copyListMatrix(ans, vals, byrow);
     } else if (isVector(vals)) { /* fill with NAs */
 	int i, j;
 	switch(TYPEOF(vals)) {
 	case STRSXP:
 	    for (i = 0; i < nr; i++)
 		for (j = 0; j < nc; j++)
-		    SET_STRING_ELT(snr, i + j * nr, NA_STRING);
+		    SET_STRING_ELT(ans, i + j * nr, NA_STRING);
 	    break;
 	case LGLSXP:
 	    for (i = 0; i < nr; i++)
 		for (j = 0; j < nc; j++)
-		    LOGICAL(snr)[i + j * nr] = NA_LOGICAL;
+		    LOGICAL(ans)[i + j * nr] = NA_LOGICAL;
 	    break;
 	case INTSXP:
 	    for (i = 0; i < nr; i++)
 		for (j = 0; j < nc; j++)
-		    INTEGER(snr)[i + j * nr] = NA_INTEGER;
+		    INTEGER(ans)[i + j * nr] = NA_INTEGER;
 	    break;
 	case REALSXP:
 	    for (i = 0; i < nr; i++)
 		for (j = 0; j < nc; j++)
-		    REAL(snr)[i + j * nr] = NA_REAL;
+		    REAL(ans)[i + j * nr] = NA_REAL;
 	    break;
 	case CPLXSXP:
 	    {
@@ -137,21 +141,22 @@ SEXP attribute_hidden do_matrix(SEXP call, SEXP op, SEXP args, SEXP rho)
 		na_cmplx.i = 0;
 		for (i = 0; i < nr; i++)
 		    for (j = 0; j < nc; j++)
-			COMPLEX(snr)[i + j * nr] = na_cmplx;
+			COMPLEX(ans)[i + j * nr] = na_cmplx;
 	    }
 	    break;
 	case RAWSXP:
 	    for (i = 0; i < nr; i++)
 		for (j = 0; j < nc; j++)
-		    RAW(snr)[i + j * nr] = 0;
+		    RAW(ans)[i + j * nr] = 0;
 	    break;
 	default:
 	    /* don't fill with anything */
 	    ;
 	}
     }
+    if(!isNull(dimnames)) ans = dimnamesgets(ans, dimnames);
     UNPROTECT(1);
-    return snr;
+    return ans;
 }
 
 
@@ -234,7 +239,7 @@ SEXP allocArray(SEXPTYPE mode, SEXP dims)
 
 SEXP DropDims(SEXP x)
 {
-    SEXP q, dims, dimnames, newnames = R_NilValue;
+    SEXP dims, dimnames, newnames = R_NilValue;
     int i, n, ndims;
 
     PROTECT(x);
@@ -260,26 +265,27 @@ SEXP DropDims(SEXP x)
     }
 
     if (n <= 1) {
-	/* We have reduced to a vector result. */
+	/* We have reduced to a vector result.
+	   If that has length one, it is ambiguous which dimnames to use,
+	   so use it if there is only one (as from R 2.7.0).
+	 */
 	if (dimnames != R_NilValue) {
-	    n = length(dims);
-	    if (TYPEOF(dimnames) == VECSXP) {
-		for (i = 0; i < n; i++) {
+	    if(LENGTH(x) != 1) {
+		for (i = 0; i < LENGTH(dims); i++) {
 		    if (INTEGER(dims)[i] != 1) {
 			newnames = VECTOR_ELT(dimnames, i);
 			break;
 		    }
 		}
-	    }
-	    else {
-		q = dimnames;
-		for (i = 0; i < n; i++) {
-		    if (INTEGER(dims)[i] != 1) {
-			newnames = CAR(q);
-			break;
+	    } else { /* drop all dims: keep names if unambiguous */
+		int cnt;
+		for(i = 0, cnt = 0; i < LENGTH(dims); i++)
+		    if(VECTOR_ELT(dimnames, i) != R_NilValue) cnt++;
+		if(cnt == 1)
+		    for (i = 0; i < LENGTH(dims); i++) {
+			newnames = VECTOR_ELT(dimnames, i);
+			if(newnames != R_NilValue) break;
 		    }
-		    q = CDR(q);
-		}
 	    }
 	}
 	PROTECT(newnames);
@@ -287,8 +293,7 @@ SEXP DropDims(SEXP x)
 	setAttrib(x, R_DimSymbol, R_NilValue);
 	setAttrib(x, R_NamesSymbol, newnames);
 	UNPROTECT(1);
-    }
-    else {
+    } else {
 	/* We have a lower dimensional array. */
 	SEXP newdims, dnn, newnamesnames = R_NilValue;
 	dnn = getAttrib(dimnames, R_NamesSymbol);
@@ -1019,7 +1024,7 @@ SEXP attribute_hidden do_aperm(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    error(_("value out of range in 'perm'"));
     for (i = 0; i < n; i++)
 	if (iip[i] == 0)
-	    error(_("invalid permutation ('perm')"));
+	    error(_("invalid '%s' argument"), "perm");
 
     /* create the stride object and permute */
 
@@ -1152,10 +1157,10 @@ SEXP attribute_hidden do_colsum(SEXP call, SEXP op, SEXP args, SEXP rho)
     p = asInteger(CAR(args)); args = CDR(args);
     NaRm = asLogical(CAR(args));
     if (n == NA_INTEGER || n < 0)
-	error(_("invalid value of 'n'"));
+	error(_("invalid '%s' argument"), "n");
     if (p == NA_INTEGER || p < 0)
-	error(_("invalid value of 'p'"));
-    if (NaRm == NA_LOGICAL) error(_("invalid value of 'na.rm'"));
+	error(_("invalid '%s' argument"), "p");
+    if (NaRm == NA_LOGICAL) error(_("invalid '%s' argument"), "na.rm");
     keepNA = !NaRm;
 
     OP = PRIMVAL(op);
