@@ -14,8 +14,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 /* <UTF8> char here is either ASCII or handled as a whole */
@@ -109,7 +109,8 @@ Should NEVER happen; please bug.report() [mkCLOSXP]"));
 
 static int isDDName(SEXP name)
 {
-    char *buf, *endp;
+    const char *buf;
+    char *endp;
     long val;
 
     buf = CHAR(name);
