@@ -15,8 +15,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 #ifdef HAVE_CONFIG_H
@@ -196,13 +196,7 @@ static SEXP duplicate1(SEXP s)
 	UNPROTECT(2);
 	break;
     case CHARSXP:
-	/* This was wrong, losing internal nuls before 2.5.0 */
-	PROTECT(s);
-	PROTECT(t = allocString(LENGTH(s)));
-	memcpy(CHAR(t), CHAR(s), LENGTH(s)+1);
-	DUPLICATE_ATTRIB(t, s);
-	markKnown(t, s);
-	UNPROTECT(2);
+        return s;
 	break;
     case EXPRSXP:
     case VECSXP:
