@@ -29,21 +29,16 @@
 
 #include <Defn.h>
 #include <Rmath.h>
-#include <Graphics.h>
+#include <R_ext/GraphicsEngine.h>
 
 SEXP attribute_hidden 
 do_getGraphicsEvent(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP prompt, onMouseDown, onMouseMove, onMouseUp, onKeybd;
-    GEDevDesc *dd;
-    NewDevDesc *nd;
+    pDevDesc nd = GEcurrentDevice()->dev;
     
     checkArity(op, args);
-    
-    dd = GEcurrentDevice();
-    nd = dd->dev;
-    
-    if (!nd->newDevStruct || !nd->getEvent) 
+    if (!nd->getEvent) 
     	error(_("graphics device does not support graphics events"));
     
     prompt = CAR(args);
@@ -90,7 +85,7 @@ static const char * mouseHandlers[] =
 {"onMouseDown", "onMouseUp", "onMouseMove"};
 
 /* used in devWindows.c and cairoDevice */
-SEXP doMouseEvent(SEXP eventRho, NewDevDesc *dd, R_MouseEvent event,
+SEXP doMouseEvent(SEXP eventRho, pDevDesc dd, R_MouseEvent event,
 		  int buttons, double x, double y)
 {
     int i;
@@ -129,7 +124,7 @@ static const char * keynames[] =
  "PgUp", "PgDn", "End", "Home", "Ins", "Del"};
 
 /* used in devWindows.c and cairoDevice */
-SEXP doKeybd(SEXP eventRho, NewDevDesc *dd, R_KeyName rkey,
+SEXP doKeybd(SEXP eventRho, pDevDesc dd, R_KeyName rkey,
 	     const char *keyname)
 {
     SEXP handler, skey, temp, result;
