@@ -505,6 +505,24 @@ static const name_value known[] = {
     {"sjis", "SHIFT_JIS"},
     {"euccn", "GB2312"},
     {"big5-hkscs", "BIG5-HKSCS"},
+#if __APPLE__
+    /* known additional Apple encodings (see locale -a) up to Mac OS X 10.5,
+       unlike other systems they correspond directly */
+    {"iso8859-1", "ISO8859-1"},
+    {"iso8859-2", "ISO8859-2"},
+    {"iso8859-4", "ISO8859-4"},
+    {"iso8859-7", "ISO8859-7"},
+    {"iso8859-9", "ISO8859-9"},
+    {"iso8859-13", "ISO8859-13"},
+    {"iso8859-15", "ISO8859-15"},
+    {"koi8-u", "KOI8-U"},
+    {"koi8-r", "KOI8-R"},
+    {"pt154", "PT154"},
+    {"us-ascii", "ASCII"},
+    {"armscii-8", "ARMSCII-8"},
+    {"iscii-dev", "ISCII-DEV"},
+    {"big5hkscs", "BIG5-HKSCS"},
+#endif
 };
 static const int known_count = (sizeof(known)/sizeof(name_value));
 
@@ -574,7 +592,8 @@ char *locale2charset(const char *locale)
     if ((locale == NULL) || (0 == strcmp(locale, "NULL")))
 	locale = setlocale(LC_CTYPE,NULL);
 
-    if (0 == strcmp(locale, "C") || 0 == strcmp(locale, "POSIX"))
+    /* in some rare circumstances Darwin may return NULL */
+    if (!locale || !strcmp(locale, "C") || !strcmp(locale, "POSIX"))
 	return ("ASCII");
 
     memset(charset,0,sizeof(charset));
@@ -670,8 +689,8 @@ char *locale2charset(const char *locale)
 	/* let's hope it is a ll_* name */
 	if (0 == strcmp(enc, "euc")) {
 	    /* This is OK as encoding names are ASCII */
-	    if(isalpha((int)enc[0]) && isalpha((int)enc[1]) 
-	       && (enc[2] == '_')) {
+	    if(isalpha((int)la_loc[0]) && isalpha((int)la_loc[1]) 
+	       && (la_loc[2] == '_')) {
 		if (0 == strncmp("ja", la_loc, 2)) return "EUC-JP";
 		if (0 == strncmp("ko", la_loc, 2)) return "EUC-KR";
 		if (0 == strncmp("zh", la_loc, 2)) return "GB2312";
@@ -680,10 +699,17 @@ char *locale2charset(const char *locale)
 	
     }
 
+#if __APPLE__
+    /* on Mac OS X *all* real locales w/o encoding part are UTF-8 locales
+       (C and POSIX are virtual and taken care of previously) */
+    return "UTF-8";
+#else
+
     if(0 == strcmp(enc, "utf8")) return "UTF-8";
 
     value = name_value_search(la_loc, guess, guess_count);
     return value == NULL ? (char *) "ASCII" : value;
+#endif
 }
 
 /*****************************************************
