@@ -374,9 +374,22 @@ SEXP attribute_hidden do_logic3(SEXP call, SEXP op, SEXP args, SEXP env)
 
     for (s = args; s != R_NilValue; s = CDR(s)) {
 	t = CAR(s);
+	/* Avoid memeory waste from coercing empty inputs, and also
+	   avoid warnings with empty lists coming from sapply */
+	if(length(t) == 0) continue;
 	/* coerceVector protects its argument so this actually works
 	   just fine */
-	if (TYPEOF(t)  != LGLSXP) t = coerceVector(t, LGLSXP);
+	if (TYPEOF(t)  != LGLSXP) {
+	    /* Coercion of integers seems reasonably safe, but for
+	       other types it is more often than not an error.
+	       One exception is perhaps the result of lapply, but
+	       then sapply was often what was intended. */
+	    if(TYPEOF(t) != INTSXP)
+		warningcall(call, 
+			    _("coercing argument of type '%s' to logical"),
+			    type2char(TYPEOF(t)));
+	    t = coerceVector(t, LGLSXP);
+	}
 	checkValues(LOGICAL(t), LENGTH(t), &haveFalse, &haveTrue, &haveNA);
     }
     if (narm)
