@@ -125,15 +125,14 @@ static int MatchVar(SEXP var1, SEXP var2)
 	return (asReal(var1) == asReal(var2));
     /* Literal Strings */
     if (isString(var1) && isString(var2))
-	return (strcmp(translateChar(STRING_ELT(var1, 0)),
-		       translateChar(STRING_ELT(var2, 0))) == 0);
+	return Seql(STRING_ELT(var1, 0), STRING_ELT(var2, 0));
     /* Nothing else matches */
     return 0;
 }
 
 
 /* InstallVar locates a ``variable'' in the model */
-/* variable list;  adding it to the list if not found. */
+/* variable list;  adding it to the global varlist if not found. */
 
 static int InstallVar(SEXP var)
 {
@@ -285,7 +284,7 @@ static void ExtractVars(SEXP formula, int checkonly)
 /* AllocTerm allocates an integer array for */
 /* bit string representation of a model term */
 
-static SEXP AllocTerm()
+static SEXP AllocTerm(void)
 {
     int i;
     SEXP term = allocVector(INTSXP, nwords);
@@ -608,7 +607,7 @@ static SEXP EncodeVars(SEXP formula)
 		c = translateChar(STRING_ELT(framenames, i));
 		for(j = 0; j < i; j++)
 		    if(!strcmp(c, translateChar(STRING_ELT(framenames, j))))
-			error(_("duplicated name '%s' in data frame using '.'"), 
+			error(_("duplicated name '%s' in data frame using '.'"),
 			      c);
 		term = AllocTerm();
 		SetBit(term, InstallVar(install(c)), 1);
@@ -760,7 +759,7 @@ SEXP attribute_hidden do_termsform(SEXP call, SEXP op, SEXP args, SEXP rho)
 	error(_("argument is not a valid model"));
 
     haveDot = FALSE;
-    
+
     PROTECT(ans = duplicate(CAR(args)));
 
     /* The formula will be returned, modified if haveDot becomes TRUE */
@@ -849,7 +848,7 @@ SEXP attribute_hidden do_termsform(SEXP call, SEXP op, SEXP args, SEXP rho)
     PROTECT(varnames = allocVector(STRSXP, nvar));
     for (v = CDR(varlist), i = 0; v != R_NilValue; v = CDR(v))
 	SET_STRING_ELT(varnames, i++, STRING_ELT(deparse1line(CAR(v), 0), 0));
-    
+
     /* Step 2b: Find and remove any offset(s) */
 
     /* first see if any of the variables are offsets */
@@ -902,12 +901,12 @@ SEXP attribute_hidden do_termsform(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    SET_VECTOR_ELT(pattern, n, CAR(call));
 	    counts[n++] = BitCount(CAR(call));
 	}
-	for (n = 0; n < nterm; n++) 
+	for (n = 0; n < nterm; n++)
 	    if(counts[n] > bitmax) bitmax = counts[n];
 	if(keepOrder) {
 	    for (n = 0; n < nterm; n++)
 		iord[n] = counts[n];
-	} else {   
+	} else {
 	    call = formula;
 	    m = 0;
 	    for (i = 0; i <= bitmax; i++) /* can order 0 occur? */
@@ -920,7 +919,7 @@ SEXP attribute_hidden do_termsform(SEXP call, SEXP op, SEXP args, SEXP rho)
 	}
 	UNPROTECT(2);
     }
-    
+
 
     /* Step 4: Compute the factor pattern for the model. */
     /* 0 - the variable does not appear in this term. */
@@ -1022,18 +1021,18 @@ SEXP attribute_hidden do_termsform(SEXP call, SEXP op, SEXP args, SEXP rho)
 	UNPROTECT(1);
     }
 
-    UNPROTECT(3);	/* keep termlabs until here */
+    UNPROTECT(2);	/* keep termlabs until here */
 
-    /* Step 6: Fix up the formula by substituting for dot, which should be 
+    /* Step 6: Fix up the formula by substituting for dot, which should be
        the framenames joined by + */
 
     if (haveDot) {
 	if(length(framenames)) {
 	    PROTECT_INDEX ind;
-	    PROTECT_WITH_INDEX(rhs = install(translateChar(STRING_ELT(framenames, 0))), 
+	    PROTECT_WITH_INDEX(rhs = install(translateChar(STRING_ELT(framenames, 0))),
 			       &ind);
 	    for (i = 1; i < LENGTH(framenames); i++) {
-		REPROTECT(rhs = lang3(plusSymbol, rhs, 
+		REPROTECT(rhs = lang3(plusSymbol, rhs,
 				      install(translateChar(STRING_ELT(framenames, i)))),
 			  ind);
 	    }
@@ -1046,15 +1045,15 @@ SEXP attribute_hidden do_termsform(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    error(_("'.' in formula and no 'data' argument"));
 	}
     }
-    
+
     SETCAR(a, allocVector(INTSXP, nterm));
     n = 0;
     {
 	int *ia = INTEGER(CAR(a)), *iord = INTEGER(ord);
-	for (call = formula; call != R_NilValue; call = CDR(call), n++) 
+	for (call = formula; call != R_NilValue; call = CDR(call), n++)
 	    ia[n] = iord[n];
     }
-    
+
     SET_TAG(a, install("order"));
     a = CDR(a);
 
@@ -1072,7 +1071,7 @@ SEXP attribute_hidden do_termsform(SEXP call, SEXP op, SEXP args, SEXP rho)
 
     SETCDR(a, R_NilValue);  /* truncate if necessary */
 
-    UNPROTECT(3);
+    UNPROTECT(4);
     return ans;
 }
 
@@ -1264,8 +1263,8 @@ SEXP attribute_hidden do_updateform(SEXP call, SEXP op, SEXP args, SEXP rho)
 
     SET_ATTRIB(_new, R_NilValue);
     SET_OBJECT(_new, 0);
-    setAttrib(_new, R_DotEnvSymbol, getAttrib(old, R_DotEnvSymbol)); 
-    
+    setAttrib(_new, R_DotEnvSymbol, getAttrib(old, R_DotEnvSymbol));
+
     return _new;
 }
 
@@ -1641,10 +1640,10 @@ SEXP attribute_hidden do_modelmatrix(SEXP call, SEXP op, SEXP args, SEXP rho)
     /* This section of the code checks the types of the variables
        in the model frame.  Note that it should really only check
        the variables if they appear in a term in the model.
-       Because it does not, we need to allow other types here, as they 
+       Because it does not, we need to allow other types here, as they
        might well occur on the LHS.
-       The R code converts all character variables in the model frame to 
-       factors, so the only types that ought to be here are logical, 
+       The R code converts all character variables in the model frame to
+       factors, so the only types that ought to be here are logical,
        integer (including factor), numeric and complex.
      */
 
@@ -1811,12 +1810,12 @@ SEXP attribute_hidden do_modelmatrix(SEXP call, SEXP op, SEXP args, SEXP rho)
     if (intrcept) INTEGER(assign)[k++] = 0;
     for (j = 0; j < nterms; j++) {
 	if(INTEGER(count)[j] <= 0)
-	    warning(_("problem with term %d in model.matrix: no columns are assigned"), 
+	    warning(_("problem with term %d in model.matrix: no columns are assigned"),
 		      j+1);
 	for (i = 0; i < INTEGER(count)[j]; i++)
 	    INTEGER(assign)[k++] = j+1;
     }
-    
+
 
     /* Create column labels for the matrix columns. */
 
@@ -1937,7 +1936,7 @@ SEXP attribute_hidden do_modelmatrix(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    var_i = VECTOR_ELT(variable, i);
 #ifdef R_MEMORY_PROFILING
 	    if (TRACE(var_i)){
-	       memtrace_report(var_i, x);	    
+	       memtrace_report(var_i, x);
 	       SET_TRACE(x, 1);
 	    }
 #endif
