@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997-2007 Robert Gentleman, Ross Ihaka and the R core team.
+ *  Copyright (C) 1997-2008 Robert Gentleman, Ross Ihaka and the R core team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -47,7 +47,6 @@
 #include <Defn.h>
 #include <Rmath.h>
 #include <Graphics.h>		/* "GPar" structure + COMMENTS */
-#include <Rdevices.h>
 
 typedef struct {
     char *name;
@@ -238,19 +237,19 @@ static void BoundsCheck(double x, double a, double b, const char *s)
 
 /* These will be defined differently for Specify() and Specify2() : */
 /* <FIXME>  do not need separate macros for a = b = c and b = a = c */
-#define R_DEV__(_P_) Rf_dpptr(dd)->_P_ = Rf_gpptr(dd)->_P_
-#define R_DEV_2(_P_) Rf_gpptr(dd)->_P_ = Rf_dpptr(dd)->_P_
+#define R_DEV__(_P_) dpptr(dd)->_P_ = gpptr(dd)->_P_
+#define R_DEV_2(_P_) gpptr(dd)->_P_ = dpptr(dd)->_P_
 /* In Emacs : -- only inside Specify() :
  *  (query-replace-regexp
-     "Rf_dpptr(dd)->\\([][A-Za-z0-9]+\\) = Rf_gpptr(dd)->\\(\\1\\)"
+     "dpptr(dd)->\\([][A-Za-z0-9]+\\) = gpptr(dd)->\\(\\1\\)"
      "R_DEV__(\\1)" nil nil nil)
 
    (query-replace-regexp
-     "Rf_gpptr(dd)->\\([][A-Za-z0-9]+\\) = Rf_dpptr(dd)->\\(\\1\\)"
+     "gpptr(dd)->\\([][A-Za-z0-9]+\\) = dpptr(dd)->\\(\\1\\)"
      "R_DEV_2(\\1)" nil nil nil)
 */
 
-static void Specify(const char *what, SEXP value, DevDesc *dd, SEXP call)
+static void Specify(const char *what, SEXP value, pGEDevDesc dd, SEXP call)
 {
 /* If you ADD a NEW par, then do NOT forget to update the code in
  *			 ../library/base/R/par.R
@@ -283,7 +282,8 @@ static void Specify(const char *what, SEXP value, DevDesc *dd, SEXP call)
 /*	  ------------
  *--- now, these are *different* from  "Specify2() use" : */
     else if (streql(what, "bg")) {
-	lengthCheck(what, value, 1, call);	ix = RGBpar(value, 0);
+	lengthCheck(what, value, 1, call);	
+	ix = RGBpar3(value, 0, dpptr(dd)->bg);
 	/*	naIntCheck(ix, what); */
 	R_DEV__(bg) = ix;
 	R_DEV__(new) = FALSE;
@@ -297,7 +297,8 @@ static void Specify(const char *what, SEXP value, DevDesc *dd, SEXP call)
 
     else if (streql(what, "fg")) {
 	/* par(fg=) sets BOTH "fg" and "col" */
-	lengthCheck(what, value, 1, call);	ix = RGBpar(value, 0);
+	lengthCheck(what, value, 1, call);	
+	ix = RGBpar3(value, 0, dpptr(dd)->bg);
 	/*	naIntCheck(ix, what); */
 	R_DEV__(col) = R_DEV__(fg) = ix;
     }
@@ -306,7 +307,7 @@ static void Specify(const char *what, SEXP value, DevDesc *dd, SEXP call)
 /*--- and these are "Specify() only" {i.e. par(nam = val)} : */
     else if (streql(what, "ask")) {
 	lengthCheck(what, value, 1, call);	ix = asLogical(value);
-	R_DEV__(ask) = (ix == 1);/* NA |-> FALSE */
+	dd->ask = (ix == 1);/* NA |-> FALSE */
     }
     else if (streql(what, "fig")) {
 	value = coerceVector(value, REALSXP);
@@ -343,8 +344,8 @@ static void Specify(const char *what, SEXP value, DevDesc *dd, SEXP call)
 	ss = translateChar(STRING_ELT(value, 0));
 	if(strlen(ss) > 200)
 	    error(_("graphical parameter 'family' has a maximum length of 200 bytes"));
-	strncpy(Rf_dpptr(dd)->family, ss, 201);
-	strncpy(Rf_gpptr(dd)->family, ss, 201);
+	strncpy(dpptr(dd)->family, ss, 201);
+	strncpy(gpptr(dd)->family, ss, 201);
     }
     else if (streql(what, "fin")) {
 	value = coerceVector(value, REALSXP);
@@ -476,8 +477,8 @@ static void Specify(const char *what, SEXP value, DevDesc *dd, SEXP call)
 	posIntCheck(INTEGER(value)[1], what);
 	row = INTEGER(value)[0];
 	col = INTEGER(value)[1];
-	nrow = Rf_dpptr(dd)->numrows;
-	ncol = Rf_dpptr(dd)->numcols;
+	nrow = dpptr(dd)->numrows;
+	ncol = dpptr(dd)->numcols;
 	if(row <= 0 || row > nrow)
 	    error(_("parameter \"i\" in \"mfg\" is out of range"));
 	if(col <= 0 || col > ncol)
@@ -493,12 +494,12 @@ static void Specify(const char *what, SEXP value, DevDesc *dd, SEXP call)
 	R_DEV_2(lastFigure) = nrow*ncol;
 	/*R_DEV__(mfind) = 1;*/
 	/* currentFigure is 1-based */
-	if(Rf_gpptr(dd)->mfind)
-	    Rf_dpptr(dd)->currentFigure = (col-1)*nrow + row;
-	else Rf_dpptr(dd)->currentFigure = (row-1)*ncol + col;
+	if(gpptr(dd)->mfind)
+	    dpptr(dd)->currentFigure = (col-1)*nrow + row;
+	else dpptr(dd)->currentFigure = (row-1)*ncol + col;
 	/*
-	  if (Rf_dpptr(dd)->currentFigure == 0)
-	  Rf_dpptr(dd)->currentFigure = Rf_dpptr(dd)->lastFigure;
+	  if (dpptr(dd)->currentFigure == 0)
+	  dpptr(dd)->currentFigure = dpptr(dd)->lastFigure;
 	*/
 	R_DEV_2(currentFigure);
 	/* R_DEV_2(defaultFigure) = TRUE;
@@ -506,14 +507,13 @@ static void Specify(const char *what, SEXP value, DevDesc *dd, SEXP call)
 	R_DEV_2(new) = TRUE;
 	GReset(dd);
 	/* Force a device clip */
-	if (Rf_dpptr(dd)->canClip)
-	    GForceClip(dd);
+	if (dd->dev->canClip) GForceClip(dd);
     } /* mfg */
 
     else if (streql(what, "new")) {
 	lengthCheck(what, value, 1, call);
 	ix = asLogical(value);
-	if(!Rf_gpptr(dd)->state) {
+	if(!gpptr(dd)->state) {
 	    /* no need to warn with new=FALSE and no plot */
 	    if(ix != 0) warning(_("calling par(new=TRUE) with no plot"));
 	} else R_DEV__(new) = (ix != 0);
@@ -533,7 +533,7 @@ static void Specify(const char *what, SEXP value, DevDesc *dd, SEXP call)
 	R_DEV__(oma[3]) = REAL(value)[3];
 	R_DEV__(oUnits) = LINES;
 	/* !!! Force eject of multiple figures !!! */
-	R_DEV__(currentFigure) = Rf_gpptr(dd)->lastFigure;
+	R_DEV__(currentFigure) = gpptr(dd)->lastFigure;
 	GReset(dd);
     }
     else if (streql(what, "omd")) {
@@ -549,7 +549,7 @@ static void Specify(const char *what, SEXP value, DevDesc *dd, SEXP call)
 	R_DEV__(omd[3]) = REAL(value)[3];
 	R_DEV__(oUnits) = NDC;
 	/* Force eject of multiple figures */
-	R_DEV__(currentFigure) = Rf_gpptr(dd)->lastFigure;
+	R_DEV__(currentFigure) = gpptr(dd)->lastFigure;
 	GReset(dd);
     }
     else if (streql(what, "omi")) {
@@ -565,7 +565,7 @@ static void Specify(const char *what, SEXP value, DevDesc *dd, SEXP call)
 	R_DEV__(omi[3]) = REAL(value)[3];
 	R_DEV__(oUnits) = INCHES;
 	/* Force eject of multiple figures */
-	R_DEV__(currentFigure) = Rf_gpptr(dd)->lastFigure;
+	R_DEV__(currentFigure) = gpptr(dd)->lastFigure;
 	GReset(dd);
     }
     /* -- */
@@ -622,7 +622,7 @@ static void Specify(const char *what, SEXP value, DevDesc *dd, SEXP call)
 	if (REAL(value)[0] == REAL(value)[1] ||
 	    REAL(value)[2] == REAL(value)[3])
 	    par_error(what);
-	if (Rf_gpptr(dd)->xlog) {
+	if (gpptr(dd)->xlog) {
 	    R_DEV_2(logusr[0]) = REAL(value)[0];
 	    R_DEV_2(logusr[1]) = REAL(value)[1];
 	    R_DEV_2(usr[0]) = pow(10., REAL(value)[0]);
@@ -634,7 +634,7 @@ static void Specify(const char *what, SEXP value, DevDesc *dd, SEXP call)
 	    R_DEV_2(logusr[0]) = R_Log10(REAL(value)[0]);
 	    R_DEV_2(logusr[1]) = R_Log10(REAL(value)[1]);
 	}
-	if (Rf_gpptr(dd)->ylog) {
+	if (gpptr(dd)->ylog) {
 	    R_DEV_2(logusr[2]) = REAL(value)[2];
 	    R_DEV_2(logusr[3]) = REAL(value)[3];
 	    R_DEV_2(usr[2]) = pow(10., REAL(value)[2]);
@@ -683,9 +683,9 @@ static void Specify(const char *what, SEXP value, DevDesc *dd, SEXP call)
 #undef R_DEV_2
 #undef R_DEV__
 /* Now defined differently in Specify2() : */
-#define R_DEV__(_P_) Rf_gpptr(dd)->_P_
+#define R_DEV__(_P_) gpptr(dd)->_P_
 
-void attribute_hidden Specify2(const char *what, SEXP value, DevDesc *dd, SEXP call)
+static void Specify2(const char *what, SEXP value, pGEDevDesc dd, SEXP call)
 {
     double x;
     int ix = 0, ptype = ParCode(what);
@@ -717,7 +717,7 @@ void attribute_hidden Specify2(const char *what, SEXP value, DevDesc *dd, SEXP c
 	   (as e.g. pch always did) */
 	if (!isVector(value) || LENGTH(value) < 1)
 	    par_error(what);
-	R_DEV__(bg) = RGBpar(value, 0);
+	R_DEV__(bg) = RGBpar3(value, 0, dpptr(dd)->bg);
     }
     else if (streql(what, "cex")) {
 	/* cex can be a vector of length > 1, so pick off first value
@@ -734,11 +734,12 @@ void attribute_hidden Specify2(const char *what, SEXP value, DevDesc *dd, SEXP c
 	ss = translateChar(STRING_ELT(value, 0));
 	if(strlen(ss) > 200)
 	    error(_("graphical parameter 'family' has a maximum length of 200 bytes"));
-	strncpy(Rf_gpptr(dd)->family, ss, 201);
+	strncpy(gpptr(dd)->family, ss, 201);
     }
     else if (streql(what, "fg")) {
 	/* highlevel arg `fg = ' does *not* set `col' (as par(fg=.) does!*/
-	lengthCheck(what, value, 1, call);	ix = RGBpar(value, 0);
+	lengthCheck(what, value, 1, call);	
+	ix = RGBpar3(value, 0, dpptr(dd)->bg);
 	/*	naIntCheck(ix, what); */
 	R_DEV__(fg) = ix;
     }
@@ -748,79 +749,79 @@ void attribute_hidden Specify2(const char *what, SEXP value, DevDesc *dd, SEXP c
 /* Do NOT forget to update  ../library/base/R/par.R */
 /* if you  ADD a NEW  par !! */
 
-static SEXP Query(const char *what, DevDesc *dd)
+static SEXP Query(const char *what, pGEDevDesc dd)
 {
     SEXP value;
 
     if (streql(what, "adj")) {
 	value = allocVector(REALSXP, 1);
-	REAL(value)[0] = Rf_dpptr(dd)->adj;
+	REAL(value)[0] = dpptr(dd)->adj;
     }
     else if (streql(what, "ann")) {
 	value = allocVector(LGLSXP, 1);
-	LOGICAL(value)[0] = (Rf_dpptr(dd)->ann != 0);
+	LOGICAL(value)[0] = (dpptr(dd)->ann != 0);
     }
     else if (streql(what, "ask")) {
 	value = allocVector(LGLSXP, 1);
-	LOGICAL(value)[0] = Rf_dpptr(dd)->ask;
+	LOGICAL(value)[0] = dd->ask;
     }
     else if (streql(what, "bg")) {
-	value = mkString(col2name(Rf_dpptr(dd)->bg));
+	value = mkString(col2name(dpptr(dd)->bg));
     }
     else if (streql(what, "bty")) {
 	char buf[2];
-	buf[0] = Rf_dpptr(dd)->bty;
+	buf[0] = dpptr(dd)->bty;
 	buf[1] = '\0';
 	value = mkString(buf);
     }
     else if (streql(what, "cex")) {
 	value = allocVector(REALSXP, 1);
-	REAL(value)[0] = Rf_dpptr(dd)->cexbase;
+	REAL(value)[0] = dpptr(dd)->cexbase;
     }
     else if (streql(what, "cex.main")) {
 	value = allocVector(REALSXP, 1);
-	REAL(value)[0] = Rf_dpptr(dd)->cexmain;
+	REAL(value)[0] = dpptr(dd)->cexmain;
     }
     else if (streql(what, "cex.lab")) {
 	value = allocVector(REALSXP, 1);
-	REAL(value)[0] = Rf_dpptr(dd)->cexlab;
+	REAL(value)[0] = dpptr(dd)->cexlab;
     }
     else if (streql(what, "cex.sub")) {
 	value = allocVector(REALSXP, 1);
-	REAL(value)[0] = Rf_dpptr(dd)->cexsub;
+	REAL(value)[0] = dpptr(dd)->cexsub;
     }
     else if (streql(what, "cex.axis")) {
 	value = allocVector(REALSXP, 1);
-	REAL(value)[0] = Rf_dpptr(dd)->cexaxis;
+	REAL(value)[0] = dpptr(dd)->cexaxis;
     }
     else if (streql(what, "cin")) {
 	value = allocVector(REALSXP, 2);
-	REAL(value)[0] = Rf_dpptr(dd)->cra[0]*Rf_dpptr(dd)->ipr[0];
-	REAL(value)[1] = Rf_dpptr(dd)->cra[1]*Rf_dpptr(dd)->ipr[1];
+	REAL(value)[0] = dpptr(dd)->scale * dd->dev->cra[0] * dd->dev->ipr[0];
+	REAL(value)[1] = dpptr(dd)->scale * dd->dev->cra[1] * dd->dev->ipr[1];
     }
     else if (streql(what, "col")) {
-	value = mkString(col2name(Rf_dpptr(dd)->col));
+	value = mkString(col2name(dpptr(dd)->col));
     }
     else if (streql(what, "col.main")) {
-	value = mkString(col2name(Rf_dpptr(dd)->colmain));
+	value = mkString(col2name(dpptr(dd)->colmain));
     }
     else if (streql(what, "col.lab")) {
-	value = mkString(col2name(Rf_dpptr(dd)->collab));
+	value = mkString(col2name(dpptr(dd)->collab));
     }
     else if (streql(what, "col.sub")) {
-	value = mkString(col2name(Rf_dpptr(dd)->colsub));
+	value = mkString(col2name(dpptr(dd)->colsub));
     }
     else if (streql(what, "col.axis")) {
-	value = mkString(col2name(Rf_dpptr(dd)->colaxis));
+	value = mkString(col2name(dpptr(dd)->colaxis));
     }
     else if (streql(what, "cra")) {
 	value = allocVector(REALSXP, 2);
-	REAL(value)[0] = Rf_dpptr(dd)->cra[0];
-	REAL(value)[1] = Rf_dpptr(dd)->cra[1];
+	REAL(value)[0] = dpptr(dd)->scale * dd->dev->cra[0];
+	REAL(value)[1] = dpptr(dd)->scale * dd->dev->cra[1];
     }
     else if (streql(what, "crt")) {
 	value = allocVector(REALSXP, 1);
-	REAL(value)[0] = Rf_dpptr(dd)->crt;
+	REAL(value)[0] = dpptr(dd)->crt;
     }
     else if (streql(what, "csi")) {
 	value = allocVector(REALSXP, 1);
@@ -829,10 +830,12 @@ static SEXP Query(const char *what, DevDesc *dd)
     else if (streql(what, "cxy")) {
 	value = allocVector(REALSXP, 2);
 	/* == par("cin") / par("pin") : */
-	REAL(value)[0] = Rf_dpptr(dd)->cra[0]*Rf_dpptr(dd)->ipr[0] / Rf_dpptr(dd)->pin[0]
-	    * (Rf_dpptr(dd)->usr[1] - Rf_dpptr(dd)->usr[0]);
-	REAL(value)[1] = Rf_dpptr(dd)->cra[1]*Rf_dpptr(dd)->ipr[1] / Rf_dpptr(dd)->pin[1]
-	    * (Rf_dpptr(dd)->usr[3] - Rf_dpptr(dd)->usr[2]);
+	REAL(value)[0] = dpptr(dd)->scale * dd->dev->cra[0] 
+	    * dd->dev->ipr[0] / dpptr(dd)->pin[0]
+	    * (dpptr(dd)->usr[1] - dpptr(dd)->usr[0]);
+	REAL(value)[1] = dpptr(dd)->scale * dd->dev->cra[1] 
+	    * dd->dev->ipr[1] / dpptr(dd)->pin[1]
+	    * (dpptr(dd)->usr[3] - dpptr(dd)->usr[2]);
     }
     else if (streql(what, "din")) {
 	value = allocVector(REALSXP, 2);
@@ -841,105 +844,105 @@ static SEXP Query(const char *what, DevDesc *dd)
     }
     else if (streql(what, "err")) {
 	value = allocVector(INTSXP, 1);
-	INTEGER(value)[0] = Rf_dpptr(dd)->err;
+	INTEGER(value)[0] = dpptr(dd)->err;
     }
     else if (streql(what, "family")) {
-	value = mkString(Rf_dpptr(dd)->family);
+	value = mkString(dpptr(dd)->family);
     }
     else if (streql(what, "fg")) {
-	value = mkString(col2name(Rf_dpptr(dd)->fg));
+	value = mkString(col2name(dpptr(dd)->fg));
     }
     else if (streql(what, "fig")) {
 	value = allocVector(REALSXP, 4);
-	REAL(value)[0] = Rf_dpptr(dd)->fig[0];
-	REAL(value)[1] = Rf_dpptr(dd)->fig[1];
-	REAL(value)[2] = Rf_dpptr(dd)->fig[2];
-	REAL(value)[3] = Rf_dpptr(dd)->fig[3];
+	REAL(value)[0] = dpptr(dd)->fig[0];
+	REAL(value)[1] = dpptr(dd)->fig[1];
+	REAL(value)[2] = dpptr(dd)->fig[2];
+	REAL(value)[3] = dpptr(dd)->fig[3];
     }
     else if (streql(what, "fin")) {
 	value = allocVector(REALSXP, 2);
-	REAL(value)[0] = Rf_dpptr(dd)->fin[0];
-	REAL(value)[1] = Rf_dpptr(dd)->fin[1];
+	REAL(value)[0] = dpptr(dd)->fin[0];
+	REAL(value)[1] = dpptr(dd)->fin[1];
     }
     else if (streql(what, "font")) {
 	value = allocVector(INTSXP, 1);
-	INTEGER(value)[0] = Rf_dpptr(dd)->font;
+	INTEGER(value)[0] = dpptr(dd)->font;
     }
     else if (streql(what, "font.main")) {
 	value = allocVector(INTSXP, 1);
-	INTEGER(value)[0] = Rf_dpptr(dd)->fontmain;
+	INTEGER(value)[0] = dpptr(dd)->fontmain;
     }
     else if (streql(what, "font.lab")) {
 	value = allocVector(INTSXP, 1);
-	INTEGER(value)[0] = Rf_dpptr(dd)->fontlab;
+	INTEGER(value)[0] = dpptr(dd)->fontlab;
     }
     else if (streql(what, "font.sub")) {
 	value = allocVector(INTSXP, 1);
-	INTEGER(value)[0] = Rf_dpptr(dd)->fontsub;
+	INTEGER(value)[0] = dpptr(dd)->fontsub;
     }
     else if (streql(what, "font.axis")) {
 	value = allocVector(INTSXP, 1);
-	INTEGER(value)[0] = Rf_dpptr(dd)->fontaxis;
+	INTEGER(value)[0] = dpptr(dd)->fontaxis;
     }
     else if (streql(what, "gamma")) {
 	value = allocVector(REALSXP, 1);
-	REAL(value)[0] = Rf_dpptr(dd)->gamma;
+	REAL(value)[0] = dpptr(dd)->gamma;
     }
     else if (streql(what, "lab")) {
 	value = allocVector(INTSXP, 3);
-	INTEGER(value)[0] = Rf_dpptr(dd)->lab[0];
-	INTEGER(value)[1] = Rf_dpptr(dd)->lab[1];
-	INTEGER(value)[2] = Rf_dpptr(dd)->lab[2];
+	INTEGER(value)[0] = dpptr(dd)->lab[0];
+	INTEGER(value)[1] = dpptr(dd)->lab[1];
+	INTEGER(value)[2] = dpptr(dd)->lab[2];
     }
     else if (streql(what, "las")) {
 	value = allocVector(INTSXP, 1);
-	INTEGER(value)[0] = Rf_dpptr(dd)->las;
+	INTEGER(value)[0] = dpptr(dd)->las;
     }
     else if (streql(what, "lend")) {
-	value = LENDget(Rf_dpptr(dd)->lend);
+	value = GE_LENDget(dpptr(dd)->lend);
     }
     else if (streql(what, "lheight")) {
 	value = allocVector(REALSXP, 1);
-	REAL(value)[0] = Rf_dpptr(dd)->lheight;
+	REAL(value)[0] = dpptr(dd)->lheight;
     }
     else if (streql(what, "ljoin")) {
-	value = LJOINget(Rf_dpptr(dd)->ljoin);
+	value = GE_LJOINget(dpptr(dd)->ljoin);
     }
     else if (streql(what, "lmitre")) {
 	value = allocVector(REALSXP, 1);
-	REAL(value)[0] = Rf_dpptr(dd)->lmitre;
+	REAL(value)[0] = dpptr(dd)->lmitre;
     }
     else if (streql(what, "lty")) {
-	value = LTYget(Rf_dpptr(dd)->lty);
+	value = GE_LTYget(dpptr(dd)->lty);
     }
     else if (streql(what, "lwd")) {
 	value =	 allocVector(REALSXP, 1);
-	REAL(value)[0] = Rf_dpptr(dd)->lwd;
+	REAL(value)[0] = dpptr(dd)->lwd;
     }
     else if (streql(what, "mai")) {
 	value = allocVector(REALSXP, 4);
-	REAL(value)[0] = Rf_dpptr(dd)->mai[0];
-	REAL(value)[1] = Rf_dpptr(dd)->mai[1];
-	REAL(value)[2] = Rf_dpptr(dd)->mai[2];
-	REAL(value)[3] = Rf_dpptr(dd)->mai[3];
+	REAL(value)[0] = dpptr(dd)->mai[0];
+	REAL(value)[1] = dpptr(dd)->mai[1];
+	REAL(value)[2] = dpptr(dd)->mai[2];
+	REAL(value)[3] = dpptr(dd)->mai[3];
     }
     else if (streql(what, "mar")) {
 	value = allocVector(REALSXP, 4);
-	REAL(value)[0] = Rf_dpptr(dd)->mar[0];
-	REAL(value)[1] = Rf_dpptr(dd)->mar[1];
-	REAL(value)[2] = Rf_dpptr(dd)->mar[2];
-	REAL(value)[3] = Rf_dpptr(dd)->mar[3];
+	REAL(value)[0] = dpptr(dd)->mar[0];
+	REAL(value)[1] = dpptr(dd)->mar[1];
+	REAL(value)[2] = dpptr(dd)->mar[2];
+	REAL(value)[3] = dpptr(dd)->mar[3];
     }
     else if (streql(what, "mex")) {
 	value = allocVector(REALSXP, 1);
-	REAL(value)[0] = Rf_dpptr(dd)->mex;
+	REAL(value)[0] = dpptr(dd)->mex;
     }
     /* NOTE that if a complex layout has been specified */
     /* then this simple information may not be very useful. */
     else if (streql(what, "mfrow") || streql(what, "mfcol")) {
 	value = allocVector(INTSXP, 2);
-	INTEGER(value)[0] = Rf_dpptr(dd)->numrows;
-	INTEGER(value)[1] = Rf_dpptr(dd)->numcols;
+	INTEGER(value)[0] = dpptr(dd)->numrows;
+	INTEGER(value)[1] = dpptr(dd)->numcols;
     }
     else if (streql(what, "mfg")) {
 	int row, col;
@@ -947,162 +950,168 @@ static SEXP Query(const char *what, DevDesc *dd)
 	currentFigureLocation(&row, &col, dd);
 	INTEGER(value)[0] = row+1;
 	INTEGER(value)[1] = col+1;
-	INTEGER(value)[2] = Rf_dpptr(dd)->numrows;
-	INTEGER(value)[3] = Rf_dpptr(dd)->numcols;
+	INTEGER(value)[2] = dpptr(dd)->numrows;
+	INTEGER(value)[3] = dpptr(dd)->numcols;
     }
     else if (streql(what, "mgp")) {
 	value = allocVector(REALSXP, 3);
-	REAL(value)[0] = Rf_dpptr(dd)->mgp[0];
-	REAL(value)[1] = Rf_dpptr(dd)->mgp[1];
-	REAL(value)[2] = Rf_dpptr(dd)->mgp[2];
+	REAL(value)[0] = dpptr(dd)->mgp[0];
+	REAL(value)[1] = dpptr(dd)->mgp[1];
+	REAL(value)[2] = dpptr(dd)->mgp[2];
     }
     else if (streql(what, "mkh")) {
+	/* Unused in R, but settable */
 	value = allocVector(REALSXP, 1);
-	REAL(value)[0] = Rf_dpptr(dd)->mkh;
+	REAL(value)[0] = dpptr(dd)->mkh;
     }
     else if (streql(what, "new")) {
 	value = allocVector(LGLSXP, 1);
-	LOGICAL(value)[0] = Rf_dpptr(dd)->new;
+	LOGICAL(value)[0] = dpptr(dd)->new;
     }
     else if (streql(what, "oma")) {
 	value = allocVector(REALSXP, 4);
-	REAL(value)[0] = Rf_dpptr(dd)->oma[0];
-	REAL(value)[1] = Rf_dpptr(dd)->oma[1];
-	REAL(value)[2] = Rf_dpptr(dd)->oma[2];
-	REAL(value)[3] = Rf_dpptr(dd)->oma[3];
+	REAL(value)[0] = dpptr(dd)->oma[0];
+	REAL(value)[1] = dpptr(dd)->oma[1];
+	REAL(value)[2] = dpptr(dd)->oma[2];
+	REAL(value)[3] = dpptr(dd)->oma[3];
     }
     else if (streql(what, "omd")) {
 	value = allocVector(REALSXP, 4);
-	REAL(value)[0] = Rf_dpptr(dd)->omd[0];
-	REAL(value)[1] = Rf_dpptr(dd)->omd[1];
-	REAL(value)[2] = Rf_dpptr(dd)->omd[2];
-	REAL(value)[3] = Rf_dpptr(dd)->omd[3];
+	REAL(value)[0] = dpptr(dd)->omd[0];
+	REAL(value)[1] = dpptr(dd)->omd[1];
+	REAL(value)[2] = dpptr(dd)->omd[2];
+	REAL(value)[3] = dpptr(dd)->omd[3];
     }
     else if (streql(what, "omi")) {
 	value = allocVector(REALSXP, 4);
-	REAL(value)[0] = Rf_dpptr(dd)->omi[0];
-	REAL(value)[1] = Rf_dpptr(dd)->omi[1];
-	REAL(value)[2] = Rf_dpptr(dd)->omi[2];
-	REAL(value)[3] = Rf_dpptr(dd)->omi[3];
+	REAL(value)[0] = dpptr(dd)->omi[0];
+	REAL(value)[1] = dpptr(dd)->omi[1];
+	REAL(value)[2] = dpptr(dd)->omi[2];
+	REAL(value)[3] = dpptr(dd)->omi[3];
     }
     else if (streql(what, "pch")) {
-	/* FIXME: see FixupPch */
-	if(Rf_dpptr(dd)->pch < ' ' || Rf_dpptr(dd)->pch > 255) {
-	    value = ScalarInteger(Rf_dpptr(dd)->pch);
-	} else {
+	int val = dpptr(dd)->pch;
+	/* we need to be careful that par("pch") is converted back
+	   to the same value */
+	if (known_to_be_latin1 && val <= -32 && val >= -255) val = -val;
+	if(val >= ' ' && val <= (mbcslocale ? 127 : 255)) {
 	    char buf[2];
-	    buf[0] = Rf_dpptr(dd)->pch;
+	    buf[0] = val;
 	    buf[1] = '\0';
 	    value = mkString(buf);
+	} else {
+	    /* Could return as UTF-8 string */
+	    value = ScalarInteger(val);
 	}
     }
     else if (streql(what, "pin")) {
 	value = allocVector(REALSXP, 2);
-	REAL(value)[0] = Rf_dpptr(dd)->pin[0];
-	REAL(value)[1] = Rf_dpptr(dd)->pin[1];
+	REAL(value)[0] = dpptr(dd)->pin[0];
+	REAL(value)[1] = dpptr(dd)->pin[1];
     }
     else if (streql(what, "plt")) {
 	value = allocVector(REALSXP, 4);
-	REAL(value)[0] = Rf_dpptr(dd)->plt[0];
-	REAL(value)[1] = Rf_dpptr(dd)->plt[1];
-	REAL(value)[2] = Rf_dpptr(dd)->plt[2];
-	REAL(value)[3] = Rf_dpptr(dd)->plt[3];
+	REAL(value)[0] = dpptr(dd)->plt[0];
+	REAL(value)[1] = dpptr(dd)->plt[1];
+	REAL(value)[2] = dpptr(dd)->plt[2];
+	REAL(value)[3] = dpptr(dd)->plt[3];
     }
     else if (streql(what, "ps")) {
 	value = allocVector(INTSXP, 1);
-	INTEGER(value)[0] = Rf_dpptr(dd)->ps;
+	/* was reporting unscaled prior to 2.7.0 */
+	INTEGER(value)[0] = dpptr(dd)->ps * dpptr(dd)->scale;
     }
     else if (streql(what, "pty")) {
 	char buf[2];
-	buf[0] = Rf_dpptr(dd)->pty;
+	buf[0] = dpptr(dd)->pty;
 	buf[1] = '\0';
 	value = mkString(buf);
     }
     else if (streql(what, "smo")) {
 	value = allocVector(REALSXP, 1);
-	REAL(value)[0] = Rf_dpptr(dd)->smo;
+	REAL(value)[0] = dpptr(dd)->smo;
     }
     else if (streql(what, "srt")) {
 	value = allocVector(REALSXP, 1);
-	REAL(value)[0] = Rf_dpptr(dd)->srt;
+	REAL(value)[0] = dpptr(dd)->srt;
     }
     else if (streql(what, "tck")) {
 	value = allocVector(REALSXP, 1);
-	REAL(value)[0] = Rf_dpptr(dd)->tck;
+	REAL(value)[0] = dpptr(dd)->tck;
     }
     else if (streql(what, "tcl")) {
 	value = allocVector(REALSXP, 1);
-	REAL(value)[0] = Rf_dpptr(dd)->tcl;
+	REAL(value)[0] = dpptr(dd)->tcl;
     }
     else if (streql(what, "usr")) {
 	value = allocVector(REALSXP, 4);
-	if (Rf_gpptr(dd)->xlog) {
-	    REAL(value)[0] = Rf_gpptr(dd)->logusr[0];
-	    REAL(value)[1] = Rf_gpptr(dd)->logusr[1];
+	if (gpptr(dd)->xlog) {
+	    REAL(value)[0] = gpptr(dd)->logusr[0];
+	    REAL(value)[1] = gpptr(dd)->logusr[1];
 	}
 	else {
-	    REAL(value)[0] = Rf_dpptr(dd)->usr[0];
-	    REAL(value)[1] = Rf_dpptr(dd)->usr[1];
+	    REAL(value)[0] = dpptr(dd)->usr[0];
+	    REAL(value)[1] = dpptr(dd)->usr[1];
 	}
-	if (Rf_gpptr(dd)->ylog) {
-	    REAL(value)[2] = Rf_gpptr(dd)->logusr[2];
-	    REAL(value)[3] = Rf_gpptr(dd)->logusr[3];
+	if (gpptr(dd)->ylog) {
+	    REAL(value)[2] = gpptr(dd)->logusr[2];
+	    REAL(value)[3] = gpptr(dd)->logusr[3];
 	}
 	else {
-	    REAL(value)[2] = Rf_dpptr(dd)->usr[2];
-	    REAL(value)[3] = Rf_dpptr(dd)->usr[3];
+	    REAL(value)[2] = dpptr(dd)->usr[2];
+	    REAL(value)[3] = dpptr(dd)->usr[3];
 	}
     }
     else if (streql(what, "xaxp")) {
 	value = allocVector(REALSXP, 3);
-	REAL(value)[0] = Rf_dpptr(dd)->xaxp[0];
-	REAL(value)[1] = Rf_dpptr(dd)->xaxp[1];
-	REAL(value)[2] = Rf_dpptr(dd)->xaxp[2];
+	REAL(value)[0] = dpptr(dd)->xaxp[0];
+	REAL(value)[1] = dpptr(dd)->xaxp[1];
+	REAL(value)[2] = dpptr(dd)->xaxp[2];
     }
     else if (streql(what, "xaxs")) {
 	char buf[2];
-	buf[0] = Rf_dpptr(dd)->xaxs;
+	buf[0] = dpptr(dd)->xaxs;
 	buf[1] = '\0';
 	value = mkString(buf);
     }
     else if (streql(what, "xaxt")) {
 	char buf[2];
-	buf[0] = Rf_dpptr(dd)->xaxt;
+	buf[0] = dpptr(dd)->xaxt;
 	buf[1] = '\0';
 	value = mkString(buf);
     }
     else if (streql(what, "xlog")) {
 	value = allocVector(LGLSXP, 1);
-	LOGICAL(value)[0] = Rf_dpptr(dd)->xlog;
+	LOGICAL(value)[0] = dpptr(dd)->xlog;
     }
     else if (streql(what, "xpd")) {
 	value = allocVector(LGLSXP, 1);
-	if (Rf_dpptr(dd)->xpd == 2)
+	if (dpptr(dd)->xpd == 2)
 	    LOGICAL(value)[0] = NA_LOGICAL;
 	else
-	    LOGICAL(value)[0] = Rf_dpptr(dd)->xpd;
+	    LOGICAL(value)[0] = dpptr(dd)->xpd;
     }
     else if (streql(what, "yaxp")) {
 	value = allocVector(REALSXP, 3);
-	REAL(value)[0] = Rf_dpptr(dd)->yaxp[0];
-	REAL(value)[1] = Rf_dpptr(dd)->yaxp[1];
-	REAL(value)[2] = Rf_dpptr(dd)->yaxp[2];
+	REAL(value)[0] = dpptr(dd)->yaxp[0];
+	REAL(value)[1] = dpptr(dd)->yaxp[1];
+	REAL(value)[2] = dpptr(dd)->yaxp[2];
     }
     else if (streql(what, "yaxs")) {
 	char buf[2];
-	buf[0] = Rf_dpptr(dd)->yaxs;
+	buf[0] = dpptr(dd)->yaxs;
 	buf[1] = '\0';
 	value = mkString(buf);
     }
     else if (streql(what, "yaxt")) {
 	char buf[2];
-	buf[0] = Rf_dpptr(dd)->yaxt;
+	buf[0] = dpptr(dd)->yaxt;
 	buf[1] = '\0';
 	value = mkString(buf);
     }
     else if (streql(what, "ylog")) {
 	value = allocVector(LGLSXP, 1);
-	LOGICAL(value)[0] = Rf_dpptr(dd)->ylog;
+	LOGICAL(value)[0] = dpptr(dd)->ylog;
     }
     else if (ParCode(what) == -2) { 
 	warning(_("graphical parameter \"%s\" is obsolete"), what);
@@ -1119,12 +1128,12 @@ SEXP attribute_hidden do_par(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP value;
     SEXP originalArgs = args;
-    DevDesc *dd;
+    pGEDevDesc dd;
     int new_spec, nargs;
 
     checkArity(op, args);
 
-    dd = CurrentDevice();
+    dd = GEcurrentDevice();
     new_spec = 0;
     args = CAR(args);
     nargs = length(args);
@@ -1168,34 +1177,23 @@ SEXP attribute_hidden do_par(SEXP call, SEXP op, SEXP args, SEXP env)
     }
     /* should really only do this if specifying new pars ?  yes! [MM] */
     if (new_spec && GRecording(call, dd))
-	recordGraphicOperation(op, originalArgs, dd);
+	GErecordGraphicOperation(op, originalArgs, dd);
     return value;
 }
 
-SEXP attribute_hidden do_readonlypars(SEXP call, SEXP op, SEXP args, SEXP env)
+SEXP attribute_hidden Rg_readonlypars(void)
 {
     SEXP result;
-    GEDevDesc *dd;
-    Rboolean canChangeGamma;
-    int nreadonly;
+    Rboolean canChangeGamma = GEcurrentDevice()->dev->canChangeGamma;
+    int nreadonly = canChangeGamma ? 5 : 6;
 
-    checkArity(op, args);
-
-    dd = GEcurrentDevice();
-    canChangeGamma = dd->dev->canChangeGamma;
-
-    if (canChangeGamma)
-	nreadonly = 5;
-    else
-	nreadonly = 6;
     PROTECT(result = allocVector(STRSXP, nreadonly));
     SET_STRING_ELT(result, 0, mkChar("cin"));
     SET_STRING_ELT(result, 1, mkChar("cra"));
     SET_STRING_ELT(result, 2, mkChar("csi"));
     SET_STRING_ELT(result, 3, mkChar("cxy"));
     SET_STRING_ELT(result, 4, mkChar("din"));
-    if (!canChangeGamma)
-	SET_STRING_ELT(result, 5, mkChar("gamma"));
+    if (!canChangeGamma) SET_STRING_ELT(result, 5, mkChar("gamma"));
     UNPROTECT(1);
     return result;
 }
@@ -1223,20 +1221,20 @@ SEXP attribute_hidden do_layout(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     int i, j, nrow, ncol, ncmrow, ncmcol;
     SEXP originalArgs = args;
-    DevDesc *dd;
+    pGEDevDesc dd;
 
     checkArity(op, args);
 
-    dd = CurrentDevice();
+    dd = GEcurrentDevice();
 
     /* num.rows: */
-    nrow = Rf_dpptr(dd)->numrows = Rf_gpptr(dd)->numrows =
+    nrow = dpptr(dd)->numrows = gpptr(dd)->numrows =
 	INTEGER(CAR(args))[0];
     if (nrow > MAX_LAYOUT_ROWS)
 	error(_("too many rows in layout, limit %d"), MAX_LAYOUT_ROWS);
     args = CDR(args);
     /* num.cols: */
-    ncol = Rf_dpptr(dd)->numcols = Rf_gpptr(dd)->numcols =
+    ncol = dpptr(dd)->numcols = gpptr(dd)->numcols =
 	INTEGER(CAR(args))[0];
     if (ncol > MAX_LAYOUT_COLS)
 	error(_("too many columns in layout, limit %d"), MAX_LAYOUT_COLS);
@@ -1245,78 +1243,98 @@ SEXP attribute_hidden do_layout(SEXP call, SEXP op, SEXP args, SEXP env)
     args = CDR(args);
     /* mat[i,j] == order[i+j*nrow] : */
     for (i = 0; i < nrow * ncol; i++)
-	Rf_dpptr(dd)->order[i] = Rf_gpptr(dd)->order[i] =
+	dpptr(dd)->order[i] = gpptr(dd)->order[i] =
 	    INTEGER(CAR(args))[i];
     args = CDR(args);
 
     /* num.figures: */
-    Rf_dpptr(dd)->currentFigure = Rf_gpptr(dd)->currentFigure =
-	Rf_dpptr(dd)->lastFigure = Rf_gpptr(dd)->lastFigure =
+    dpptr(dd)->currentFigure = gpptr(dd)->currentFigure =
+	dpptr(dd)->lastFigure = gpptr(dd)->lastFigure =
 	INTEGER(CAR(args))[0];
     args = CDR(args);
     /* col.widths: */
     for (j = 0; j < ncol; j++)
-	Rf_dpptr(dd)->widths[j] = Rf_gpptr(dd)->widths[j] =
+	dpptr(dd)->widths[j] = gpptr(dd)->widths[j] =
 	    REAL(CAR(args))[j];
     args = CDR(args);
     /* row.heights: */
     for (i = 0; i < nrow; i++)
-	Rf_dpptr(dd)->heights[i] = Rf_gpptr(dd)->heights[i] =
+	dpptr(dd)->heights[i] = gpptr(dd)->heights[i] =
 	    REAL(CAR(args))[i];
     args = CDR(args);
     /* cm.widths: */
     ncmcol = length(CAR(args));
     for (j = 0; j < ncol; j++)
-	Rf_dpptr(dd)->cmWidths[j] = Rf_gpptr(dd)->cmWidths[j] = 0;
+	dpptr(dd)->cmWidths[j] = gpptr(dd)->cmWidths[j] = 0;
     for (j = 0; j < ncmcol; j++) {
-	Rf_dpptr(dd)->cmWidths[INTEGER(CAR(args))[j] - 1]
-	    = Rf_gpptr(dd)->cmWidths[INTEGER(CAR(args))[j] - 1]
+	dpptr(dd)->cmWidths[INTEGER(CAR(args))[j] - 1]
+	    = gpptr(dd)->cmWidths[INTEGER(CAR(args))[j] - 1]
 	    = 1;
     }
     args = CDR(args);
     /* cm.heights: */
     ncmrow = length(CAR(args));
     for (i = 0; i < nrow; i++)
-	Rf_dpptr(dd)->cmHeights[i] = Rf_gpptr(dd)->cmHeights[i] = 0;
+	dpptr(dd)->cmHeights[i] = gpptr(dd)->cmHeights[i] = 0;
     for (i = 0; i < ncmrow; i++) {
-	Rf_dpptr(dd)->cmHeights[INTEGER(CAR(args))[i] - 1]
-	    = Rf_gpptr(dd)->cmHeights[INTEGER(CAR(args))[i]-1]
+	dpptr(dd)->cmHeights[INTEGER(CAR(args))[i] - 1]
+	    = gpptr(dd)->cmHeights[INTEGER(CAR(args))[i]-1]
 	    = 1;
     }
     args = CDR(args);
     /* respect =  0 (FALSE), 1 (TRUE), or 2 (matrix) : */
-    Rf_dpptr(dd)->rspct = Rf_gpptr(dd)->rspct = INTEGER(CAR(args))[0];
+    dpptr(dd)->rspct = gpptr(dd)->rspct = INTEGER(CAR(args))[0];
     args = CDR(args);
     /* respect.mat */
     for (i = 0; i < nrow * ncol; i++)
-	Rf_dpptr(dd)->respect[i] = Rf_gpptr(dd)->respect[i]
+	dpptr(dd)->respect[i] = gpptr(dd)->respect[i]
 		= INTEGER(CAR(args))[i];
 
     /*------------------------------------------------------*/
 
     if (nrow > 2 || ncol > 2) {
-	Rf_gpptr(dd)->cexbase = Rf_dpptr(dd)->cexbase = 0.66;
-	Rf_gpptr(dd)->mex = Rf_dpptr(dd)->mex = 1.0;
+	gpptr(dd)->cexbase = dpptr(dd)->cexbase = 0.66;
+	gpptr(dd)->mex = dpptr(dd)->mex = 1.0;
     }
     else if (nrow == 2 && ncol == 2) {
-	Rf_gpptr(dd)->cexbase = Rf_dpptr(dd)->cexbase = 0.83;
-	Rf_gpptr(dd)->mex = Rf_dpptr(dd)->mex = 1.0;
+	gpptr(dd)->cexbase = dpptr(dd)->cexbase = 0.83;
+	gpptr(dd)->mex = dpptr(dd)->mex = 1.0;
     }
     else {
-	Rf_gpptr(dd)->cexbase = Rf_dpptr(dd)->cexbase = 1.0;
-	Rf_gpptr(dd)->mex = Rf_dpptr(dd)->mex = 1.0;
+	gpptr(dd)->cexbase = dpptr(dd)->cexbase = 1.0;
+	gpptr(dd)->mex = dpptr(dd)->mex = 1.0;
     }
 
-    Rf_dpptr(dd)->defaultFigure = Rf_gpptr(dd)->defaultFigure = TRUE;
-    Rf_dpptr(dd)->layout = Rf_gpptr(dd)->layout = TRUE;
+    dpptr(dd)->defaultFigure = gpptr(dd)->defaultFigure = TRUE;
+    dpptr(dd)->layout = gpptr(dd)->layout = TRUE;
 
     GReset(dd);
 
     if (GRecording(call, dd))
-	recordGraphicOperation(op, originalArgs, dd);
+	GErecordGraphicOperation(op, originalArgs, dd);
     return R_NilValue;
 }
 
+
+/* ProcessInLinePars handles inline par specifications 
+   in graphics functions. */
+
+attribute_hidden
+void ProcessInlinePars(SEXP s, pGEDevDesc dd, SEXP call)
+{
+    if (isList(s)) {
+	while (s != R_NilValue) {
+	    if (isList(CAR(s)))
+		ProcessInlinePars(CAR(s), dd, call);
+	    else if (TAG(s) != R_NilValue)
+		Specify2(CHAR(PRINTNAME(TAG(s))), CAR(s), dd, call);
+	    s = CDR(s);
+	}
+    }
+}
+
+
+
 /*= Local Variables: **/
 /*= mode: C **/
 /*= kept-old-versions: 12 **/
