@@ -1,11 +1,11 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996, 1997 Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2007	The R Development Core Team
+ *  Copyright (C) 1998-2008	The R Development Core Team
  *
  *  This source code module:
  *  Copyright (C) 1997, 1998 Paul Murrell and Ross Ihaka
- *  Copyright (C) 1998-2006	The R Development Core Team
+ *  Copyright (C) 1998-2008	The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  *  http://www.r-project.org/Licenses/
  */
 
-/* <UTF8-FIXME>
+/* <UTF8>
    byte-level access and use of ctype functions for symbol font
 
    Has encoding of symbol font hard-coded.  Not clear if this is enough.
@@ -42,7 +42,7 @@
 
 
 #include <Rmath.h>
-#include <Graphics.h>
+#include <R_ext/GraphicsEngine.h>
 
 
 /*
@@ -105,7 +105,7 @@ static double ItalicFactor = 0.15;
 /* Convert CurrentX and CurrentY from */
 /* 0 angle to and CurrentAngle */
 
-static double ConvertedX(mathContext *mc, GEDevDesc *dd)
+static double ConvertedX(mathContext *mc, pGEDevDesc dd)
 {
     double rotatedX = mc->ReferenceX +
 	(mc->CurrentX - mc->ReferenceX) * mc->CosAngle -
@@ -113,7 +113,7 @@ static double ConvertedX(mathContext *mc, GEDevDesc *dd)
     return toDeviceX(rotatedX, MetricUnit, dd);
 }
 
-static double ConvertedY(mathContext *mc, GEDevDesc *dd)
+static double ConvertedY(mathContext *mc, pGEDevDesc dd)
 {
     double rotatedY = mc->ReferenceY +
 	(mc->CurrentY - mc->ReferenceY) * mc->CosAngle +
@@ -139,95 +139,85 @@ static void PMoveTo(double x, double y, mathContext *mc)
 
 /* Basic Font Properties */
 
-static double xHeight(R_GE_gcontext *gc, GEDevDesc *dd)
+static double xHeight(pGEcontext gc, pGEDevDesc dd)
 {
     double height, depth, width;
-    GEMetricInfo('x', gc,
-		&height, &depth, &width, dd);
+    GEMetricInfo('x', gc, &height, &depth, &width, dd);
     return fromDeviceHeight(height, MetricUnit, dd);
 }
 
-static double XHeight(R_GE_gcontext *gc, GEDevDesc *dd)
+static double XHeight(pGEcontext gc, pGEDevDesc dd)
 {
     double height, depth, width;
-    GEMetricInfo('X', gc,
-		&height, &depth, &width, dd);
+    GEMetricInfo('X', gc, &height, &depth, &width, dd);
     return fromDeviceHeight(height, MetricUnit, dd);
 }
 
-static double AxisHeight(R_GE_gcontext *gc, GEDevDesc *dd)
+static double AxisHeight(pGEcontext gc, pGEDevDesc dd)
 {
     double height, depth, width;
-    GEMetricInfo('+', gc,
-		&height, &depth, &width, dd);
+    GEMetricInfo('+', gc, &height, &depth, &width, dd);
     return fromDeviceHeight(0.5 * height, MetricUnit, dd);
 }
 
-static double Quad(R_GE_gcontext *gc, GEDevDesc *dd)
+static double Quad(pGEcontext gc, pGEDevDesc dd)
 {
     double height, depth, width;
-    GEMetricInfo('M', gc,
-		&height, &depth, &width, dd);
+    GEMetricInfo('M', gc, &height, &depth, &width, dd);
     return fromDeviceHeight(width, MetricUnit, dd);
 }
 
 /* The height of digits */
-static double FigHeight(R_GE_gcontext *gc, GEDevDesc *dd)
+static double FigHeight(pGEcontext gc, pGEDevDesc dd)
 {
     double height, depth, width;
-    GEMetricInfo('0', gc,
-		&height, &depth, &width, dd);
+    GEMetricInfo('0', gc, &height, &depth, &width, dd);
     return fromDeviceHeight(height, MetricUnit, dd);
 }
 
 /* Depth of lower case descenders */
-static double DescDepth(R_GE_gcontext *gc, GEDevDesc *dd)
+static double DescDepth(pGEcontext gc, pGEDevDesc dd)
 {
     double height, depth, width;
-    GEMetricInfo('g', gc,
-		&height, &depth, &width, dd);
+    GEMetricInfo('g', gc, &height, &depth, &width, dd);
     return fromDeviceHeight(depth, MetricUnit, dd);
 }
 
 /* Thickness of rules */
-static double RuleThickness()
+static double RuleThickness(void)
 {
     return 0.015;
 }
 
-static double ThinSpace(R_GE_gcontext *gc, GEDevDesc *dd)
+static double ThinSpace(pGEcontext gc, pGEDevDesc dd)
 {
     double height, depth, width;
     static double OneSixth = 0.16666666666666666666;
-    GEMetricInfo('M', gc,
-		&height, &depth, &width, dd);
+    GEMetricInfo('M', gc, &height, &depth, &width, dd);
     return fromDeviceHeight(OneSixth * width, MetricUnit, dd);
 }
 
-static double MediumSpace(R_GE_gcontext *gc, GEDevDesc *dd)
+static double MediumSpace(pGEcontext gc, pGEDevDesc dd)
 {
     double height, depth, width;
     static double TwoNinths = 0.22222222222222222222;
-    GEMetricInfo('M', gc,
-		&height, &depth, &width, dd);
+    GEMetricInfo('M', gc, &height, &depth, &width, dd);
     return fromDeviceHeight(TwoNinths * width, MetricUnit, dd);
 }
 
-static double ThickSpace(R_GE_gcontext *gc, GEDevDesc *dd)
+static double ThickSpace(pGEcontext gc, pGEDevDesc dd)
 {
     double height, depth, width;
     static double FiveEighteenths = 0.27777777777777777777;
-    GEMetricInfo('M', gc,
-		&height, &depth, &width, dd);
+    GEMetricInfo('M', gc, &height, &depth, &width, dd);
     return fromDeviceHeight(FiveEighteenths * width, MetricUnit, dd);
 }
 
-static double MuSpace(R_GE_gcontext *gc, GEDevDesc *dd)
+static double MuSpace(pGEcontext gc, pGEDevDesc dd)
 {
     double height, depth, width;
     static double OneEighteenth = 0.05555555555555555555;
-    GEMetricInfo('M', gc,
-		&height, &depth, &width, dd);
+    GEMetricInfo('M', gc, &height, &depth, &width, dd);
     return fromDeviceHeight(OneEighteenth * width, MetricUnit, dd);
 }
 
@@ -253,7 +243,7 @@ TEXPAR;
 
 #define SUBS	       0.7
 
-static double TeX(TEXPAR which, R_GE_gcontext *gc, GEDevDesc *dd)
+static double TeX(TEXPAR which, pGEcontext gc, pGEDevDesc dd)
 {
     switch(which) {
     case sigma2:  /* space */
@@ -332,7 +322,7 @@ static STYLE GetStyle(mathContext *mc)
     return mc->CurrentStyle;
 }
 
-static void SetStyle(STYLE newstyle, mathContext *mc, R_GE_gcontext *gc)
+static void SetStyle(STYLE newstyle, mathContext *mc, pGEcontext gc)
 {
     switch (newstyle) {
     case STYLE_D:
@@ -355,7 +345,7 @@ static void SetStyle(STYLE newstyle, mathContext *mc, R_GE_gcontext *gc)
     mc->CurrentStyle = newstyle;
 }
 
-static void SetPrimeStyle(STYLE style, mathContext *mc, R_GE_gcontext *gc)
+static void SetPrimeStyle(STYLE style, mathContext *mc, pGEcontext gc)
 {
     switch (style) {
     case STYLE_D:
@@ -377,7 +367,7 @@ static void SetPrimeStyle(STYLE style, mathContext *mc, R_GE_gcontext *gc)
     }
 }
 
-static void SetSupStyle(STYLE style, mathContext *mc, R_GE_gcontext *gc)
+static void SetSupStyle(STYLE style, mathContext *mc, pGEcontext gc)
 {
     switch (style) {
     case STYLE_D:
@@ -399,7 +389,7 @@ static void SetSupStyle(STYLE style, mathContext *mc, R_GE_gcontext *gc)
     }
 }
 
-static void SetSubStyle(STYLE style, mathContext *mc, R_GE_gcontext *gc)
+static void SetSubStyle(STYLE style, mathContext *mc, pGEcontext gc)
 {
     switch (style) {
     case STYLE_D:
@@ -417,7 +407,7 @@ static void SetSubStyle(STYLE style, mathContext *mc, R_GE_gcontext *gc)
     }
 }
 
-static void SetNumStyle(STYLE style, mathContext *mc, R_GE_gcontext *gc)
+static void SetNumStyle(STYLE style, mathContext *mc, pGEcontext gc)
 {
     switch (style) {
     case STYLE_D:
@@ -431,7 +421,7 @@ static void SetNumStyle(STYLE style, mathContext *mc, R_GE_gcontext *gc)
     }
 }
 
-static void SetDenomStyle(STYLE style, mathContext *mc, R_GE_gcontext *gc)
+static void SetDenomStyle(STYLE style, mathContext *mc, pGEcontext gc)
 {
     if (style > STYLE_T)
 	SetStyle(STYLE_T1, mc, gc);
@@ -439,7 +429,7 @@ static void SetDenomStyle(STYLE style, mathContext *mc, R_GE_gcontext *gc)
 	SetSubStyle(style, mc, gc);
 }
 
-static int IsCompactStyle(STYLE style, mathContext *mc, R_GE_gcontext *gc)
+static int IsCompactStyle(STYLE style, mathContext *mc, pGEcontext gc)
 {
     switch (style) {
     case STYLE_D1:
@@ -495,7 +485,7 @@ static BBOX MakeBBox(double height, double depth, double width)
     return bbox;
 }
 
-static BBOX NullBBox()
+static BBOX NullBBox(void)
 {
     BBOX bbox;
     bboxHeight(bbox) = 0;
@@ -567,7 +557,7 @@ static double CenterShift(BBOX bbox)
 
 #ifdef NOT_used_currently/*-- out 'def'	 (-Wall) --*/
 static BBOX DrawBBox(BBOX bbox, double xoffset, double yoffset,
-		     mathContext *mc, R_GE_gcontext *gc, GEDevDesc *dd)
+		     mathContext *mc, pGEcontext gc, pGEDevDesc dd)
 {
     double xsaved = mc->CurrentX;
     double ysaved = mc->CurrentY;
@@ -936,30 +926,33 @@ static int symbolAtom(SEXP expr)
 #ifdef NOT_used_currently/*-- out 'def'	 (-Wall) --*/
 static FontType mc->CurrentFont = 3;
 #endif
-static FontType GetFont(R_GE_gcontext *gc)
+static FontType GetFont(pGEcontext gc)
 {
     return gc->fontface;
 }
 
-static FontType SetFont(FontType font, R_GE_gcontext *gc)
+static FontType SetFont(FontType font, pGEcontext gc)
 {
     FontType prevfont = gc->fontface;
     gc->fontface = font;
     return prevfont;
 }
 
-static int UsingItalics(R_GE_gcontext *gc)
+static int UsingItalics(pGEcontext gc)
 {
     return (gc->fontface == ItalicFont ||
 	    gc->fontface == BoldItalicFont);
 }
 
-static BBOX GlyphBBox(int chr, R_GE_gcontext *gc, GEDevDesc *dd)
+extern int Rf_AdobeSymbol2ucs2(int n);
+static BBOX GlyphBBox(int chr, pGEcontext gc, pGEDevDesc dd)
 {
     BBOX bbox;
     double height, depth, width;
-    GEMetricInfo(chr, gc,
-		&height, &depth, &width, dd);
+    int chr1 = chr;
+    if(dd->dev->wantSymbolUTF8 && gc->fontface == 5) 
+	chr1 = -Rf_AdobeSymbol2ucs2(chr);
+    GEMetricInfo(chr1, gc, &height, &depth, &width, dd);
     bboxHeight(bbox) = fromDeviceHeight(height, MetricUnit, dd);
     bboxDepth(bbox)  = fromDeviceHeight(depth, MetricUnit, dd);
     bboxWidth(bbox)  = fromDeviceHeight(width, MetricUnit, dd);
@@ -968,20 +961,17 @@ static BBOX GlyphBBox(int chr, R_GE_gcontext *gc, GEDevDesc *dd)
     return bbox;
 }
 
-static BBOX RenderElement(SEXP, int,
-			  mathContext*, R_GE_gcontext*, GEDevDesc*);
+static BBOX RenderElement(SEXP, int, mathContext*, pGEcontext , pGEDevDesc);
 static BBOX RenderOffsetElement(SEXP, double, double, int,
-				mathContext*, R_GE_gcontext*, GEDevDesc*);
-static BBOX RenderExpression(SEXP, int,
-			     mathContext*, R_GE_gcontext*, GEDevDesc*);
-static BBOX RenderSymbolChar(int, int,
-			     mathContext*, R_GE_gcontext*, GEDevDesc*);
+				mathContext*, pGEcontext , pGEDevDesc);
+static BBOX RenderExpression(SEXP, int, mathContext*, pGEcontext , pGEDevDesc);
+static BBOX RenderSymbolChar(int, int, mathContext*, pGEcontext , pGEDevDesc);
 
 
 /*  Code to Generate Bounding Boxes and Draw Formulae.	*/
 
 static BBOX RenderItalicCorr(BBOX bbox, int draw, mathContext *mc,
-			     R_GE_gcontext *gc, GEDevDesc *dd)
+			     pGEcontext gc, pGEDevDesc dd)
 {
     if (bboxItalic(bbox) > 0) {
 	if (draw)
@@ -993,7 +983,7 @@ static BBOX RenderItalicCorr(BBOX bbox, int draw, mathContext *mc,
 }
 
 static BBOX RenderGap(double gap, int draw, mathContext *mc,
-		      R_GE_gcontext *gc, GEDevDesc *dd)
+		      pGEcontext gc, pGEDevDesc dd)
 {
     if (draw)
 	PMoveAcross(gap, mc);
@@ -1005,7 +995,7 @@ static BBOX RenderGap(double gap, int draw, mathContext *mc,
  */
 
 static BBOX RenderSymbolChar(int ascii, int draw, mathContext *mc,
-			     R_GE_gcontext *gc, GEDevDesc *dd)
+			     pGEcontext gc, pGEDevDesc dd)
 {
     FontType prev;
     BBOX bbox;
@@ -1018,7 +1008,8 @@ static BBOX RenderSymbolChar(int ascii, int draw, mathContext *mc,
     if (draw) {
 	asciiStr[0] = ascii;
 	asciiStr[1] = '\0';
-	GEText(ConvertedX(mc ,dd), ConvertedY(mc, dd), asciiStr,
+	GEText(ConvertedX(mc ,dd), ConvertedY(mc, dd), asciiStr, 
+	       CE_SYMBOL,
 	       0.0, 0.0, mc->CurrentAngle, gc,
 	       dd);
 	PMoveAcross(bboxWidth(bbox), mc);
@@ -1032,7 +1023,7 @@ static BBOX RenderSymbolChar(int ascii, int draw, mathContext *mc,
 /* every character. */
 
 static BBOX RenderSymbolStr(const char *str, int draw, mathContext *mc,
-			    R_GE_gcontext *gc, GEDevDesc *dd)
+			    pGEcontext gc, pGEDevDesc dd)
 {
     char chr[7] = "";
     const char *s = str;
@@ -1044,16 +1035,19 @@ static BBOX RenderSymbolStr(const char *str, int draw, mathContext *mc,
 
     if (str) {
 #ifdef SUPPORT_MBCS
-	if(mbcslocale) {  /* need to advance by character, not byte */
+	/* Need to advance by character, not byte, except in the symbol font.
+	   The latter would be hard to achieve, but perhaps not impossible.
+	 */
+	if(mbcslocale && gc->fontface != 5) {
 	    wchar_t wc;
 	    mbstate_t mb_st;
 	    size_t res;
 
-	    memset(&mb_st, 0, sizeof(mb_st));
+	    mbs_init(&mb_st);
 	    while (*s) {
 		wc = 0;
 		res = mbrtowc(&wc, s, MB_LEN_MAX, &mb_st);
-		if(res == -1) error("invalid multibyte string");
+		if(res == -1) error("invalid multibyte string '%s'", s);
 		if (iswdigit(wc) && font != PlainFont) {
 		    font = PlainFont;
 		    SetFont(PlainFont, gc);
@@ -1070,12 +1064,12 @@ static BBOX RenderSymbolStr(const char *str, int draw, mathContext *mc,
 		    bboxItalic(glyphBBox) = 0;
 		if (draw) {
 		    memset(chr, 0, sizeof(chr));
-		    res  = wcrtomb(chr, wc, &mb_st);
-		    if(res == -1) error("invalid multibyte string");
+		    /* should not be possible, as we just converted to wc */
+		    if(wcrtomb(chr, wc, &mb_st) == -1)
+			error("invalid multibyte string");
 		    PMoveAcross(lastItalicCorr, mc);
-		    GEText(ConvertedX(mc ,dd), ConvertedY(mc, dd), chr,
-			   0.0, 0.0, mc->CurrentAngle, gc,
-			   dd);
+		    GEText(ConvertedX(mc ,dd), ConvertedY(mc, dd), chr, CE_NATIVE,
+			   0.0, 0.0, mc->CurrentAngle, gc, dd);
 		    PMoveAcross(bboxWidth(glyphBBox), mc);
 		}
 		bboxWidth(resultBBox) += lastItalicCorr;
@@ -1104,9 +1098,8 @@ static BBOX RenderSymbolStr(const char *str, int draw, mathContext *mc,
 		if (draw) {
 		    chr[0] = *s;
 		    PMoveAcross(lastItalicCorr, mc);
-		    GEText(ConvertedX(mc ,dd), ConvertedY(mc, dd), chr,
-			   0.0, 0.0, mc->CurrentAngle, gc,
-			   dd);
+		    GEText(ConvertedX(mc ,dd), ConvertedY(mc, dd), chr, CE_NATIVE,
+			   0.0, 0.0, mc->CurrentAngle, gc, dd);
 		    PMoveAcross(bboxWidth(glyphBBox), mc);
 		}
 		bboxWidth(resultBBox) += lastItalicCorr;
@@ -1126,7 +1119,7 @@ static BBOX RenderSymbolStr(const char *str, int draw, mathContext *mc,
 
 /* This only gets called from RenderAccent */
 static BBOX RenderChar(int ascii, int draw, mathContext *mc,
-		       R_GE_gcontext *gc, GEDevDesc *dd)
+		       pGEcontext gc, pGEDevDesc dd)
 {
     BBOX bbox;
     char asciiStr[7];
@@ -1142,7 +1135,7 @@ static BBOX RenderChar(int ascii, int draw, mathContext *mc,
 	} else
 #endif
 	    asciiStr[0] = ascii;
-	GEText(ConvertedX(mc ,dd), ConvertedY(mc, dd), asciiStr,
+	GEText(ConvertedX(mc ,dd), ConvertedY(mc, dd), asciiStr, CE_NATIVE,
 	       0.0, 0.0, mc->CurrentAngle, gc,
 	       dd);
 	PMoveAcross(bboxWidth(bbox), mc);
@@ -1152,13 +1145,15 @@ static BBOX RenderChar(int ascii, int draw, mathContext *mc,
 
 /* This gets called on strings and PRINTNAMES */
 static BBOX RenderStr(const char *str, int draw, mathContext *mc,
-		      R_GE_gcontext *gc, GEDevDesc *dd)
+		      pGEcontext gc, pGEDevDesc dd)
 {
-    BBOX glyphBBox;
+    BBOX glyphBBox = NullBBox(); /* might be use do italic corr on str="" */
     BBOX resultBBox = NullBBox();
+    int nc = 0;
 
     if (str) {
 #ifdef SUPPORT_MBCS
+	/* need to advance by character, not byte, except in the symbol font */
 	if(mbcslocale && gc->fontface != 5) {
 	    int n = strlen(str), used;
 	    wchar_t wc;
@@ -1168,7 +1163,7 @@ static BBOX RenderStr(const char *str, int draw, mathContext *mc,
 	    while ((used = Mbrtowc(&wc, p, n, &mb_st)) > 0) {
 		glyphBBox = GlyphBBox(wc, gc, dd);
 		resultBBox = CombineBBoxes(resultBBox, glyphBBox);
-		p += used; n -= used;
+		p += used; n -= used; nc++;
 	    }
 	} else
 #endif
@@ -1177,11 +1172,16 @@ static BBOX RenderStr(const char *str, int draw, mathContext *mc,
 	    while (*s) {
 		glyphBBox = GlyphBBox(*s, gc, dd);
 		resultBBox = CombineBBoxes(resultBBox, glyphBBox);
-		s++;
+		s++; nc++;
 	    }
 	}
+	if(nc > 1) {
+	    /* Finding the width by adding up boxes is incorrect (kerning) */
+	    double wd = GEStrWidth(str, CE_NATIVE, gc, dd);
+	    bboxWidth(resultBBox) = fromDeviceHeight(wd, MetricUnit, dd);
+	}
 	if (draw) {
-	    GEText(ConvertedX(mc ,dd), ConvertedY(mc, dd), str,
+	    GEText(ConvertedX(mc ,dd), ConvertedY(mc, dd), str, CE_NATIVE,
 		   0.0, 0.0, mc->CurrentAngle, gc, dd);
 	    PMoveAcross(bboxWidth(resultBBox), mc);
 	}
@@ -1198,7 +1198,7 @@ static BBOX RenderStr(const char *str, int draw, mathContext *mc,
 /* Code for Symbol Font Atoms */
 
 static BBOX RenderSymbol(SEXP expr, int draw, mathContext *mc,
-			 R_GE_gcontext *gc, GEDevDesc *dd)
+			 pGEcontext gc, pGEDevDesc dd)
 {
     int code;
     if ((code = TranslatedSymbol(expr)))
@@ -1208,7 +1208,7 @@ static BBOX RenderSymbol(SEXP expr, int draw, mathContext *mc,
 }
 
 static BBOX RenderSymbolString(SEXP expr, int draw, mathContext *mc,
-			       R_GE_gcontext *gc, GEDevDesc *dd)
+			       pGEcontext gc, pGEDevDesc dd)
 {
     int code;
     if ((code = TranslatedSymbol(expr)))
@@ -1221,7 +1221,7 @@ static BBOX RenderSymbolString(SEXP expr, int draw, mathContext *mc,
 /* Code for Numeric Atoms */
 
 static BBOX RenderNumber(SEXP expr, int draw, mathContext *mc,
-			 R_GE_gcontext *gc, GEDevDesc *dd)
+			 pGEcontext gc, pGEDevDesc dd)
 {
     BBOX bbox;
     FontType prevfont = SetFont(PlainFont, gc);
@@ -1233,7 +1233,7 @@ static BBOX RenderNumber(SEXP expr, int draw, mathContext *mc,
 /* Code for String Atoms */
 
 static BBOX RenderString(SEXP expr, int draw, mathContext *mc,
-			 R_GE_gcontext *gc, GEDevDesc *dd)
+			 pGEcontext gc, pGEDevDesc dd)
 {
     return RenderStr(translateChar(STRING_ELT(expr, 0)), draw, mc, gc, dd);
 }
@@ -1250,7 +1250,7 @@ static int DotsAtom(SEXP expr)
 }
 
 static BBOX RenderDots(SEXP expr, int draw, mathContext *mc,
-		       R_GE_gcontext *gc, GEDevDesc *dd)
+		       pGEcontext gc, pGEDevDesc dd)
 {
     BBOX bbox = RenderSymbolChar(S_ELLIPSIS, 0, mc, gc, dd);
     if (NameMatch(expr, "cdots") || NameMatch(expr, "...")) {
@@ -1276,7 +1276,7 @@ static BBOX RenderDots(SEXP expr, int draw, mathContext *mc,
  */
 
 static BBOX RenderAtom(SEXP expr, int draw, mathContext *mc,
-		       R_GE_gcontext *gc, GEDevDesc *dd)
+		       pGEcontext gc, pGEDevDesc dd)
 {
     if (NameAtom(expr)) {
 	if (DotsAtom(expr))
@@ -1308,7 +1308,7 @@ static int SpaceAtom(SEXP expr)
 
 
 static BBOX RenderSpace(SEXP expr, int draw, mathContext *mc,
-			R_GE_gcontext *gc, GEDevDesc *dd)
+			pGEcontext gc, pGEDevDesc dd)
 {
 
     BBOX opBBox, arg1BBox, arg2BBox;
@@ -1360,8 +1360,8 @@ static int BinAtom(SEXP expr)
 
 #define SLASH2
 
-static BBOX RenderSlash(int draw, mathContext *mc, R_GE_gcontext *gc,
-			GEDevDesc *dd)
+static BBOX RenderSlash(int draw, mathContext *mc, pGEcontext gc,
+			pGEDevDesc dd)
 {
 #ifdef SLASH0
     /* The Default Font Character */
@@ -1434,7 +1434,7 @@ static BBOX RenderSlash(int draw, mathContext *mc, R_GE_gcontext *gc,
 }
 
 static BBOX RenderBin(SEXP expr, int draw, mathContext *mc,
-		      R_GE_gcontext *gc, GEDevDesc *dd)
+		      pGEcontext gc, pGEDevDesc dd)
 {
     int op = BinAtom(CAR(expr));
     int nexpr = length(expr);
@@ -1506,7 +1506,7 @@ static int SubAtom(SEXP expr)
 /* We do not need to save and restore the */
 /* current location here.  This is paranoia. */
 static BBOX RenderSub(SEXP expr, int draw, mathContext *mc,
-		      R_GE_gcontext *gc, GEDevDesc *dd)
+		      pGEcontext gc, pGEDevDesc dd)
 {
     BBOX bodyBBox, subBBox;
     SEXP body = CADR(expr);
@@ -1532,7 +1532,7 @@ static BBOX RenderSub(SEXP expr, int draw, mathContext *mc,
 }
 
 static BBOX RenderSup(SEXP expr, int draw, mathContext *mc,
-		      R_GE_gcontext *gc, GEDevDesc *dd)
+		      pGEcontext gc, pGEDevDesc dd)
 {
     BBOX bodyBBox, subBBox, supBBox;
     SEXP body = CADR(expr);
@@ -1626,7 +1626,7 @@ static int WideTildeAtom(SEXP expr)
 }
 
 static BBOX RenderWideTilde(SEXP expr, int draw, mathContext *mc,
-			    R_GE_gcontext *gc, GEDevDesc *dd)
+			    pGEcontext gc, pGEDevDesc dd)
 {
     double savedX = mc->CurrentX;
     double savedY = mc->CurrentY;
@@ -1679,7 +1679,7 @@ static int WideHatAtom(SEXP expr)
 }
 
 static BBOX RenderWideHat(SEXP expr, int draw, mathContext *mc,
-			  R_GE_gcontext *gc, GEDevDesc *dd)
+			  pGEcontext gc, pGEDevDesc dd)
 {
     double savedX = mc->CurrentX;
     double savedY = mc->CurrentY;
@@ -1722,7 +1722,7 @@ static int BarAtom(SEXP expr)
 }
 
 static BBOX RenderBar(SEXP expr, int draw, mathContext *mc,
-		      R_GE_gcontext *gc, GEDevDesc *dd)
+		      pGEcontext gc, pGEDevDesc dd)
 {
     double savedX = mc->CurrentX;
     double savedY = mc->CurrentY;
@@ -1786,7 +1786,7 @@ static void InvalidAccent(SEXP expr)
 }
 
 static BBOX RenderAccent(SEXP expr, int draw, mathContext *mc,
-			 R_GE_gcontext *gc, GEDevDesc *dd)
+			 pGEcontext gc, pGEDevDesc dd)
 {
     SEXP body, accent;
     double savedX = mc->CurrentX;
@@ -1845,7 +1845,7 @@ static BBOX RenderAccent(SEXP expr, int draw, mathContext *mc,
 
 static void NumDenomVShift(BBOX numBBox, BBOX denomBBox,
 			   double *u, double *v,
-			   mathContext *mc, R_GE_gcontext *gc, GEDevDesc *dd)
+			   mathContext *mc, pGEcontext gc, pGEDevDesc dd)
 {
     double a, delta, phi, theta;
     a = TeX(sigma22, gc, dd);
@@ -1889,7 +1889,7 @@ static void NumDenomHShift(BBOX numBBox, BBOX denomBBox,
 }
 
 static BBOX RenderFraction(SEXP expr, int rule, int draw,
-			   mathContext *mc, R_GE_gcontext *gc, GEDevDesc *dd)
+			   mathContext *mc, pGEcontext gc, pGEDevDesc dd)
 {
     SEXP numerator = CADR(expr);
     SEXP denominator = CADDR(expr);
@@ -1954,7 +1954,7 @@ static BBOX RenderFraction(SEXP expr, int rule, int draw,
 }
 
 static BBOX RenderUnderline(SEXP expr, int draw, mathContext *mc,
-	        	    R_GE_gcontext *gc, GEDevDesc *dd)
+	        	    pGEcontext gc, pGEDevDesc dd)
 {
     SEXP body = CADR(expr);
     BBOX BBox;
@@ -2002,7 +2002,7 @@ static int OverAtom(SEXP expr)
 }
 
 static BBOX RenderOver(SEXP expr, int draw, mathContext *mc,
-		       R_GE_gcontext *gc, GEDevDesc *dd)
+		       pGEcontext gc, pGEDevDesc dd)
 {
     return RenderFraction(expr, 1, draw, mc, gc, dd);
 }
@@ -2013,7 +2013,7 @@ static int UnderlAtom(SEXP expr)
 }
 
 static BBOX RenderUnderl(SEXP expr, int draw, mathContext *mc,
-			 R_GE_gcontext *gc, GEDevDesc *dd)
+			 pGEcontext gc, pGEDevDesc dd)
 {
     return RenderUnderline(expr, draw, mc, gc, dd);
 }
@@ -2025,7 +2025,7 @@ static int AtopAtom(SEXP expr)
 }
 
 static BBOX RenderAtop(SEXP expr, int draw, mathContext *mc,
-		       R_GE_gcontext *gc, GEDevDesc *dd)
+		       pGEcontext gc, pGEDevDesc dd)
 {
     return RenderFraction(expr, 0, draw, mc, gc, dd);
 }
@@ -2081,7 +2081,7 @@ static int DelimCode(SEXP expr, SEXP head)
 }
 
 static BBOX RenderDelimiter(int delim, int draw, mathContext *mc,
-			    R_GE_gcontext *gc, GEDevDesc *dd)
+			    pGEcontext gc, pGEDevDesc dd)
 {
     BBOX bbox;
     double savecex = gc->cex;
@@ -2097,7 +2097,7 @@ static int GroupAtom(SEXP expr)
 }
 
 static BBOX RenderGroup(SEXP expr, int draw, mathContext *mc,
-			R_GE_gcontext *gc, GEDevDesc *dd)
+			pGEcontext gc, pGEDevDesc dd)
 {
     double cexSaved = gc->cex;
     BBOX bbox;
@@ -2134,7 +2134,7 @@ static int BGroupAtom(SEXP expr)
 }
 
 static BBOX RenderDelim(int which, double dist, int draw, mathContext *mc,
-			R_GE_gcontext *gc, GEDevDesc *dd)
+			pGEcontext gc, pGEDevDesc dd)
 {
     double savedX = mc->CurrentX;
     double savedY = mc->CurrentY;
@@ -2241,7 +2241,7 @@ static BBOX RenderDelim(int which, double dist, int draw, mathContext *mc,
 }
 
 static BBOX RenderBGroup(SEXP expr, int draw, mathContext *mc,
-			 R_GE_gcontext *gc, GEDevDesc *dd)
+			 pGEcontext gc, pGEDevDesc dd)
 {
     double dist;
     BBOX bbox;
@@ -2275,7 +2275,7 @@ static int ParenAtom(SEXP expr)
 }
 
 static BBOX RenderParen(SEXP expr, int draw, mathContext *mc,
-			R_GE_gcontext *gc, GEDevDesc *dd)
+			pGEcontext gc, pGEDevDesc dd)
 {
     BBOX bbox;
     bbox = RenderDelimiter(S_PARENLEFT, draw, mc, gc, dd);
@@ -2296,8 +2296,8 @@ static int IntAtom(SEXP expr)
 }
 
 
-static BBOX RenderIntSymbol(int draw, mathContext *mc, R_GE_gcontext *gc,
-			    GEDevDesc *dd)
+static BBOX RenderIntSymbol(int draw, mathContext *mc, pGEcontext gc,
+			    pGEDevDesc dd)
 {
     double savedX = mc->CurrentX;
     double savedY = mc->CurrentY;
@@ -2326,7 +2326,7 @@ static BBOX RenderIntSymbol(int draw, mathContext *mc, R_GE_gcontext *gc,
 }
 
 static BBOX RenderInt(SEXP expr, int draw, mathContext *mc,
-		      R_GE_gcontext *gc, GEDevDesc *dd)
+		      pGEcontext gc, pGEDevDesc dd)
 {
     BBOX opBBox, lowerBBox, upperBBox, bodyBBox;
     int nexpr = length(expr);
@@ -2405,7 +2405,7 @@ static int OpAtom(SEXP expr)
 }
 
 static BBOX RenderOpSymbol(SEXP op, int draw, mathContext *mc,
-			   R_GE_gcontext *gc, GEDevDesc *dd)
+			   pGEcontext gc, pGEDevDesc dd)
 {
     BBOX bbox;
     double cexSaved = gc->cex;
@@ -2441,9 +2441,9 @@ static BBOX RenderOpSymbol(SEXP op, int draw, mathContext *mc,
 }
 
 static BBOX RenderOp(SEXP expr, int draw, mathContext *mc,
-		     R_GE_gcontext *gc, GEDevDesc *dd)
+		     pGEcontext gc, pGEDevDesc dd)
 {
-    BBOX lowerBBox, upperBBox, bodyBBox;
+    BBOX lowerBBox = NullBBox() /* -Wall */, upperBBox = NullBBox(), bodyBBox;
     double savedX = mc->CurrentX;
     double savedY = mc->CurrentY;
     int nexpr = length(expr);
@@ -2528,7 +2528,7 @@ static int RadicalAtom(SEXP expr)
 }
 
 static BBOX RenderScript(SEXP expr, int draw, mathContext *mc,
-			 R_GE_gcontext *gc, GEDevDesc *dd)
+			 pGEcontext gc, pGEDevDesc dd)
 {
     BBOX bbox;
     STYLE style = GetStyle(mc);
@@ -2539,7 +2539,7 @@ static BBOX RenderScript(SEXP expr, int draw, mathContext *mc,
 }
 
 static BBOX RenderRadical(SEXP expr, int draw, mathContext *mc,
-			  R_GE_gcontext *gc, GEDevDesc *dd)
+			  pGEcontext gc, pGEDevDesc dd)
 {
     SEXP body = CADR(expr);
     SEXP order = CADDR(expr);
@@ -2638,7 +2638,7 @@ static int AbsAtom(SEXP expr)
 }
 
 static BBOX RenderAbs(SEXP expr, int draw, mathContext *mc,
-		      R_GE_gcontext *gc, GEDevDesc *dd)
+		      pGEcontext gc, pGEDevDesc dd)
 {
     BBOX bbox = RenderElement(CADR(expr), 0, mc, gc, dd);
     double height = bboxHeight(bbox);
@@ -2701,7 +2701,7 @@ static int CurlyAtom(SEXP expr)
 }
 
 static BBOX RenderCurly(SEXP expr, int draw, mathContext *mc,
-			R_GE_gcontext *gc, GEDevDesc *dd)
+			pGEcontext gc, pGEDevDesc dd)
 {
     return RenderElement(CADR(expr), draw, mc, gc, dd);
 }
@@ -2759,7 +2759,7 @@ static int RelAtom(SEXP expr)
 }
 
 static BBOX RenderRel(SEXP expr, int draw, mathContext *mc,
-		      R_GE_gcontext *gc, GEDevDesc *dd)
+		      pGEcontext gc, pGEDevDesc dd)
 {
     int op = RelAtom(CAR(expr));
     int nexpr = length(expr);
@@ -2795,7 +2795,7 @@ static int BoldAtom(SEXP expr)
 }
 
 static BBOX RenderBold(SEXP expr, int draw, mathContext *mc,
-		       R_GE_gcontext *gc, GEDevDesc *dd)
+		       pGEcontext gc, pGEDevDesc dd)
 {
     BBOX bbox;
     FontType prevfont = SetFont(BoldFont, gc);
@@ -2817,7 +2817,7 @@ static int ItalicAtom(SEXP expr)
 }
 
 static BBOX RenderItalic(SEXP expr, int draw, mathContext *mc,
-			 R_GE_gcontext *gc, GEDevDesc *dd)
+			 pGEcontext gc, pGEDevDesc dd)
 {
     BBOX bbox;
     FontType prevfont = SetFont(ItalicFont, gc);
@@ -2839,7 +2839,7 @@ static int PlainAtom(SEXP expr)
 }
 
 static BBOX RenderPlain(SEXP expr, int draw, mathContext *mc,
-			R_GE_gcontext *gc, GEDevDesc *dd)
+			pGEcontext gc, pGEDevDesc dd)
 {
     BBOX bbox;
     int prevfont = SetFont(PlainFont, gc);
@@ -2867,7 +2867,7 @@ static int SymbolFaceAtom(SEXP expr)
 }
 
 static BBOX RenderSymbolFace(SEXP expr, int draw, mathContext *mc,
-                             R_GE_gcontext *gc, GEDevDesc *dd)
+                             pGEcontext gc, pGEDevDesc dd)
 {
     BBOX bbox;
     int prevfont = SetFont(SymbolFont, gc);
@@ -2889,7 +2889,7 @@ static int BoldItalicAtom(SEXP expr)
 }
 
 static BBOX RenderBoldItalic(SEXP expr, int draw, mathContext *mc,
-			     R_GE_gcontext *gc, GEDevDesc *dd)
+			     pGEcontext gc, pGEDevDesc dd)
 {
     BBOX bbox;
     int prevfont = SetFont(BoldItalicFont, gc);
@@ -2914,7 +2914,7 @@ static int StyleAtom(SEXP expr)
 }
 
 static BBOX RenderStyle(SEXP expr, int draw, mathContext *mc,
-			R_GE_gcontext *gc, GEDevDesc *dd)
+			pGEcontext gc, pGEDevDesc dd)
 {
     STYLE prevstyle = GetStyle(mc);
     BBOX bbox;
@@ -2945,7 +2945,7 @@ static int PhantomAtom(SEXP expr)
 }
 
 static BBOX RenderPhantom(SEXP expr, int draw, mathContext *mc,
-			  R_GE_gcontext *gc, GEDevDesc *dd)
+			  pGEcontext gc, pGEDevDesc dd)
 {
     BBOX bbox = RenderElement(CADR(expr), 0, mc, gc, dd);
     if (NameMatch(CAR(expr), "vphantom")) {
@@ -2968,7 +2968,7 @@ static int ConcatenateAtom(SEXP expr)
 }
 
 static BBOX RenderConcatenate(SEXP expr, int draw, mathContext *mc,
-			      R_GE_gcontext *gc, GEDevDesc *dd)
+			      pGEcontext gc, pGEDevDesc dd)
 {
     BBOX bbox = NullBBox();
     int i, n;
@@ -2992,7 +2992,7 @@ static BBOX RenderConcatenate(SEXP expr, int draw, mathContext *mc,
  */
 
 static BBOX RenderCommaList(SEXP expr, int draw, mathContext *mc,
-			    R_GE_gcontext *gc, GEDevDesc *dd)
+			    pGEcontext gc, pGEDevDesc dd)
 {
     BBOX bbox = NullBBox();
     double small = 0.4 * ThinSpace(gc, dd);
@@ -3032,7 +3032,7 @@ static BBOX RenderCommaList(SEXP expr, int draw, mathContext *mc,
  */
 
 static BBOX RenderExpression(SEXP expr, int draw, mathContext *mc,
-			     R_GE_gcontext *gc, GEDevDesc *dd)
+			     pGEcontext gc, pGEDevDesc dd)
 {
     BBOX bbox;
     if (NameAtom(CAR(expr)))
@@ -3059,7 +3059,7 @@ static int ListAtom(SEXP expr)
 }
 
 static BBOX RenderList(SEXP expr, int draw, mathContext *mc,
-		       R_GE_gcontext *gc, GEDevDesc *dd)
+		       pGEcontext gc, pGEDevDesc dd)
 {
     return RenderCommaList(CDR(expr), draw, mc, gc, dd);
 }
@@ -3068,7 +3068,7 @@ static BBOX RenderList(SEXP expr, int draw, mathContext *mc,
 
 
 static BBOX RenderFormula(SEXP expr, int draw, mathContext *mc,
-			  R_GE_gcontext *gc, GEDevDesc *dd)
+			  pGEcontext gc, pGEDevDesc dd)
 {
     SEXP head = CAR(expr);
 
@@ -3139,7 +3139,7 @@ static BBOX RenderFormula(SEXP expr, int draw, mathContext *mc,
 /* or formula (some sort of expression) */
 
 static BBOX RenderElement(SEXP expr, int draw, mathContext *mc,
-			  R_GE_gcontext *gc, GEDevDesc *dd)
+			  pGEcontext gc, pGEDevDesc dd)
 {
     if (FormulaExpression(expr))
 	return RenderFormula(expr, draw, mc, gc, dd);
@@ -3148,8 +3148,8 @@ static BBOX RenderElement(SEXP expr, int draw, mathContext *mc,
 }
 
 static BBOX RenderOffsetElement(SEXP expr, double x, double y, int draw,
-				mathContext *mc, R_GE_gcontext *gc,
-				GEDevDesc *dd)
+				mathContext *mc, pGEcontext gc,
+				pGEDevDesc dd)
 {
     BBOX bbox;
     double savedX = mc->CurrentX;
@@ -3174,8 +3174,8 @@ static BBOX RenderOffsetElement(SEXP expr, double x, double y, int draw,
 /* BBOXes are in INCHES (see MetricUnit) */
 
 double GEExpressionWidth(SEXP expr,
-			 R_GE_gcontext *gc,
-			 GEDevDesc *dd)
+			 pGEcontext gc,
+			 pGEDevDesc dd)
 {
     BBOX bbox;
     double width;
@@ -3213,8 +3213,8 @@ double GEExpressionWidth(SEXP expr,
 }
 
 double GEExpressionHeight(SEXP expr,
-			  R_GE_gcontext *gc,
-			  GEDevDesc *dd)
+			  pGEcontext gc,
+			  pGEDevDesc dd)
 {
     BBOX bbox;
     double height;
@@ -3252,22 +3252,18 @@ double GEExpressionHeight(SEXP expr,
 
 void GEMathText(double x, double y, SEXP expr,
 		double xc, double yc, double rot,
-		R_GE_gcontext *gc,
-		GEDevDesc *dd)
+		pGEcontext gc,
+		pGEDevDesc dd)
 {
     BBOX bbox;
     mathContext mc;
 
-#ifdef BUG61
-#else
-    /* IF font metric information is not available for device */
-    /* then bail out */
+    /* If font metric information is not available for device
+       then bail out */
     double ascent, descent, width;
-    GEMetricInfo(0, gc,
-		&ascent, &descent, &width, dd);
-    if ((ascent==0) && (descent==0) && (width==0))
-	error(_("Metric information not available for this device"));
-#endif
+    GEMetricInfo('M', gc, &ascent, &descent, &width, dd);
+    if ((ascent == 0.0) && (descent == 0.0) && (width == 0.0))
+	error(_("Metric information not available for this family/device"));
 
     /*
      * Build a "drawing context" for the current expression
@@ -3275,6 +3271,7 @@ void GEMathText(double x, double y, SEXP expr,
     mc.BaseCex = gc->cex;
     mc.BoxColor = name2col("pink");
     mc.CurrentStyle = STYLE_D;
+
     /*
      * Some "empty" values.  Will be filled in after BBox is calc'ed
      */
@@ -3293,7 +3290,7 @@ void GEMathText(double x, double y, SEXP expr,
     if (R_FINITE(xc))
 	mc.CurrentX = mc.ReferenceX - xc * bboxWidth(bbox);
     else
-	/* Paul 11/2/02
+	/* Paul 2002-02-11
 	 * If xc == NA then should centre horizontally.
 	 * Used to left-adjust.
 	 */
@@ -3314,152 +3311,3 @@ void GEMathText(double x, double y, SEXP expr,
     mc.SinAngle = sin(rot);
     RenderElement(expr, 1, &mc, gc, dd);
 }/* GEMathText */
-
-
-/********************************
- * Code below here ...
- * ... should be moved to base.c and
- * ... is part of the base graphics API NOT the graphics engine API
- ********************************
- */
-double GExpressionWidth(SEXP expr, GUnit units, DevDesc *dd)
-{
-    R_GE_gcontext gc;
-    double width;
-    gcontextFromGP(&gc, dd);
-    width = GEExpressionWidth(expr, &gc, (GEDevDesc*) dd);
-    if (units == DEVICE)
-	return width;
-    else
-	return GConvertXUnits(width, DEVICE, units, dd);
-}
-
-double GExpressionHeight(SEXP expr, GUnit units, DevDesc *dd)
-{
-    R_GE_gcontext gc;
-    double height;
-    gcontextFromGP(&gc, dd);
-    height = GEExpressionHeight(expr, &gc, (GEDevDesc*) dd);
-    if (units == DEVICE)
-	return height;
-    else
-	return GConvertYUnits(height, DEVICE, units, dd);
-}
-
-/* This is just here to satisfy the Rgraphics.h API.
- * This allows new graphics API (GraphicsDevice.h, GraphicsEngine.h)
- * to be developed alongside.
- * Could be removed if Rgraphics.h ever gets REPLACED by new API
- * NOTE that base graphics code no longer calls this -- the base
- * graphics system directly calls the graphics engine for mathematical
- * annotation (GEMathText)
- */
-void GMathText(double x, double y, int coords, SEXP expr,
-	       double xc, double yc, double rot,
-	       DevDesc *dd)
-{
-    R_GE_gcontext gc;
-    gcontextFromGP(&gc, dd);
-    GConvert(&x, &y, coords, DEVICE, dd);
-    GClip(dd);
-    GEMathText(x, y, expr, xc, yc, rot, &gc, (GEDevDesc*) dd);
-}
-
-void GMMathText(SEXP str, int side, double line, int outer,
-		double at, int las, double yadj, DevDesc *dd)
-{
-    int coords = 0, subcoords;
-    double xadj, angle = 0;
-
-#ifdef BUG61
-#else
-    /* IF font metric information is not available for device */
-    /* then bail out */
-    double ascent, descent, width;
-    GMetricInfo(0, &ascent, &descent, &width, DEVICE, dd);
-    if ((ascent==0) && (descent==0) && (width==0))
-	error(_("Metric information not available for this device"));
-#endif
-
-    xadj = Rf_gpptr(dd)->adj;
-
-    /* This is MOSTLY the same as the same section of GMtext
-     * BUT it differs because it sets different values for yadj for
-     * different situations.
-     * Paul
-     */
-     /* changed to unify behaviour with changes in GMText. Uwe */
-    if(outer) {
-	switch(side) {
-	case 1:	    coords = OMA1;	break;
-	case 2:	    coords = OMA2;	break;
-	case 3:	    coords = OMA3;	break;
-	case 4:	    coords = OMA4;	break;
-	}
-	subcoords = NIC;
-    }
-    else {
-	switch(side) {
-	case 1:	    coords = MAR1;	break;
-	case 2:	    coords = MAR2;	break;
-	case 3:	    coords = MAR3;	break;
-	case 4:	    coords = MAR4;	break;
-	}
-	subcoords = USER;
-    }
-    /* Note: I changed Rf_gpptr(dd)->yLineBias to 0.3 here. */
-    /* Purely visual tuning. RI */
-    /* Note: I removed the 0.3 fiddle here because mathematical
-     * annotation stuff can do "exact" centering.
-     * i.e., 0.3 fiddle is effectively replaced by yadj=0.5
-     */
-    switch(side) {
-    case 1:
-	if(las == 2 || las == 3) {
-	    angle = 90;
-	}
-	else {
-	    /*	    line = line + 1 - Rf_gpptr(dd)->yLineBias;
-		    angle = 0;
-		    yadj = NA_REAL; */
-	    line = line + 1;
-	    angle = 0;
-	}
-	break;
-    case 2:
-	if(las == 1 || las == 2) {
-	    angle = 0;
-	}
-	else {
-	    /*	    line = line + Rf_gpptr(dd)->yLineBias;
-		    angle = 90;
-		    yadj = NA_REAL; */
-	    angle = 90;
-	}
-	break;
-    case 3:
-	if(las == 2 || las == 3) {
-	    angle = 90;
-	}
-	else {
-	    /*   line = line + Rf_gpptr(dd)->yLineBias;
-		 angle = 0;
-		 yadj = NA_REAL; */
-	    angle = 0;
-	}
-	break;
-    case 4:
-	if(las == 1 || las == 2) {
-	    angle = 0;
-	}
-	else {
-	    /*   line = line + 1 - Rf_gpptr(dd)->yLineBias;
-		 angle = 90;
-		 yadj = NA_REAL; */
-	    line = line + 1;
-	    angle = 90;
-	}
-	break;
-    }
-    GMathText(at, line, coords, str, xadj, yadj, angle, dd);
-}/* GMMathText */
