@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2002-2007	The R Development Core Team.
+ *  Copyright (C) 2002-2008	The R Development Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -55,7 +55,6 @@
     a C++ compiler because the linkage changes as the declarations
     are (currently) within extern "C" blocks.
 */
-#include <Rdevices.h>       /* For declaration of InitGraphics() */
 #include <R_ext/Callbacks.h>
 #include <Rdynpriv.h>
 
@@ -150,8 +149,11 @@ static R_CMethodDef cMethods [] = {
     {"signrank_free", (DL_FUNC)&signrank_free, 0, NULL},
     {"wilcox_free", (DL_FUNC)&wilcox_free, 0, NULL},
 
+#if 0
+    /* Why are these here?  Not used (currently) */
     {"InitGraphics", (DL_FUNC)&Rf_InitGraphics, 0, NULL},
     {"InitColors", (DL_FUNC)&Rf_InitColors, 0, NULL},
+#endif
     {NULL, NULL, 0}
 };
 
@@ -164,6 +166,10 @@ static R_CallMethodDef callMethods [] = {
     CALLDEF(La_svd, 7),
     CALLDEF(La_rs, 2),
     CALLDEF(La_rg, 2),
+    CALLDEF(La_dgecon, 2),
+    CALLDEF(La_dtrcon, 2),
+    CALLDEF(La_zgecon, 2),
+    CALLDEF(La_ztrcon, 2),
     CALLDEF(La_zgesv, 2),
     CALLDEF(La_zgeqp3, 1),
     CALLDEF(qr_coef_cmplx, 2),
@@ -229,6 +235,9 @@ static R_CallMethodDef callMethods [] = {
     CALLDEF(R_stopbcprof, 0),
 #endif
 
+    /* base graphics */
+    CALLDEF(Rg_contourDef, 0),
+    CALLDEF(Rg_readonlypars, 0),
 
     {NULL, NULL, 0}
 };
