@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2007  R Development Core Team
+ *  Copyright (C) 1997--2008  R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -435,18 +435,6 @@ static SEXP complex_relop(RELOP_TYPE code, SEXP s1, SEXP s2, SEXP call)
     return ans;
 }
 
-#if defined(Win32) && defined(SUPPORT_UTF8)
-#define STRCOLL Rstrcoll
-#else
-
-#ifdef HAVE_STRCOLL
-#define STRCOLL strcoll
-#else
-#define STRCOLL strcmp
-#endif
-
-#endif
-
 
 /* POSIX allows EINVAL when one of the strings contains characters
    outside the collation domain. */
@@ -472,7 +460,7 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
  	    else if (c1 == c2)  /* This will pretest all cached strings */
 		LOGICAL(ans)[i] = 1;
 	    else
-		if (strcmp(translateChar(c1), translateChar(c2)) == 0)
+		if (Seql(c1, c2))
 		    LOGICAL(ans)[i] = 1;
 		else
 		    LOGICAL(ans)[i] = 0;
@@ -487,7 +475,7 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
  	    else if (c1 == c2)
 		LOGICAL(ans)[i] = 0;
  	    else
-		if (strcmp(translateChar(c1), translateChar(c2)) == 0)
+		if (Seql(c1, c2))
 		    LOGICAL(ans)[i] = 0;
 		else
 		    LOGICAL(ans)[i] = 1;
@@ -503,7 +491,7 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 		LOGICAL(ans)[i] = 0;
  	    else {
 		errno = 0;
-		res = STRCOLL(translateChar(c1), translateChar(c2));
+		res = Scollate(c1, c2);
 		if(errno)  
 		    LOGICAL(ans)[i] = NA_LOGICAL;
 		else
@@ -521,7 +509,7 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 		LOGICAL(ans)[i] = 0;
  	    else {
 		errno = 0;
-		res = STRCOLL(translateChar(c1), translateChar(c2));
+		res = Scollate(c1, c2);
 		if(errno)  
 		    LOGICAL(ans)[i] = NA_LOGICAL;
 		else
@@ -539,8 +527,7 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 		LOGICAL(ans)[i] = 1;
  	    else {
 		errno = 0;
-		res = STRCOLL(translateChar(STRING_ELT(s1, i % n1)),
-			      translateChar(STRING_ELT(s2, i % n2)));
+		res = Scollate(STRING_ELT(s1, i % n1), STRING_ELT(s2, i % n2));
 		if(errno)  
 		    LOGICAL(ans)[i] = NA_LOGICAL;
 		else
@@ -558,8 +545,7 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 		LOGICAL(ans)[i] = 1;
   	    else {
 		errno = 0;
-		res = STRCOLL(translateChar(STRING_ELT(s1, i % n1)),
-			      translateChar(STRING_ELT(s2, i % n2)));
+		res = Scollate(STRING_ELT(s1, i % n1), STRING_ELT(s2, i % n2));
 		if(errno)  
 		    LOGICAL(ans)[i] = NA_LOGICAL;
 		else
