@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2007   The R Development Core Team.
+ *  Copyright (C) 1998-2008   The R Development Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -82,17 +82,29 @@ typedef struct {
 
 static SEXP insertString(char *str, LocalData *l)
 {
-    if (!utf8strIsASCII(str)) {
-        if (l->isLatin1) return mkCharEnc(str, LATIN1_MASK);
-        if (l->isUTF8)   return mkCharEnc(str, UTF8_MASK);
+    if (!strIsASCII(str)) {
+        if (l->con->UTF8out || l->isUTF8) return mkCharCE(str, CE_UTF8);
+        else if (l->isLatin1) return mkCharCE(str, CE_LATIN1);
     }
-    return mkCharEnc(str, 0);
+    return mkChar(str);
+}
+
+static R_INLINE Rboolean Rspace(unsigned int c)
+{
+    if (c == ' ' || c == '\t' || c == '\n' || c == '\r') return TRUE;
+#ifdef Win32
+    /* 0xa0 is NBSP in all 8-bit Windows locales */
+    if(!mbcslocale && c == 0xa0) return TRUE;
+#else
+     /* 0xa0 is NBSP in Latin-1 */
+    if(known_to_be_latin1 && c == 0xa0) return TRUE;
+#endif
+    return FALSE;
 }
 
-#define Rspace(c) (c == ' ' || c == '\t' || c == '\n' || c == '\r')
 
 /* used by readline() and menu() */
-static int ConsoleGetchar()
+static int ConsoleGetchar(void)
 {
     if (--ConsoleBufCnt < 0) {
 	ConsoleBuf[CONSOLE_BUFFER_SIZE] = '\0';
@@ -145,53 +157,10 @@ static int Strtoi(const char *nptr, int base)
     return(res);
 }
 
-/* Like R_strtod, but allow NA to be a failure if NA arg is false */
-static double Rs_strtod(const char *c, char **end, Rboolean NA)
-{
-    double x;
-
-    if (NA && strncmp(c, "NA", 2) == 0){
-	x = NA_REAL; *end = (char *)c + 2; /* coercion for -Wall */
-    }
-    else if (strncmp(c, "NaN", 3) == 0) {
-	x = R_NaN; *end = (char *)c + 3;
-    }
-    else if (strncmp(c, "Inf", 3) == 0) {
-	x = R_PosInf; *end = (char *)c + 3;
-    }
-    else if (strncmp(c, "-Inf", 4) == 0) {
-	x = R_NegInf; *end = (char *)c + 4;
-    }
-    else
-        x = strtod(c, end);
-    return x;
-}
-
 static double
 Strtod (const char *nptr, char **endptr, Rboolean NA, LocalData *d)
 {
-    if (d->decchar == '.')
-	return Rs_strtod(nptr, endptr, NA);
-    else {
-	/* jump through some hoops... This is a kludge!
-	   Should most likely use regexps instead */
-
-	char *end;
-	double x;
-	int i;
-
-	strncpy(d->convbuf, nptr, 100);
-	for ( i = 0 ; i < 100 ; i++ )
-	    /* switch '.' and decchar around */
-	    if (d->convbuf[i] == d->decchar)
-		d->convbuf[i] = '.';
-	    else if (d->convbuf[i] == '.')
-		d->convbuf[i] = d->decchar;
-	x = Rs_strtod(d->convbuf, &end, NA);
-	if(endptr)
-  	   *endptr = (char *) nptr + (end - d->convbuf);
-	return x;
-    }
+    return R_strtod4(nptr, endptr, d->decchar, NA);
 }
 
 static Rcomplex
@@ -204,25 +173,22 @@ strtoc(const char *nptr, char **endptr, Rboolean NA, LocalData *d)
     x = Strtod(nptr, &endp, NA, d);
     if (isBlankString(endp)) {
 	z.r = x; z.i = 0;
-    }
-    else if (*endp == 'i')  {
+    } else if (*endp == 'i')  {
 	z.r = 0; z.i = x;
 	endp++;
-    }
-    else {
+    } else {
 	s = endp;
 	y = Strtod(s, &endp, NA, d);
 	if (*endp == 'i') {
 	    z.r = x; z.i = y;
 	    endp++;
-	}
-	else {
+	} else {
 	    z.r = 0; z.i = 0;
 	    endp = (char *) nptr; /* -Wall */
 	}
     }
     *endptr = endp;
-    return(z);
+    return z;
 }
 
 static Rbyte
@@ -426,7 +392,7 @@ fillBuffer(SEXPTYPE type, int strip, int *bch, LocalData *d,
 			}
 		/* CSV style quoted string handling */
 		if ((type == STRSXP || type == NILSXP)
-		    && strchr(d->quoteset, c)) {
+		    && c != 0 && strchr(d->quoteset, c)) {
 		    quote = c;
 		inquote:
 		    while ((c = scanchar(TRUE, d)) != R_EOF && c != quote) {
@@ -704,7 +670,7 @@ static SEXP scanFrame(SEXP what, int maxitems, int maxlines, int flush,
 	w = VECTOR_ELT(what, i);
 	if (!isNull(w)) {
 	    if (!isVector(w)) {
-		error(_("invalid 'what' specified"));
+		error(_("invalid '%s' argument"), "what");
 	    }
 	    if(TYPEOF(w) == STRSXP) nstring++;
 	    SET_VECTOR_ELT(ans, i, allocVector(TYPEOF(w), blksize));
@@ -871,7 +837,7 @@ SEXP attribute_hidden do_scan(SEXP call, SEXP op, SEXP args, SEXP rho)
     comstr = CAR(args);            args = CDR(args);
     escapes = asLogical(CAR(args));args = CDR(args);
     if(!isString(CAR(args)) || LENGTH(CAR(args)) != 1)
-	error(_("invalid '%s' value"), "encoding");
+	error(_("invalid '%s' argument"), "encoding");
     encoding = CHAR(STRING_ELT(CAR(args), 0)); /* ASCII */
     if(streql(encoding, "latin1")) data.isLatin1 = TRUE;
     if(streql(encoding, "UTF-8"))  data.isUTF8 = TRUE;
@@ -884,13 +850,13 @@ SEXP attribute_hidden do_scan(SEXP call, SEXP op, SEXP args, SEXP rho)
     if (nmax < 0 || nmax == NA_INTEGER)		nmax = 0;
 
     if (TYPEOF(stripwhite) != LGLSXP)
-	error(_("invalid '%s' value"), "strip.white");
+	error(_("invalid '%s' argument"), "strip.white");
     if (length(stripwhite) != 1 && length(stripwhite) != length(what))
 	error(_("invalid 'strip.white' length"));
     if (TYPEOF(data.NAstrings) != STRSXP)
-	error(_("invalid '%s' value"), "na.strings");
+	error(_("invalid '%s' argument"), "na.strings");
     if (TYPEOF(comstr) != STRSXP || length(comstr) != 1)
-	error(_("invalid '%s' value"), "comment.char");
+	error(_("invalid '%s' argument"), "comment.char");
 
     if (isString(sep) || isNull(sep)) {
 	if (length(sep) == 0) data.sepchar = 0;
@@ -901,7 +867,7 @@ SEXP attribute_hidden do_scan(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    data.sepchar = (unsigned char) sc[0];
 	}
 	/* gets compared to chars: bug prior to 1.7.0 */
-    } else error(_("invalid '%s' value"), "sep");
+    } else error(_("invalid '%s' argument"), "sep");
 
     if (isString(dec) || isNull(dec)) {
 	if (length(dec) == 0)
@@ -928,10 +894,10 @@ SEXP attribute_hidden do_scan(SEXP call, SEXP op, SEXP args, SEXP rho)
     p = translateChar(STRING_ELT(comstr, 0));
     data.comchar = NO_COMCHAR; /*  here for -Wall */
     if (strlen(p) > 1)
-	error(_("invalid '%s' value"), "comment.char");
+	error(_("invalid '%s' argument"), "comment.char");
     else if (strlen(p) == 1) data.comchar = (unsigned char)*p;
     if(escapes == NA_LOGICAL)
-	error(_("invalid '%s' value"), "allowEscapes");
+	error(_("invalid '%s' argument"), "allowEscapes");
     data.escapes = escapes != 0;
 
     i = asInteger(file);
@@ -942,6 +908,7 @@ SEXP attribute_hidden do_scan(SEXP call, SEXP op, SEXP args, SEXP rho)
 	data.ttyflag = 0;
 	data.wasopen = data.con->isopen;
 	if(!data.wasopen) {
+	    data.con->UTF8out = TRUE;  /* a request */
 	    strcpy(data.con->mode, "r");
 	    if(!data.con->open(data.con))
 		error(_("cannot open the connection"));
@@ -976,7 +943,7 @@ SEXP attribute_hidden do_scan(SEXP call, SEXP op, SEXP args, SEXP rho)
 			blskip, multiline, &data);
 	break;
     default:
-	error(_("invalid 'what' specified"));
+	error(_("invalid '%s' argument"), "what");
     }
     endcontext(&cntxt);
 
@@ -1015,11 +982,11 @@ SEXP attribute_hidden do_countfields(SEXP call, SEXP op, SEXP args, SEXP rho)
     blskip = asLogical(CAR(args)); args = CDR(args);
     comstr = CAR(args);
     if (TYPEOF(comstr) != STRSXP || length(comstr) != 1)
-	error(_("invalid '%s' value"), "comment.char");
+	error(_("invalid '%s' argument"), "comment.char");
     p = translateChar(STRING_ELT(comstr, 0));
     data.comchar = NO_COMCHAR; /*  here for -Wall */
     if (strlen(p) > 1)
-	error(_("invalid '%s' value"), "comment.char");
+	error(_("invalid '%s' argument"), "comment.char");
     else if (strlen(p) == 1) data.comchar = (unsigned char)*p;
 
     if (nskip < 0 || nskip == NA_INTEGER) nskip = 0;
@@ -1029,7 +996,7 @@ SEXP attribute_hidden do_countfields(SEXP call, SEXP op, SEXP args, SEXP rho)
 	if (length(sep) == 0) data.sepchar = 0;
 	else data.sepchar = (unsigned char) translateChar(STRING_ELT(sep, 0))[0];
 	/* gets compared to chars: bug prior to 1.7.0 */
-    } else error(_("invalid '%s' value"), "sep");
+    } else error(_("invalid '%s' argument"), "sep");
 
     if (isString(quotes)) {
 	const char *sc = translateChar(STRING_ELT(quotes, 0));
@@ -1238,7 +1205,7 @@ SEXP attribute_hidden do_typecvt(SEXP call, SEXP op, SEXP args, SEXP env)
 
     data.NAstrings = CADR(args);
     if (TYPEOF(data.NAstrings) != STRSXP)
-	error(_("invalid '%s' value"), "na.strings");
+	error(_("invalid '%s' argument"), "na.strings");
 
     asIs = asLogical(CADDR(args));
     if (asIs == NA_LOGICAL) asIs = 0;
@@ -1514,7 +1481,7 @@ SEXP attribute_hidden do_readtablehead(SEXP call, SEXP op, SEXP args, SEXP rho)
     sep = CAR(args);
 
     if (nlines <= 0 || nlines == NA_INTEGER)
-	error(_("invalid '%s' value"), "nlines");
+	error(_("invalid '%s' argument"), "nlines");
     if (blskip == NA_LOGICAL) blskip = 1;
     if (isString(quotes)) {
 	const char *sc = translateChar(STRING_ELT(quotes, 0));
@@ -1526,17 +1493,17 @@ SEXP attribute_hidden do_readtablehead(SEXP call, SEXP op, SEXP args, SEXP rho)
 	error(_("invalid quote symbol set"));
 
     if (TYPEOF(comstr) != STRSXP || length(comstr) != 1)
-	error(_("invalid '%s' value"), "comment.char");
+	error(_("invalid '%s' argument"), "comment.char");
     p = translateChar(STRING_ELT(comstr, 0));
     data.comchar = NO_COMCHAR; /*  here for -Wall */
     if (strlen(p) > 1)
-	error(_("invalid '%s' value"), "comment.char");
+	error(_("invalid '%s' argument"), "comment.char");
     else if (strlen(p) == 1) data.comchar = (int)*p;
     if (isString(sep) || isNull(sep)) {
 	if (length(sep) == 0) data.sepchar = 0;
 	else data.sepchar = (unsigned char) translateChar(STRING_ELT(sep, 0))[0];
 	/* gets compared to chars: bug prior to 1.7.0 */
-    } else error(_("invalid '%s' value"), "sep");
+    } else error(_("invalid '%s' argument"), "sep");
 
     i = asInteger(file);
     data.con = getConnection(i);
@@ -1764,15 +1731,15 @@ SEXP attribute_hidden do_writetable(SEXP call, SEXP op, SEXP args, SEXP rho)
     quote = CAR(args);		   args = CDR(args);
     qmethod = asLogical(CAR(args));
 
-    if(nr == NA_INTEGER) error(_("invalid '%s' value"), "nr");
-    if(nc == NA_INTEGER) error(_("invalid '%s' value"), "nc");
+    if(nr == NA_INTEGER) error(_("invalid '%s' argument"), "nr");
+    if(nc == NA_INTEGER) error(_("invalid '%s' argument"), "nc");
     if(!isNull(rnames) && !isString(rnames))
-	error(_("invalid '%s' value"), "rnames");
-    if(!isString(sep)) error(_("invalid '%s' value"), "sep");
-    if(!isString(eol)) error(_("invalid '%s' value"), "eol");
-    if(!isString(na)) error(_("invalid '%s' value"), "na");
-    if(!isString(dec)) error(_("invalid '%s' value"), "dec");
-    if(qmethod == NA_LOGICAL) error(_("invalid '%s' value"), "qmethod");
+	error(_("invalid '%s' argument"), "rnames");
+    if(!isString(sep)) error(_("invalid '%s' argument"), "sep");
+    if(!isString(eol)) error(_("invalid '%s' argument"), "eol");
+    if(!isString(na)) error(_("invalid '%s' argument"), "na");
+    if(!isString(dec)) error(_("invalid '%s' argument"), "dec");
+    if(qmethod == NA_LOGICAL) error(_("invalid '%s' argument"), "qmethod");
     csep = translateChar(STRING_ELT(sep, 0));
     ceol = translateChar(STRING_ELT(eol, 0));
     cna = translateChar(STRING_ELT(na, 0));
