@@ -43,7 +43,6 @@ static SEXP cross_colon(SEXP call, SEXP s, SEXP t)
     if (length(s) != length(t))
 	errorcall(call, _("unequal factor lengths"));
     n = length(s);
-    /* <FIXME> arrange to translate these */
     ls = getAttrib(s, R_LevelsSymbol);
     lt = getAttrib(t, R_LevelsSymbol);
     nls = LENGTH(ls);
@@ -63,13 +62,15 @@ static SEXP cross_colon(SEXP call, SEXP s, SEXP t)
     if (!isNull(ls) && !isNull(lt)) {
 	PROTECT(la = allocVector(STRSXP, nls * nlt));
 	k = 0;
+	/* FIXME: possibly UTF-8 version */
 	for (i = 0; i < nls; i++) {
-	    vs = strlen(CHAR(STRING_ELT(ls, i)));
+	    const char *vi = translateChar(STRING_ELT(ls, i));
+	    vs = strlen(vi);
 	    for (j = 0; j < nlt; j++) {
-		vt = strlen(CHAR(STRING_ELT(lt, j)));
+		const char *vj = translateChar(STRING_ELT(lt, j));
+		vt = strlen(vj);
                 cbuf = R_AllocStringBuffer(vs + vt + 1, &cbuff);
-		sprintf(cbuf, "%s:%s",
-			CHAR(STRING_ELT(ls, i)), CHAR(STRING_ELT(lt, j)));
+		sprintf(cbuf, "%s:%s", vi, vj);
                 SET_STRING_ELT(la, k, mkChar(cbuf));
 		k++;
 	    }
