@@ -109,9 +109,9 @@ FILE *R_OpenSiteFile(void)
 static char workspace_name[100] = ".RData";
 
 #else
-static char workspace_name[MAX_PATH] = ".RData";
+static char workspace_name[PATH_MAX] = ".RData";
 
-void set_workspace_name(char *fn)
+void set_workspace_name(const char *fn)
 {
     strcpy(workspace_name, fn);
 }
