@@ -14,9 +14,19 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street Fifth Floor, Boston, MA 02110-1301  USA
- *
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
+ */
+
+/*		Warnings/Errors
+
+    In this file we generally do not make use of the call, as it 
+    will be something like `[<-`(`*tmp`, ...) and that just confuses
+    the user.  The call that is deduced from the context is generally
+    much clearer.
+ */
+
+/*
  *
  *  Subset Mutation for Lists and Vectors
  *
@@ -194,9 +204,14 @@ static int SubassignTypeFix(SEXP *x, SEXP *y, int stretch, int level,
     Rboolean x_is_object = OBJECT(*x);
 
     switch (which) {
-
+    case 1000:	/* logical    <- null       */
+    case 1300:	/* integer    <- null       */
+    case 1400:	/* real	      <- null       */
+    case 1500:	/* complex    <- null       */
+    case 1606:	/* character  <- null       */
     case 1900:  /* vector     <- null       */
     case 2000:  /* expression <- null       */
+    case 2400:	/* raw        <- null       */
 
     case 1010:	/* logical    <- logical    */
     case 1310:	/* integer    <- logical    */
@@ -419,12 +434,12 @@ static SEXP VectorAssign(SEXP call, SEXP x, SEXP s, SEXP y)
     if (isMatrix(s) && isArray(x) &&
 	    (isInteger(s) || isReal(s)) &&
 	    ncols(s) == length(dim)) {
-	s = mat2indsub(dim, s);
+	s = mat2indsub(dim, s, R_NilValue);
     }
     PROTECT(s);
 
     stretch = 1;
-    PROTECT(indx = makeSubscript(x, s, &stretch));
+    PROTECT(indx = makeSubscript(x, s, &stretch, R_NilValue));
     n = length(indx);
     if(length(y) > 1)
 	for(i = 0; i < n; i++)
@@ -436,6 +451,10 @@ static SEXP VectorAssign(SEXP call, SEXP x, SEXP s, SEXP y)
     /* accept elements from the RHS. */
     which = SubassignTypeFix(&x, &y, stretch, 1, call);
     /* = 100 * TYPEOF(x) + TYPEOF(y);*/
+    if (n == 0) {
+	UNPROTECT(2);
+	return x;
+    }
     ny = length(y);
     nx = length(x);
 
@@ -718,6 +737,7 @@ static SEXP MatrixAssign(SEXP call, SEXP x, SEXP s, SEXP y)
 	error(_("number of items to replace is not a multiple of replacement length"));
 
     which = SubassignTypeFix(&x, &y, 0, 1, call);
+    if (n == 0) return x;
 
     PROTECT(x);
 
@@ -945,7 +965,7 @@ static SEXP ArrayAssign(SEXP call, SEXP x, SEXP s, SEXP y)
     int **subs, *indx, *bound, *offset;
     SEXP dims, tmp;
     double ry;
-    char *vmax = vmaxget();
+    void *vmax = vmaxget();
 
     PROTECT(dims = getAttrib(x, R_DimSymbol));
     if (dims == R_NilValue || (k = LENGTH(dims)) != length(s))
@@ -1000,7 +1020,7 @@ static SEXP ArrayAssign(SEXP call, SEXP x, SEXP s, SEXP y)
 
     which = SubassignTypeFix(&x, &y, 0, 1, call);/* = 100 * TYPEOF(x) + TYPEOF(y);*/
 
-    if (ny == 0) {
+    if (n == 0) {
 	UNPROTECT(1);
 	return(x);
     }
@@ -1142,7 +1162,7 @@ static SEXP SimpleListAssign(SEXP call, SEXP x, SEXP s, SEXP y)
     if (length(s) > 1)
 	error(_("invalid number of subscripts to list assign"));
 
-    PROTECT(indx = makeSubscript(x, CAR(s), &stretch));
+    PROTECT(indx = makeSubscript(x, CAR(s), &stretch, R_NilValue));
     n = length(indx);
 
     /* The shallow copy here is so that */
@@ -1198,11 +1218,11 @@ static SEXP listRemove(SEXP x, SEXP s)
 {
     SEXP a, pa, px;
     int i, ii, *ind, ns, nx, stretch=0;
-    char *vmax;
+    void *vmax;
 
     vmax = vmaxget();
     nx = length(x);
-    PROTECT(s = makeSubscript(x, s, &stretch));
+    PROTECT(s = makeSubscript(x, s, &stretch, R_NilValue));
     ns = length(s);
     ind = (int*)R_alloc(nx, sizeof(int));
     for (i = 0; i < nx; i++)
@@ -1235,52 +1255,6 @@ static SEXP listRemove(SEXP x, SEXP s)
     return CDR(a);
 }
 
-/* unused
-static SEXP listAssign1(SEXP call, SEXP x, SEXP subs, SEXP y)
-{
-    SEXP ax, ay, px, py, dims;
-    int i, nsubs, ny;
-
-    nsubs = length(subs);
-    switch (nsubs) {
-    case 0:
-	break;
-    case 1:
-	if (y == R_NilValue)
-	    x = listRemove(x, CAR(subs));
-	else
-	    x = SimpleListAssign(call, x, subs, y);
-	break;
-    default:
-	dims = getAttrib(x, R_DimSymbol);
-	if (dims == R_NilValue || LENGTH(dims) != length(subs))
-	    error(_("incorrect number of subscripts"));
-
-	PROTECT(ax = allocArray(STRSXP, dims));
-	for (px = x, i = 0; px != R_NilValue; px = CDR(px))
-	    SET_STRING_ELT(ax, i++, CAR(px));
-	setAttrib(ax, R_DimNamesSymbol, getAttrib(x, R_DimNamesSymbol));
-	if (isList(y)) {
-	    ny = length(y);
-	    PROTECT(ay = allocVector(STRSXP, ny));
-	    for (py = y, i = 0; py != R_NilValue; py = CDR(py))
-		SET_STRING_ELT(ay, i++, CAR(py));
-	}
-	else {
-	    ny = 1;
-	    PROTECT(ay = allocVector(STRSXP, 1));
-	    SET_STRING_ELT(ay, 0, y);
-	}
-	if (nsubs == 2) ax = MatrixAssign(call, ax, subs, ay);
-	else ax = ArrayAssign(call, ax, subs, ay);
-	for (px = x, i = 0; px != R_NilValue; px = CDR(px))
-	    CAR(px) = duplicate(STRING_ELT(ax, i++));
-	UNPROTECT(2);
-	break;
-    }
-    return x;
-}
-*/
 
 static void SubAssignArgs(SEXP args, SEXP *x, SEXP *s, SEXP *y)
 {
@@ -1334,7 +1308,7 @@ SEXP attribute_hidden do_subassign_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
     /* duplicate it so that only the local version is mutated. */
     /* This will duplicate more often than necessary, but saves */
     /* over always duplicating. */
-    /* FIXME: shouldn't x be protected?  It is (as args is)! */
+    /* Shouldn't x be protected?  It is (as args is)! */
 
     if (NAMED(CAR(args)) == 2)
 	x = SETCAR(args, duplicate(CAR(args)));
@@ -1388,13 +1362,17 @@ SEXP attribute_hidden do_subassign_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 	}
 	break;
     default:
-	error(_("object is not subsettable"));
+	error(_("object of type '%s' is not subsettable"),
+	      type2char(TYPEOF(x)));
 	break;
     }
 
     if (oldtype == LANGSXP) {
-	x = VectorToPairList(x);
-	SET_TYPEOF(x, LANGSXP);
+	if(length(x)) {
+	    x = VectorToPairList(x);
+	    SET_TYPEOF(x, LANGSXP);
+	} else
+	    error(_("result is zero-length and so cannot be a language object"));
     }
 
     /* Note the setting of NAMED(x) to zero here.  This means */
@@ -1453,7 +1431,8 @@ SEXP attribute_hidden do_subassign2(SEXP call, SEXP op, SEXP args, SEXP rho)
     return do_subassign2_dflt(call, op, ans, rho);
 }
 
-SEXP attribute_hidden do_subassign2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden 
+do_subassign2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP dims, indx, names, newname, subs, x, xtop, xup, y;
     int i, ndims, nsubs, offset, off = -1 /* -Wall */, stretch, which;
@@ -1518,7 +1497,7 @@ SEXP attribute_hidden do_subassign2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho
 			error(_("recursive indexing failed at level %d\n"), 
 			      i+1);
 		    off = get1index(CAR(subs), getAttrib(x, R_NamesSymbol),
-				    length(x), /*partial ok*/TRUE, i);
+				    length(x), /*partial ok*/TRUE, i, call);
 		    if(off < 0 || off >= LENGTH(x))
 			error(_("no such index at level %d\n"), i+1);
 		    xup = x;
@@ -1528,7 +1507,7 @@ SEXP attribute_hidden do_subassign2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho
 	    }
 	    if (recursed && !isVectorList(x) && LENGTH(y) > 1)
 		error(_("more elements supplied than there are to replace"));
-	    offset = OneIndex(x, thesub, length(x), 0, &newname, i);
+	    offset = OneIndex(x, thesub, length(x), 0, &newname, i, R_NilValue);
 	    if (isVectorList(x) && isNull(y)) {
 		x = DeleteOneVectorListItem(x, offset);
 		if(recursed) SET_VECTOR_ELT(xup, off, x);
@@ -1550,7 +1529,7 @@ SEXP attribute_hidden do_subassign2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho
 		INTEGER(indx)[i] = get1index(CAR(subs), isNull(names) ?
 					      R_NilValue : VECTOR_ELT(names, i),
 					      INTEGER(dims)[i],
-					      /*partial ok*/FALSE, -1);
+					      /*partial ok*/FALSE, -1, call);
 		subs = CDR(subs);
 		if (INTEGER(indx)[i] < 0 ||
 		    INTEGER(indx)[i] >= INTEGER(dims)[i])
@@ -1731,7 +1710,7 @@ SEXP attribute_hidden do_subassign2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho
 	    for (i = 0; i < ndims; i++) {
 		INTEGER(indx)[i] = get1index(CAR(subs), CAR(names),
 					     INTEGER(dims)[i],
-					     /*partial ok*/FALSE, -1);
+					     /*partial ok*/FALSE, -1, call);
 		subs = CDR(subs);
 		if (INTEGER(indx)[i] < 0 ||
 		    INTEGER(indx)[i] >= INTEGER(dims)[i])
@@ -1747,7 +1726,8 @@ SEXP attribute_hidden do_subassign2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho
 	xtop = x;
 	UNPROTECT(1);
     }
-    else error(_("object is not subsettable"));
+    else error(_("object of type '%s' is not subsettable"),
+	       type2char(TYPEOF(x)));
 
     UNPROTECT(1);
     SET_NAMED(xtop, 0);
@@ -1777,7 +1757,7 @@ SEXP attribute_hidden do_subassign3(SEXP call, SEXP op, SEXP args, SEXP env)
     else if(isString(nlist) )
 	SET_STRING_ELT(input, 0, STRING_ELT(nlist, 0));
     else {
-	error(_("invalid subscript type"));
+	error(_("invalid subscript type '%s'"), type2char(TYPEOF(nlist)));
 	return R_NilValue; /*-Wall*/
     }
 
