@@ -1,7 +1,7 @@
 /*
  *  Mathlib : A C Library of Special Functions
  *  Copyright (C) 1998 Ross Ihaka
- *  Copyright (C) 2000 The R Development Core Team
+ *  Copyright (C) 2000-8 The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,17 +29,17 @@
 #include "nmath.h"
 #include "dpq.h"
 
-double pnbinom(double x, double n, double p, int lower_tail, int log_p)
+double pnbinom(double x, double size, double prob, int lower_tail, int log_p)
 {
 #ifdef IEEE_754
-    if (ISNAN(x) || ISNAN(n) || ISNAN(p))
-	return x + n + p;
-    if(!R_FINITE(n) || !R_FINITE(p))	ML_ERR_return_NAN;
+    if (ISNAN(x) || ISNAN(size) || ISNAN(prob))
+	return x + size + prob;
+    if(!R_FINITE(size) || !R_FINITE(prob))	ML_ERR_return_NAN;
 #endif
-    if (n <= 0 || p <= 0 || p > 1)	ML_ERR_return_NAN;
+    if (size <= 0 || prob <= 0 || prob > 1)	ML_ERR_return_NAN;
 
     if (x < 0) return R_DT_0;
     if (!R_FINITE(x)) return R_DT_1;
     x = floor(x + 1e-7);
-    return pbeta(p, n, x + 1, lower_tail, log_p);
+    return pbeta(prob, size, x + 1, lower_tail, log_p);
 }
