@@ -54,7 +54,8 @@
 
 #ifdef HAVE_DYNAMIC_LOADING
 
-static void *loadLibrary(const char *path, int asLocal, int now);
+static void *loadLibrary(const char *path, int asLocal, int now, 
+			 const char *search);
 static void closeLibrary(void *handle);
 static void deleteCachedSymbols(DllInfo *);
 static DL_FUNC R_local_dlsym(DllInfo *info, char const *name);
@@ -81,7 +82,8 @@ static void getSystemError(char *buf, int len)
     strcpy(buf, dlerror());
 }
 
-static void *loadLibrary(const char *path, int asLocal, int now)
+static void *loadLibrary(const char *path, int asLocal, int now,
+			 const char *search)
 {
     void *handle;
     int openFlag = 0;
