@@ -79,3 +79,19 @@ if(require(MASS)) {
     stats:::drop1.default(hills.glm)
 }
 ## wrong AIC() and drop1 prior to 2.3.0.
+
+## calculating no of signif digits
+print(1.001, digits=16)
+## 2.4.1 gave  1.001000000000000
+## 2.5.0 errs on the side of caution.
+
+
+## iconv to UCS-2
+x <- "A test string"
+y <- try(iconv(x, "", "UCS-2BE"))
+if(!inherits(y, "try-error")) {
+    print(y)
+    z <- iconv(y, "UCS-2BE", "")
+    stopifnot(identical(x, z))
+}
+## works in R >= 2.6.0
